@@ -32,11 +32,14 @@
 
 #include "test/src/helpers.h"
 #include "test/src/vfs_helpers.h"
+#include "tiledb/common/common.h"
+#include "tiledb/common/dynamic_memory/dynamic_memory.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/query/reader.h"
+#include "tiledb/type/range/range.h"
 
 #ifdef _WIN32
 #include "tiledb/sm/filesystem/win.h"
@@ -48,6 +51,7 @@
 #include <iostream>
 
 using namespace tiledb::sm;
+using namespace tiledb::type;
 using namespace tiledb::test;
 
 /* ********************************* */
@@ -107,11 +111,15 @@ TEST_CASE_METHOD(
   std::unordered_map<std::string, tiledb::sm::QueryBuffer> buffers;
   Subarray subarray;
   QueryCondition condition;
+  ThreadPool tp_cpu, tp_io;
+  StorageManager storage_manager(
+      &tp_cpu, &tp_io, &g_helper_stats, g_helper_logger());
+  Array array(URI("my_array"), &storage_manager);
   Reader reader(
       &g_helper_stats,
       g_helper_logger(),
       nullptr,
-      nullptr,
+      &array,
       config,
       buffers,
       subarray,
@@ -168,11 +176,11 @@ TEST_CASE_METHOD(
 
   std::vector<TileDomain<int32_t>> frag_tile_domains;
   frag_tile_domains.emplace_back(
-      TileDomain<int32_t>(2, domain, ds3, tile_extents, layout));
+      TileDomain<int32_t>(3, domain, ds3, tile_extents, layout));
   frag_tile_domains.emplace_back(
-      TileDomain<int32_t>(1, domain, ds2, tile_extents, layout));
+      TileDomain<int32_t>(2, domain, ds2, tile_extents, layout));
   frag_tile_domains.emplace_back(
-      TileDomain<int32_t>(0, domain, ds1, tile_extents, layout));
+      TileDomain<int32_t>(1, domain, ds1, tile_extents, layout));
   TileDomain<int32_t> array_tile_domain(
       UINT32_MAX, domain, dsd, tile_extents, layout);
 
@@ -183,17 +191,19 @@ TEST_CASE_METHOD(
   d2.set_domain(&domain_vec[2]);
   d2.set_tile_extent(&tile_extents_vec[1]);
   Domain dom;
-  CHECK(dom.add_dimension(&d1).ok());
-  CHECK(dom.add_dimension(&d2).ok());
-  ArraySchema schema;
-  CHECK(schema.set_domain(&dom).ok());
+  CHECK(dom.add_dimension(make_shared<Dimension>(HERE(), &d1)).ok());
+  CHECK(dom.add_dimension(make_shared<Dimension>(HERE(), &d2)).ok());
 
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragments;
-  for (uint64_t i = 0; i < frag_tile_domains.size(); i++) {
-    tdb_shared_ptr<FragmentMetadata> fragment = tdb_make_shared(
-        FragmentMetadata,
+  auto schema = make_shared<ArraySchema>(HERE());
+  CHECK(schema->set_domain(make_shared<Domain>(HERE(), &dom)).ok());
+
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  for (uint64_t i = 0; i < frag_tile_domains.size() + 1; i++) {
+    shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+        HERE(),
         nullptr,
-        &schema,
+        nullptr,
+        schema,
         URI(),
         std::make_pair<uint64_t, uint64_t>(0, 0),
         true);
@@ -207,53 +217,49 @@ TEST_CASE_METHOD(
       tile_coords,
       array_tile_domain,
       frag_tile_domains,
-      &result_space_tiles);
+      result_space_tiles);
   CHECK(result_space_tiles.size() == 6);
 
-  // Result tiles for fragment #0
-  ResultTile result_tile_1_0_0(0, 0, &schema);
-  ResultTile result_tile_1_2_0(0, 2, &schema);
-
   // Result tiles for fragment #1
-  ResultTile result_tile_1_0_1(1, 0, &schema);
+  ResultTile result_tile_1_0_1(1, 0, *(schema.get()));
+  ResultTile result_tile_1_2_1(1, 2, *(schema.get()));
 
   // Result tiles for fragment #2
-  ResultTile result_tile_2_0_2(2, 0, &schema);
-  ResultTile result_tile_3_0_2(2, 2, &schema);
+  ResultTile result_tile_1_0_2(2, 0, *(schema.get()));
+
+  // Result tiles for fragment #3
+  ResultTile result_tile_2_0_3(3, 0, *(schema.get()));
+  ResultTile result_tile_3_0_3(3, 2, *(schema.get()));
 
   // Initialize result_space_tiles
   ResultSpaceTile<int32_t> rst_1_0;
   rst_1_0.set_start_coords({3, 1});
-  rst_1_0.append_frag_domain(1, ds2);
-  rst_1_0.append_frag_domain(0, ds1);
-  rst_1_0.set_result_tile(0, result_tile_1_0_0);
+  rst_1_0.append_frag_domain(2, ds2);
+  rst_1_0.append_frag_domain(1, ds1);
   rst_1_0.set_result_tile(1, result_tile_1_0_1);
+  rst_1_0.set_result_tile(2, result_tile_1_0_2);
   ResultSpaceTile<int32_t> rst_1_2;
   rst_1_2.set_start_coords({3, 11});
-  rst_1_2.append_frag_domain(0, ds1);
-  rst_1_2.set_result_tile(0, result_tile_1_2_0);
+  rst_1_2.append_frag_domain(1, ds1);
+  rst_1_2.set_result_tile(1, result_tile_1_2_1);
   ResultSpaceTile<int32_t> rst_2_0;
   rst_2_0.set_start_coords({5, 1});
-  rst_2_0.append_frag_domain(2, ds3);
-  rst_2_0.set_result_tile(2, result_tile_2_0_2);
+  rst_2_0.append_frag_domain(3, ds3);
+  rst_2_0.set_result_tile(3, result_tile_2_0_3);
   ResultSpaceTile<int32_t> rst_2_2;
   rst_2_2.set_start_coords({5, 11});
   ResultSpaceTile<int32_t> rst_3_0;
   rst_3_0.set_start_coords({7, 1});
-  rst_3_0.append_frag_domain(2, ds3);
-  rst_3_0.set_result_tile(2, result_tile_3_0_2);
+  rst_3_0.append_frag_domain(3, ds3);
+  rst_3_0.set_result_tile(3, result_tile_3_0_3);
   ResultSpaceTile<int32_t> rst_3_2;
   rst_3_2.set_start_coords({7, 11});
 
-  // Prepare correct space tiles map
-  std::map<const int32_t*, ResultSpaceTile<int32_t>> c_result_space_tiles;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[0][0])] = rst_1_0;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[1][0])] = rst_1_2;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[2][0])] = rst_2_0;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[3][0])] = rst_2_2;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[4][0])] = rst_3_0;
-  c_result_space_tiles[(const int32_t*)&(tile_coords[5][0])] = rst_3_2;
-
   // Check correctness
-  CHECK(result_space_tiles == c_result_space_tiles);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[0][0])] == rst_1_0);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[1][0])] == rst_1_2);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[2][0])] == rst_2_0);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[3][0])] == rst_2_2);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[4][0])] == rst_3_0);
+  CHECK(result_space_tiles[(const int32_t*)&(tile_coords[5][0])] == rst_3_2);
 }
