@@ -92,8 +92,12 @@ TEST_CASE(
   REQUIRE(err == nullptr);
   tiledb_ctx_t* ctx_array_not_encrypted;
   REQUIRE(tiledb_ctx_alloc(cfg, &ctx_array_not_encrypted) == TILEDB_OK);
+  rc = tiledb_fragment_info_set_config(ctx, fragment_info, cfg);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_fragment_info_load(ctx_array_not_encrypted, fragment_info);
   CHECK(rc == TILEDB_ERR);
+  tiledb_fragment_info_free(&fragment_info);
+
   tiledb_config_free(&cfg);
   tiledb_ctx_free(&ctx_array_not_encrypted);
 
@@ -106,6 +110,10 @@ TEST_CASE(
   write_array(ctx, array_name, 1, subarray, TILEDB_ROW_MAJOR, buffers);
 
   // Load again
+  // We need to allocate the fragment info again with `ctx`, which does not
+  // contain a key.
+  rc = tiledb_fragment_info_alloc(ctx, array_name.c_str(), &fragment_info);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
 
@@ -248,6 +256,11 @@ TEST_CASE(
   CHECK(rc == TILEDB_OK);
   CHECK(std::string(uri) == written_frag_uri);
 
+  // Get fragment name
+  const char* name;
+  rc = tiledb_fragment_info_get_fragment_name(ctx, fragment_info, 1, &name);
+  CHECK(rc == TILEDB_OK);
+
   // Get schema name
   const char* schema_name;
   rc = tiledb_fragment_info_get_array_schema_name(
@@ -264,7 +277,7 @@ TEST_CASE(
   uint64_t size;
   rc = tiledb_fragment_info_get_fragment_size(ctx, fragment_info, 1, &size);
   CHECK(rc == TILEDB_OK);
-  CHECK(size == 1662);
+  CHECK(size == 3084);
 
   // Get dense / sparse
   int32_t dense;
@@ -402,6 +415,8 @@ TEST_CASE(
   REQUIRE(err == nullptr);
   tiledb_ctx_t* ctx_correct_key;
   REQUIRE(tiledb_ctx_alloc(cfg, &ctx_correct_key) == TILEDB_OK);
+  rc = tiledb_fragment_info_set_config(ctx, fragment_info, cfg);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_fragment_info_load(ctx_correct_key, fragment_info);
   CHECK(rc == TILEDB_OK);
   tiledb_config_free(&cfg);
@@ -490,11 +505,16 @@ TEST_CASE(
   CHECK(rc == TILEDB_OK);
   CHECK(std::string(uri) == written_frag_uri);
 
+  // Get fragment name
+  const char* name;
+  rc = tiledb_fragment_info_get_fragment_name(ctx, fragment_info, 1, &name);
+  CHECK(rc == TILEDB_OK);
+
   // Get fragment size
   uint64_t size;
   rc = tiledb_fragment_info_get_fragment_size(ctx, fragment_info, 1, &size);
   CHECK(rc == TILEDB_OK);
-  CHECK(size == 2855);
+  CHECK(size == 5382);
 
   // Get dense / sparse
   int32_t dense;
@@ -706,6 +726,8 @@ TEST_CASE("C API: Test MBR fragment info", "[capi][fragment_info][mbr]") {
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(err == nullptr);
   REQUIRE(tiledb_ctx_alloc(cfg, &ctx) == TILEDB_OK);
+  rc = tiledb_fragment_info_set_config(ctx, fragment_info, cfg);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_fragment_info_load(ctx, fragment_info);
   CHECK(rc == TILEDB_OK);
   tiledb_config_free(&cfg);
@@ -752,7 +774,7 @@ TEST_CASE("C API: Test MBR fragment info", "[capi][fragment_info][mbr]") {
 
 TEST_CASE(
     "C API: Test fragment info, load from array with string dimension",
-    "[capi][fragment_info][load][string-dim][mbr]") {
+    "[capi][fragment_info][load][string-dims][mbr]") {
   // Create TileDB context
   tiledb_ctx_t* ctx = nullptr;
   int rc = tiledb_ctx_alloc(nullptr, &ctx);
@@ -761,6 +783,8 @@ TEST_CASE(
   rc = tiledb_vfs_alloc(ctx, nullptr, &vfs);
   REQUIRE(rc == TILEDB_OK);
 
+  remove_dir(array_name, ctx, vfs);
+
   // Create array
   create_array(
       ctx,
@@ -1321,22 +1345,24 @@ TEST_CASE("C API: Test fragment info, dump", "[capi][fragment_info][dump]") {
   CHECK(tiledb_vfs_remove_file(ctx, vfs, "frag3_schema.txt") == TILEDB_OK);
 
   // Check dump
+  const auto ver = std::to_string(tiledb::sm::constants::format_version);
   std::string dump_str =
       std::string("- Fragment num: 3\n") +
       "- Unconsolidated metadata num: 3\n" + "- To vacuum num: 0\n" +
       "- Fragment #1:\n" + "  > URI: " + written_frag_uri_1 + "\n" +
       "  > Type: dense\n" + "  > Non-empty domain: [1, 6]\n" +
-      "  > Size: 1662\n" + "  > Cell num: 10\n" +
-      "  > Timestamp range: [1, 1]\n" + "  > Format version: 10\n" +
+      "  > Size: 3084\n" + "  > Cell num: 10\n" +
+      "  > Timestamp range: [1, 1]\n" + "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n" + "- Fragment #2:\n" +
       "  > URI: " + written_frag_uri_2 + "\n" + "  > Type: dense\n" +
-      "  > Non-empty domain: [1, 4]\n" + "  > Size: 1619\n" +
+      "  > Non-empty domain: [1, 4]\n" + "  > Size: 3033\n" +
       "  > Cell num: 5\n" + "  > Timestamp range: [2, 2]\n" +
-      "  > Format version: 10\n" + "  > Has consolidated metadata: no\n" +
-      "- Fragment #3:\n" + "  > URI: " + written_frag_uri_3 + "\n" +
-      "  > Type: dense\n" + "  > Non-empty domain: [5, 6]\n" +
-      "  > Size: 1662\n" + "  > Cell num: 10\n" +
-      "  > Timestamp range: [3, 3]\n" + "  > Format version: 10\n" +
+      "  > Format version: " + ver + "\n" +
+      "  > Has consolidated metadata: no\n" + "- Fragment #3:\n" +
+      "  > URI: " + written_frag_uri_3 + "\n" + "  > Type: dense\n" +
+      "  > Non-empty domain: [5, 6]\n" + "  > Size: 3081\n" +
+      "  > Cell num: 10\n" + "  > Timestamp range: [3, 3]\n" +
+      "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n";
   FILE* gold_fout = fopen("gold_fout.txt", "w");
   const char* dump = dump_str.c_str();
@@ -1458,15 +1484,17 @@ TEST_CASE(
   CHECK(rc == TILEDB_OK);
 
   // Check dump
+  const auto ver = std::to_string(tiledb::sm::constants::format_version);
   std::string dump_str =
       std::string("- Fragment num: 1\n") +
       "- Unconsolidated metadata num: 1\n" + "- To vacuum num: 3\n" +
       "- To vacuum URIs:\n" + "  > " + written_frag_uri_1 + "\n  > " +
       written_frag_uri_2 + "\n  > " + written_frag_uri_3 + "\n" +
       "- Fragment #1:\n" + "  > URI: " + uri + "\n" + "  > Type: dense\n" +
-      "  > Non-empty domain: [1, 10]\n" + "  > Size: 1662\n" +
+      "  > Non-empty domain: [1, 10]\n" + "  > Size: 3099\n" +
       "  > Cell num: 10\n" + "  > Timestamp range: [1, 3]\n" +
-      "  > Format version: 10\n" + "  > Has consolidated metadata: no\n";
+      "  > Format version: " + ver + "\n" +
+      "  > Has consolidated metadata: no\n";
   FILE* gold_fout = fopen("gold_fout.txt", "w");
   const char* dump = dump_str.c_str();
   fwrite(dump, sizeof(char), strlen(dump), gold_fout);
@@ -1491,7 +1519,7 @@ TEST_CASE(
 
 TEST_CASE(
     "C API: Test fragment info, dump with string dimension",
-    "[capi][fragment_info][dump][string-dim]") {
+    "[capi][fragment_info][dump][string-dims]") {
   // Create TileDB context
   tiledb_ctx_t* ctx = nullptr;
   int rc = tiledb_ctx_alloc(nullptr, &ctx);
@@ -1541,13 +1569,14 @@ TEST_CASE(
   CHECK(rc == TILEDB_OK);
 
   // Check dump
+  const auto ver = std::to_string(tiledb::sm::constants::format_version);
   std::string dump_str =
       std::string("- Fragment num: 1\n") +
       "- Unconsolidated metadata num: 1\n" + "- To vacuum num: 0\n" +
       "- Fragment #1:\n" + "  > URI: " + written_frag_uri + "\n" +
       "  > Type: sparse\n" + "  > Non-empty domain: [a, ddd]\n" +
-      "  > Size: 1903\n" + "  > Cell num: 4\n" +
-      "  > Timestamp range: [1, 1]\n" + "  > Format version: 10\n" +
+      "  > Size: 3330\n" + "  > Cell num: 4\n" +
+      "  > Timestamp range: [1, 1]\n" + "  > Format version: " + ver + "\n" +
       "  > Has consolidated metadata: no\n";
   FILE* gold_fout = fopen("gold_fout.txt", "w");
   const char* dump = dump_str.c_str();
@@ -1659,4 +1688,183 @@ TEST_CASE(
   remove_dir(array_name, ctx, vfs);
   tiledb_ctx_free(&ctx);
   tiledb_vfs_free(&vfs);
+}
+
+TEST_CASE(
+    "C API: Test fragment info, consolidated fragment metadata multiple",
+    "[capi][fragment_info][consolidated-metadata][multiple]") {
+  // Create TileDB context
+  tiledb_ctx_t* ctx = nullptr;
+  int rc = tiledb_ctx_alloc(nullptr, &ctx);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_vfs_t* vfs = nullptr;
+  rc = tiledb_vfs_alloc(ctx, nullptr, &vfs);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Create array
+  uint64_t domain[] = {1, 10};
+  uint64_t tile_extent = 5;
+  create_array(
+      ctx,
+      array_name,
+      TILEDB_DENSE,
+      {"d"},
+      {TILEDB_UINT64},
+      {domain},
+      {&tile_extent},
+      {"a"},
+      {TILEDB_INT32},
+      {1},
+      {tiledb::test::Compressor(TILEDB_FILTER_NONE, -1)},
+      TILEDB_ROW_MAJOR,
+      TILEDB_ROW_MAJOR,
+      2);
+
+  // Write a dense fragment
+  QueryBuffers buffers;
+  uint64_t subarray[] = {1, 6};
+  std::vector<int32_t> a = {1, 2, 3, 4, 5, 6};
+  uint64_t a_size = a.size() * sizeof(int32_t);
+  buffers["a"] = tiledb::test::QueryBuffer({&a[0], a_size, nullptr, 0});
+  std::string written_frag_uri;
+  write_array(
+      ctx,
+      array_name,
+      1,
+      subarray,
+      TILEDB_ROW_MAJOR,
+      buffers,
+      &written_frag_uri);
+
+  // Write another dense fragment
+  subarray[0] = 1;
+  subarray[1] = 7;
+  a = {7, 1, 2, 3, 4, 5, 6};
+  a_size = a.size() * sizeof(int32_t);
+  buffers["a"] = tiledb::test::QueryBuffer({&a[0], a_size, nullptr, 0});
+  write_array(ctx, array_name, 3, subarray, TILEDB_ROW_MAJOR, buffers);
+
+  // Create fragment info object
+  tiledb_fragment_info_t* fragment_info = nullptr;
+  rc = tiledb_fragment_info_alloc(ctx, array_name.c_str(), &fragment_info);
+  CHECK(rc == TILEDB_OK);
+
+  // Load fragment info
+  rc = tiledb_fragment_info_load(ctx, fragment_info);
+  CHECK(rc == TILEDB_OK);
+
+  // Check for consolidated metadata
+  int32_t has;
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 0, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 0);
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 1, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 0);
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 2, &has);
+  CHECK(rc == TILEDB_ERR);
+
+  // Get number of unconsolidated fragment metadata
+  uint32_t unconsolidated = 0;
+  rc = tiledb_fragment_info_get_unconsolidated_metadata_num(
+      ctx, fragment_info, &unconsolidated);
+  CHECK(rc == TILEDB_OK);
+  CHECK(unconsolidated == 2);
+
+  // Consolidate fragment metadata
+  tiledb_config_t* config = nullptr;
+  tiledb_error_t* error = nullptr;
+  REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.consolidation.mode", "fragment_meta", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+
+  // Consolidate - this will consolidate only the fragment metadata
+  rc = tiledb_array_consolidate(ctx, array_name.c_str(), config);
+  CHECK(rc == TILEDB_OK);
+
+  // Load fragment info
+  rc = tiledb_fragment_info_load(ctx, fragment_info);
+  CHECK(rc == TILEDB_OK);
+
+  // Check for consolidated metadata
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 0, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 1);
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 1, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 1);
+
+  // Get number of unconsolidated fragment metadata
+  rc = tiledb_fragment_info_get_unconsolidated_metadata_num(
+      ctx, fragment_info, &unconsolidated);
+  CHECK(rc == TILEDB_OK);
+  CHECK(unconsolidated == 0);
+
+  // Write another dense fragment in between the existing 2
+  subarray[0] = 2;
+  subarray[1] = 9;
+  a = {6, 7, 1, 2, 3, 4, 5, 6};
+  a_size = a.size() * sizeof(int32_t);
+  buffers["a"] = tiledb::test::QueryBuffer({&a[0], a_size, nullptr, 0});
+  write_array(ctx, array_name, 2, subarray, TILEDB_ROW_MAJOR, buffers);
+
+  // Load fragment info
+  rc = tiledb_fragment_info_load(ctx, fragment_info);
+  CHECK(rc == TILEDB_OK);
+
+  // Check for consolidated metadata
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 0, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 1);
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 1, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 0);
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 2, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 1);
+
+  // Get number of unconsolidated fragment metadata
+  rc = tiledb_fragment_info_get_unconsolidated_metadata_num(
+      ctx, fragment_info, &unconsolidated);
+  CHECK(rc == TILEDB_OK);
+  CHECK(unconsolidated == 1);
+
+  // Consolidate - this will consolidate only the fragment metadata
+  rc = tiledb_array_consolidate(ctx, array_name.c_str(), config);
+  CHECK(rc == TILEDB_OK);
+
+  // Load fragment info
+  rc = tiledb_fragment_info_load(ctx, fragment_info);
+  CHECK(rc == TILEDB_OK);
+
+  // Check again
+  rc = tiledb_fragment_info_has_consolidated_metadata(
+      ctx, fragment_info, 1, &has);
+  CHECK(rc == TILEDB_OK);
+  CHECK(has == 1);
+
+  // Get number of unconsolidated fragment metadata
+  rc = tiledb_fragment_info_get_unconsolidated_metadata_num(
+      ctx, fragment_info, &unconsolidated);
+  CHECK(rc == TILEDB_OK);
+  CHECK(unconsolidated == 0);
+
+  // Clean up
+  tiledb_fragment_info_free(&fragment_info);
+  remove_dir(array_name, ctx, vfs);
+  tiledb_ctx_free(&ctx);
+  tiledb_vfs_free(&vfs);
+  tiledb_error_free(&error);
+  tiledb_config_free(&config);
 }
\ No newline at end of file
