@@ -32,12 +32,13 @@
 
 #include "catch.hpp"
 #include "test/src/helpers.h"
+#include "tiledb/common/common.h"
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/query/reader.h"
-#include "tiledb/sm/query/writer.h"
+#include "tiledb/sm/query/writer_base.h"
 #include "tiledb/sm/serialization/query.h"
 
 #ifdef _WIN32
@@ -78,8 +79,8 @@ template <class TResult, class TExpected>
 bool check_result(
     const TResult a,
     const TExpected b,
-    std::optional<size_t> start = nullopt,
-    std::optional<size_t> end = nullopt) {
+    optional<size_t> start = nullopt,
+    optional<size_t> end = nullopt) {
   TResult b_typed;
   if constexpr (std::is_same<TExpected, std::any>::value) {
     b_typed = std::any_cast<TResult>(b);
@@ -119,7 +120,7 @@ struct SerializationFx {
   }
 
   static void check_read_stats(const Query& query) {
-    auto stats = ((sm::Writer*)query.ptr()->query_->strategy())->stats();
+    auto stats = ((sm::WriterBase*)query.ptr()->query_->strategy())->stats();
     REQUIRE(stats != nullptr);
     auto counters = stats->counters();
     REQUIRE(counters != nullptr);
@@ -391,6 +392,7 @@ struct SerializationFx {
       bool clientside) {
     // Serialize
     tiledb_buffer_list_t* buff_list;
+
     ctx.handle_error(tiledb_serialize_query(
         ctx.ptr().get(),
         query.ptr().get(),
@@ -662,14 +664,13 @@ TEST_CASE_METHOD(
       REQUIRE(std::get<1>(result_el["a3"]) == 110);
 
       auto null_val = std::numeric_limits<uint32_t>::max();
-      auto null_val_char = std::numeric_limits<int8_t>::min();
       for (uint64_t i = 5; i < 100; i++) {
         REQUIRE(a1[i] == null_val);
         REQUIRE(a2[i * 2] == null_val);
         REQUIRE(a2[i * 2 + 1] == null_val);
         REQUIRE(a2_nullable[i] == 0);
         REQUIRE(a3_offsets[i] == 10 + i);
-        REQUIRE(a3_data[10 + i] == null_val_char);
+        REQUIRE(a3_data[10 + i] == 0);
       }
     } else {
       REQUIRE(std::get<1>(result_el["a1"]) == 5);
