@@ -33,10 +33,13 @@
 #ifdef HAVE_GCS
 
 #include "catch.hpp"
+#include "tiledb/common/filesystem/directory_entry.h"
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/filesystem/gcs.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
+#include "tiledb/sm/misc/tdb_math.h"
+#include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/utils.h"
 
 #include <sstream>
@@ -105,6 +108,20 @@ std::string GCSFx::random_bucket_name(const std::string& prefix) {
   return ss.str();
 }
 
+TEST_CASE_METHOD(GCSFx, "Test GCS init", "[gcs]") {
+  try {
+    Config config;
+    config.set("vfs.gcs.use_multi_part_upload", "true");
+    init_gcs(std::move(config));
+  } catch (...) {
+    INFO(
+        "GCS initialization failed. In order to run GCS tests, be sure to "
+        "source scripts/run-gcs.sh in this shell session before starting test "
+        "runner.");
+    REQUIRE(false);
+  }
+}
+
 TEST_CASE_METHOD(GCSFx, "Test GCS filesystem, file management", "[gcs]") {
   Config config;
   config.set("vfs.gcs.use_multi_part_upload", "true");
@@ -184,6 +201,23 @@ TEST_CASE_METHOD(GCSFx, "Test GCS filesystem, file management", "[gcs]") {
   REQUIRE(gcs_.is_dir(URI(TEST_DIR + "dir"), &is_dir).ok());
   REQUIRE(is_dir);  // This is viewed as a dir
 
+  // ls_with_sizes
+  std::string s = "abcdef";
+  CHECK(gcs_.write(URI(file3), s.data(), s.size()).ok());
+  REQUIRE(gcs_.flush_object(URI(file3)).ok());
+
+  auto&& [status, rv] = gcs_.ls_with_sizes(URI(dir));
+  auto children = *rv;
+  REQUIRE(status.ok());
+
+  REQUIRE(children.size() == 2);
+  CHECK(children[0].path().native() == file3);
+  CHECK(children[1].path().native() == subdir.substr(0, subdir.size() - 1));
+
+  CHECK(children[0].file_size() == s.size());
+  // Directories don't get a size
+  CHECK(children[1].file_size() == 0);
+
   // Move file
   REQUIRE(gcs_.move_object(URI(file5), URI(file6)).ok());
   REQUIRE(gcs_.is_object(URI(file5), &is_object).ok());
