@@ -30,6 +30,7 @@
  * Tests the `RTree` class.
  */
 
+#include "tiledb/common/common.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/layout.h"
@@ -87,16 +88,32 @@ Domain create_domain(
   assert(dim_names.size() == dim_domains.size());
   assert(dim_names.size() == dim_tile_extents.size());
 
-  Domain domain;
+  std::vector<shared_ptr<Dimension>> dimensions;
   for (size_t d = 0; d < dim_names.size(); ++d) {
-    Dimension dim(dim_names[d], dim_types[d]);
-    dim.set_domain(dim_domains[d]);
-    dim.set_tile_extent(dim_tile_extents[d]);
-    domain.add_dimension(&dim);
+    uint32_t cell_val_num =
+        (datatype_is_string(dim_types[d])) ? constants::var_num : 1;
+    Range range;
+    if (dim_domains[d] != nullptr) {
+      range = Range(dim_domains[d], 2 * datatype_size(dim_types[d]));
+    }
+    ByteVecValue tile_extent;
+    if (dim_tile_extents[d] != nullptr) {
+      auto tile_extent_size = 2 * datatype_size(dim_types[d]);
+      tile_extent.resize(tile_extent_size);
+      std::memcpy(tile_extent.data(), dim_tile_extents[d], tile_extent_size);
+    }
+    shared_ptr<Dimension> dim = make_shared<Dimension>(
+        HERE(),
+        dim_names[d],
+        dim_types[d],
+        cell_val_num,
+        range,
+        FilterPipeline(),
+        tile_extent);
+    dimensions.emplace_back(std::move(dim));
   }
-  domain.init(Layout::ROW_MAJOR, Layout::ROW_MAJOR);
 
-  return domain;
+  return Domain(Layout::ROW_MAJOR, dimensions, Layout::ROW_MAJOR);
 }
 
 TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
@@ -107,13 +124,16 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
   CHECK(rtree0.domain() == nullptr);
   CHECK(rtree0.fanout() == 0);
 
+  std::vector<bool> is_default(2, false);
+
   // 1D
   int32_t dim_dom[] = {1, 1000};
   int32_t dim_extent = 10;
   Domain dom1 =
       create_domain({"d"}, {Datatype::INT32}, {dim_dom}, {&dim_extent});
   std::vector<NDRange> mbrs_1d = create_mbrs<int32_t, 1>({1, 3, 5, 10, 20, 22});
-  RTree rtree1(&dom1, 3);
+  const Domain d1{&dom1};
+  RTree rtree1(&d1, 3);
   CHECK(!rtree1.set_leaf(0, mbrs_1d[0]).ok());
   rtree1.set_leaf_num(mbrs_1d.size());
   for (size_t m = 0; m < mbrs_1d.size(); ++m)
@@ -142,25 +162,25 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
   int32_t r1_right[] = {7, 11};
   int32_t r1_no_right[] = {11, 15};
   range1[0].set_range(r1_no_left, 2 * sizeof(int32_t));
-  double ratio1 = dom1.overlap_ratio(range1, mbr1);
+  double ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 0.0);
   range1[0].set_range(r1_left, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 3.0 / 6);
   range1[0].set_range(r1_exact, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 1.0);
   range1[0].set_range(r1_full, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 1.0);
   range1[0].set_range(r1_contained, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 2.0 / 6);
   range1[0].set_range(r1_right, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 4.0 / 6);
   range1[0].set_range(r1_no_right, 2 * sizeof(int32_t));
-  ratio1 = dom1.overlap_ratio(range1, mbr1);
+  ratio1 = dom1.overlap_ratio(range1, is_default, mbr1);
   CHECK(ratio1 == 0.0);
 
   // 2D
@@ -173,7 +193,8 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
       {&dim_extent_2, &dim_extent_2});
   std::vector<NDRange> mbrs_2d =
       create_mbrs<int64_t, 2>({1, 3, 5, 10, 20, 22, 24, 25, 11, 15, 30, 31});
-  RTree rtree2(&dom2, 5);
+  const Domain d2{&dom2};
+  RTree rtree2(&d2, 5);
   rtree2.set_leaves(mbrs_2d);
   rtree2.build_tree();
   CHECK(rtree2.height() == 2);
@@ -193,15 +214,15 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
   int64_t r2_partial[] = {7, 11, 4, 5};
   range2[0].set_range(&r2_no[0], 2 * sizeof(int64_t));
   range2[1].set_range(&r2_no[2], 2 * sizeof(int64_t));
-  double ratio2 = dom2.overlap_ratio(range2, mbr2);
+  double ratio2 = dom2.overlap_ratio(range2, is_default, mbr2);
   CHECK(ratio2 == 0.0);
   range2[0].set_range(&r2_full[0], 2 * sizeof(int64_t));
   range2[1].set_range(&r2_full[2], 2 * sizeof(int64_t));
-  ratio2 = dom2.overlap_ratio(range2, mbr2);
+  ratio2 = dom2.overlap_ratio(range2, is_default, mbr2);
   CHECK(ratio2 == 1.0);
   range2[0].set_range(&r2_partial[0], 2 * sizeof(int64_t));
   range2[1].set_range(&r2_partial[2], 2 * sizeof(int64_t));
-  ratio2 = dom2.overlap_ratio(range2, mbr2);
+  ratio2 = dom2.overlap_ratio(range2, is_default, mbr2);
   CHECK(ratio2 == (4.0 / 6) * (2.0 / 8));
 
   // Float datatype
@@ -211,7 +232,8 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
       create_mbrs<float, 1>({1.0f, 3.0f, 5.0f, 10.0f, 20.0f, 22.0f});
   Domain dom2f =
       create_domain({"d"}, {Datatype::FLOAT32}, {dim_dom_f}, {&dim_extent_f});
-  RTree rtreef(&dom2f, 5);
+  const Domain d2f{&dom2f};
+  RTree rtreef(&d2f, 5);
   rtreef.set_leaves(mbrs_f);
   rtreef.build_tree();
 
@@ -226,33 +248,35 @@ TEST_CASE("RTree: Test R-Tree, basic functions", "[rtree][basic]") {
   float rf_right[] = {7.0, 11.0};
   float rf_no_right[] = {11.0, 15.0};
   rangef[0].set_range(rf_no_left, 2 * sizeof(float));
-  double ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  double ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 0.0);
   rangef[0].set_range(rf_left, 2 * sizeof(float));
-  ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 2.0 / 5);
   rangef[0].set_range(rf_exact, 2 * sizeof(float));
-  ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 1.0);
   rangef[0].set_range(rf_full, 2 * sizeof(float));
-  ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 1.0);
   rangef[0].set_range(rf_right, 2 * sizeof(float));
-  ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 3.0 / 5);
   rangef[0].set_range(rf_no_right, 2 * sizeof(float));
-  ratiof = dom2f.overlap_ratio(rangef, mbrf);
+  ratiof = dom2f.overlap_ratio(rangef, is_default, mbrf);
   CHECK(ratiof == 0.0);
 }
 
 TEST_CASE("RTree: Test 1D R-tree, height 2", "[rtree][1d][2h]") {
   // Build tree
+  std::vector<bool> is_default(1, false);
   int32_t dim_dom[] = {1, 1000};
   int32_t dim_extent = 10;
   Domain dom1 =
       create_domain({"d"}, {Datatype::INT32}, {dim_dom}, {&dim_extent});
   std::vector<NDRange> mbrs = create_mbrs<int32_t, 1>({1, 3, 5, 10, 20, 22});
-  RTree rtree(&dom1, 3);
+  const Domain d1{&dom1};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -270,17 +294,17 @@ TEST_CASE("RTree: Test 1D R-tree, height 2", "[rtree][1d][2h]") {
   int32_t r_full[] = {0, 22};
   int32_t r_partial[] = {6, 21};
   range[0].set_range(r_no, 2 * sizeof(int32_t));
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
   range[0].set_range(r_full, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
   CHECK(overlap.tile_ranges_[0].second == 2);
   range[0].set_range(r_partial, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 1);
@@ -291,13 +315,15 @@ TEST_CASE("RTree: Test 1D R-tree, height 2", "[rtree][1d][2h]") {
 
 TEST_CASE("RTree: Test 1D R-tree, height 3", "[rtree][1d][3h]") {
   // Build tree
+  std::vector<bool> is_default(1, false);
   int32_t dim_dom[] = {1, 1000};
   int32_t dim_extent = 10;
   std::vector<NDRange> mbrs = create_mbrs<int32_t, 1>(
       {1, 3, 5, 10, 20, 22, 30, 35, 36, 38, 40, 49, 50, 51, 65, 69});
   Domain dom1 =
       create_domain({"d"}, {Datatype::INT32}, {dim_dom}, {&dim_extent});
-  RTree rtree(&dom1, 3);
+  const Domain d1(&dom1);
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 3);
@@ -318,17 +344,17 @@ TEST_CASE("RTree: Test 1D R-tree, height 3", "[rtree][1d][3h]") {
   int32_t r_only_ranges[] = {30, 69};
   int32_t r_tiles_and_ranges[] = {1, 32};
   range[0].set_range(r_no, 2 * sizeof(int32_t));
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
   range[0].set_range(r_full, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
   CHECK(overlap.tile_ranges_[0].second == 7);
   range[0].set_range(r_only_tiles, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 1);
@@ -336,7 +362,7 @@ TEST_CASE("RTree: Test 1D R-tree, height 3", "[rtree][1d][3h]") {
   CHECK(overlap.tiles_[1].first == 2);
   CHECK(overlap.tiles_[1].second == 1.0 / 3);
   range[0].set_range(r_only_ranges, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 2);
   CHECK(overlap.tile_ranges_[0].first == 3);
@@ -344,7 +370,7 @@ TEST_CASE("RTree: Test 1D R-tree, height 3", "[rtree][1d][3h]") {
   CHECK(overlap.tile_ranges_[1].first == 6);
   CHECK(overlap.tile_ranges_[1].second == 7);
   range[0].set_range(r_tiles_and_ranges, 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
   CHECK(overlap.tile_ranges_[0].second == 2);
@@ -355,6 +381,7 @@ TEST_CASE("RTree: Test 1D R-tree, height 3", "[rtree][1d][3h]") {
 
 TEST_CASE("RTree: Test 2D R-tree, height 2", "[rtree][2d][2h]") {
   // Build tree
+  std::vector<bool> is_default(2, false);
   int32_t dim_dom[] = {1, 1000};
   int32_t dim_extent = 10;
   Domain dom2 = create_domain(
@@ -364,7 +391,8 @@ TEST_CASE("RTree: Test 2D R-tree, height 2", "[rtree][2d][2h]") {
       {&dim_extent, &dim_extent});
   std::vector<NDRange> mbrs =
       create_mbrs<int32_t, 2>({1, 3, 2, 4, 5, 7, 6, 9, 10, 12, 10, 15});
-  RTree rtree(&dom2, 3);
+  const Domain d2{&dom2};
+  RTree rtree(&d2, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -383,19 +411,19 @@ TEST_CASE("RTree: Test 2D R-tree, height 2", "[rtree][2d][2h]") {
   int32_t r_partial[] = {5, 12, 8, 12};
   range[0].set_range(&r_no[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_no[2], 2 * sizeof(int32_t));
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
   range[0].set_range(&r_full[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_full[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
   CHECK(overlap.tile_ranges_[0].second == 2);
   range[0].set_range(&r_partial[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_partial[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 1);
@@ -406,6 +434,7 @@ TEST_CASE("RTree: Test 2D R-tree, height 2", "[rtree][2d][2h]") {
 
 TEST_CASE("RTree: Test 2D R-tree, height 3", "[rtree][2d][3h]") {
   // Build tree
+  std::vector<bool> is_default(2, false);
   int32_t dim_dom[] = {1, 1000};
   int32_t dim_extent = 10;
   Domain dom2 = create_domain(
@@ -416,7 +445,8 @@ TEST_CASE("RTree: Test 2D R-tree, height 3", "[rtree][2d][3h]") {
   std::vector<NDRange> mbrs = create_mbrs<int32_t, 2>(
       {1,  3,  2,  4,  5,  7,  6,  9,  10, 12, 10, 15, 11, 15, 20, 22, 16, 16,
        23, 23, 19, 20, 24, 26, 25, 28, 30, 32, 30, 35, 35, 37, 40, 42, 40, 42});
-  RTree rtree(&dom2, 3);
+  const Domain d2{&dom2};
+  RTree rtree(&d2, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 3);
@@ -438,19 +468,19 @@ TEST_CASE("RTree: Test 2D R-tree, height 3", "[rtree][2d][3h]") {
   int32_t r_tiles_and_ranges[] = {19, 50, 25, 50};
   range[0].set_range(&r_no[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_no[2], 2 * sizeof(int32_t));
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
   range[0].set_range(&r_full[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_full[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
   CHECK(overlap.tile_ranges_[0].second == 8);
   range[0].set_range(&r_only_tiles[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_only_tiles[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 2);
@@ -459,7 +489,7 @@ TEST_CASE("RTree: Test 2D R-tree, height 3", "[rtree][2d][3h]") {
   CHECK(overlap.tiles_[1].second == (4.0 / 5) * (2.0 / 3));
   range[0].set_range(&r_only_ranges[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_only_ranges[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 2);
   CHECK(overlap.tile_ranges_[0].first == 3);
@@ -468,7 +498,7 @@ TEST_CASE("RTree: Test 2D R-tree, height 3", "[rtree][2d][3h]") {
   CHECK(overlap.tile_ranges_[1].second == 8);
   range[0].set_range(&r_tiles_and_ranges[0], 2 * sizeof(int32_t));
   range[1].set_range(&r_tiles_and_ranges[2], 2 * sizeof(int32_t));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 6);
   CHECK(overlap.tile_ranges_[0].second == 8);
@@ -481,6 +511,7 @@ TEST_CASE(
     "RTree: Test R-Tree, heterogeneous (uint8, int32), basic functions",
     "[rtree][basic][heter]") {
   // Create RTree with dimensions uint8, int32
+  std::vector<bool> is_default(2, false);
   uint8_t uint8_dom[] = {0, 10};
   int32_t int32_dom[] = {5, 10};
   uint8_t uint8_extent = 2;
@@ -492,7 +523,8 @@ TEST_CASE(
       {&uint8_extent, &int32_extent});
   std::vector<NDRange> mbrs =
       create_mbrs<uint8_t, int32_t>({0, 1, 3, 5}, {5, 6, 7, 9});
-  RTree rtree(&dom, 5);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 5);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -507,7 +539,7 @@ TEST_CASE(
   int32_t int32_r_no[] = {1, 10};
   range_no[0].set_range(uint8_r_no, sizeof(uint8_r_no));
   range_no[1].set_range(int32_r_no, sizeof(int32_r_no));
-  double ratio = dom.overlap_ratio(range_no, mbrs[0]);
+  double ratio = dom.overlap_ratio(range_no, is_default, mbrs[0]);
   CHECK(ratio == 0.0);
 
   // Check full domain overlap
@@ -516,9 +548,9 @@ TEST_CASE(
   int32_t int32_r_full[] = {1, 10};
   range_full[0].set_range(uint8_r_full, sizeof(uint8_r_full));
   range_full[1].set_range(int32_r_full, sizeof(int32_r_full));
-  ratio = dom.overlap_ratio(range_full, mbrs[0]);
+  ratio = dom.overlap_ratio(range_full, is_default, mbrs[0]);
   CHECK(ratio == 1.0);
-  ratio = dom.overlap_ratio(range_full, mbrs[1]);
+  ratio = dom.overlap_ratio(range_full, is_default, mbrs[1]);
   CHECK(ratio == 1.0);
 
   // Check partial domain overlap
@@ -527,7 +559,7 @@ TEST_CASE(
   int32_t int32_r_part[] = {5, 5};
   range_part[0].set_range(uint8_r_part, sizeof(uint8_r_part));
   range_part[1].set_range(int32_r_part, sizeof(int32_r_part));
-  ratio = dom.overlap_ratio(range_part, mbrs[0]);
+  ratio = dom.overlap_ratio(range_part, is_default, mbrs[0]);
   CHECK(ratio == 0.25);
 }
 
@@ -535,6 +567,7 @@ TEST_CASE(
     "RTree: Test R-Tree, heterogeneous (uint64, float32), basic functions",
     "[rtree][basic][heter]") {
   // Create RTree with dimensions uint64, float32
+  std::vector<bool> is_default(2, false);
   uint64_t uint64_dom[] = {0, 10};
   float float_dom[] = {0.1f, 0.9f};
   uint64_t uint64_extent = 2;
@@ -546,7 +579,8 @@ TEST_CASE(
       {&uint64_extent, &float_extent});
   std::vector<NDRange> mbrs =
       create_mbrs<uint64_t, float>({0, 1, 3, 5}, {.5f, .6f, .7f, .9f});
-  RTree rtree(&dom, 5);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 5);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -561,7 +595,7 @@ TEST_CASE(
   float float_r_no[] = {.1f, .9f};
   range_no[0].set_range(uint64_r_no, sizeof(uint64_r_no));
   range_no[1].set_range(float_r_no, sizeof(float_r_no));
-  double ratio = dom.overlap_ratio(range_no, mbrs[0]);
+  double ratio = dom.overlap_ratio(range_no, is_default, mbrs[0]);
   CHECK(ratio == 0.0);
 
   // Check full domain overlap
@@ -570,9 +604,9 @@ TEST_CASE(
   float float_r_full[] = {.1f, 1.0f};
   range_full[0].set_range(uint64_r_full, sizeof(uint64_r_full));
   range_full[1].set_range(float_r_full, sizeof(float_r_full));
-  ratio = dom.overlap_ratio(range_full, mbrs[0]);
+  ratio = dom.overlap_ratio(range_full, is_default, mbrs[0]);
   CHECK(ratio == 1.0);
-  ratio = dom.overlap_ratio(range_full, mbrs[1]);
+  ratio = dom.overlap_ratio(range_full, is_default, mbrs[1]);
   CHECK(ratio == 1.0);
 
   // Check partial domain overlap
@@ -581,7 +615,7 @@ TEST_CASE(
   float float_r_part[] = {.5f, .55f};
   range_part[0].set_range(uint64_r_part, sizeof(uint64_r_part));
   range_part[1].set_range(float_r_part, sizeof(float_r_part));
-  ratio = dom.overlap_ratio(range_part, mbrs[0]);
+  ratio = dom.overlap_ratio(range_part, is_default, mbrs[0]);
   CHECK(ratio == 0.25);
 }
 
@@ -589,6 +623,7 @@ TEST_CASE(
     "RTree: Test 2D R-tree, height 2, heterogeneous (uint8, int32)",
     "[rtree][2d][2h][heter]") {
   // Create RTree with dimensions uint8, int32
+  std::vector<bool> is_default(2, false);
   uint8_t uint8_dom[] = {0, 200};
   int32_t int32_dom[] = {5, 100};
   uint8_t uint8_extent = 2;
@@ -600,7 +635,8 @@ TEST_CASE(
       {&uint8_extent, &int32_extent});
   std::vector<NDRange> mbrs =
       create_mbrs<uint8_t, int32_t>({0, 1, 3, 5, 11, 20}, {5, 6, 7, 9, 11, 30});
-  RTree rtree(&dom, 3);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -621,7 +657,7 @@ TEST_CASE(
   int32_t int32_r_no[] = {1, 10};
   range_no[0].set_range(uint8_r_no, sizeof(uint8_r_no));
   range_no[1].set_range(int32_r_no, sizeof(int32_r_no));
-  auto overlap = rtree.get_tile_overlap(range_no);
+  auto overlap = rtree.get_tile_overlap(range_no, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -631,7 +667,7 @@ TEST_CASE(
   int32_t int32_r_full[] = {1, 100};
   range_full[0].set_range(uint8_r_full, sizeof(uint8_r_full));
   range_full[1].set_range(int32_r_full, sizeof(int32_r_full));
-  overlap = rtree.get_tile_overlap(range_full);
+  overlap = rtree.get_tile_overlap(range_full, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -643,7 +679,7 @@ TEST_CASE(
   int32_t int32_r_part[] = {7, 20};
   range_part[0].set_range(uint8_r_part, sizeof(uint8_r_part));
   range_part[1].set_range(int32_r_part, sizeof(int32_r_part));
-  overlap = rtree.get_tile_overlap(range_part);
+  overlap = rtree.get_tile_overlap(range_part, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 1);
@@ -656,6 +692,7 @@ TEST_CASE(
     "RTree: Test 2D R-tree, height 3, heterogeneous (uint8, int32)",
     "[rtree][2d][2h][heter]") {
   // Create RTree with dimensions uint8, int32
+  std::vector<bool> is_default(2, false);
   uint8_t uint8_dom[] = {0, 200};
   int32_t int32_dom[] = {5, 100};
   uint8_t uint8_extent = 2;
@@ -667,7 +704,8 @@ TEST_CASE(
       {&uint8_extent, &int32_extent});
   std::vector<NDRange> mbrs = create_mbrs<uint8_t, int32_t>(
       {0, 1, 3, 5, 11, 20, 21, 26}, {5, 6, 7, 9, 11, 30, 31, 40});
-  RTree rtree(&dom, 2);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 2);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 3);
@@ -690,7 +728,7 @@ TEST_CASE(
   int32_t int32_r_no[] = {1, 10};
   range_no[0].set_range(uint8_r_no, sizeof(uint8_r_no));
   range_no[1].set_range(int32_r_no, sizeof(int32_r_no));
-  auto overlap = rtree.get_tile_overlap(range_no);
+  auto overlap = rtree.get_tile_overlap(range_no, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -700,7 +738,7 @@ TEST_CASE(
   int32_t int32_r_full[] = {1, 100};
   range_full[0].set_range(uint8_r_full, sizeof(uint8_r_full));
   range_full[1].set_range(int32_r_full, sizeof(int32_r_full));
-  overlap = rtree.get_tile_overlap(range_full);
+  overlap = rtree.get_tile_overlap(range_full, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -712,7 +750,7 @@ TEST_CASE(
   int32_t int32_r_part[] = {7, 20};
   range_part[0].set_range(uint8_r_part, sizeof(uint8_r_part));
   range_part[1].set_range(int32_r_part, sizeof(int32_r_part));
-  overlap = rtree.get_tile_overlap(range_part);
+  overlap = rtree.get_tile_overlap(range_part, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 1);
@@ -726,7 +764,7 @@ TEST_CASE(
   int32_t int32_r_ranges[] = {11, 40};
   range_ranges[0].set_range(uint8_r_ranges, sizeof(uint8_r_ranges));
   range_ranges[1].set_range(int32_r_ranges, sizeof(int32_r_ranges));
-  overlap = rtree.get_tile_overlap(range_ranges);
+  overlap = rtree.get_tile_overlap(range_ranges, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 2);
@@ -738,7 +776,7 @@ TEST_CASE(
   int32_t int32_r_mixed[] = {8, 40};
   range_mixed[0].set_range(uint8_r_mixed, sizeof(uint8_r_mixed));
   range_mixed[1].set_range(int32_r_mixed, sizeof(int32_r_mixed));
-  overlap = rtree.get_tile_overlap(range_mixed);
+  overlap = rtree.get_tile_overlap(range_mixed, is_default);
   CHECK(overlap.tiles_.size() == 1);
   CHECK(overlap.tiles_[0].first == 1);
   CHECK(overlap.tiles_[0].second == (2.0 / 3) * (2.0 / 3));
@@ -800,12 +838,14 @@ TEST_CASE(
     "RTree: Test 1D R-tree, string dims, height 2",
     "[rtree][1d][string-dims][2h]") {
   // Build tree
+  std::vector<bool> is_default(1, false);
   Domain dom1 =
       create_domain({"d"}, {Datatype::STRING_ASCII}, {nullptr}, {nullptr});
   std::vector<NDRange> mbrs =
       create_str_mbrs<1>({"aa", "b", "eee", "g", "gggg", "ii"});
 
-  RTree rtree(&dom1, 3);
+  const Domain d1{&dom1};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -823,7 +863,7 @@ TEST_CASE(
   std::string r_no_end = "dd";
   range[0].set_range_var(
       r_no_start.data(), r_no_start.size(), r_no_end.data(), r_no_end.size());
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -835,7 +875,7 @@ TEST_CASE(
       r_full_start.size(),
       r_full_end.data(),
       r_full_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -849,7 +889,7 @@ TEST_CASE(
       r_partial_start.size(),
       r_partial_end.data(),
       r_partial_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 0);
@@ -865,7 +905,7 @@ TEST_CASE(
       r_partial_start.size(),
       r_partial_end.data(),
       r_partial_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 1);
   CHECK(overlap.tiles_[0].first == 1);
@@ -876,6 +916,7 @@ TEST_CASE(
     "RTree: Test 1D R-tree, string dims, height 3",
     "[rtree][1d][string-dims][3h]") {
   // Build tree
+  std::vector<bool> is_default(1, false);
   Domain dom1 =
       create_domain({"d"}, {Datatype::STRING_ASCII}, {nullptr}, {nullptr});
   std::vector<NDRange> mbrs = create_str_mbrs<1>({"aa",
@@ -891,7 +932,8 @@ TEST_CASE(
                                                   "oo",
                                                   "oop"});
 
-  RTree rtree(&dom1, 3);
+  const Domain d1{&dom1};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 3);
@@ -910,7 +952,7 @@ TEST_CASE(
   std::string r_no_end = "dd";
   range[0].set_range_var(
       r_no_start.data(), r_no_start.size(), r_no_end.data(), r_no_end.size());
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -922,7 +964,7 @@ TEST_CASE(
       r_full_start.size(),
       r_full_end.data(),
       r_full_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -936,7 +978,7 @@ TEST_CASE(
       r_partial_start.size(),
       r_partial_end.data(),
       r_partial_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 0);
@@ -952,7 +994,7 @@ TEST_CASE(
       r_partial_start.size(),
       r_partial_end.data(),
       r_partial_end.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 3);
   CHECK(overlap.tile_ranges_[0].second == 5);
@@ -965,6 +1007,7 @@ TEST_CASE(
     "RTree: Test 2D R-tree, string dims, height 2",
     "[rtree][2d][string-dims][2h]") {
   // Build tree
+  std::vector<bool> is_default(2, false);
   Domain dom = create_domain(
       {"d1", "d2"},
       {Datatype::STRING_ASCII, Datatype::STRING_ASCII},
@@ -983,7 +1026,8 @@ TEST_CASE(
                                                   "oo",
                                                   "qqq"});
 
-  RTree rtree(&dom, 3);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -1003,7 +1047,7 @@ TEST_CASE(
       r_no_start.data(), r_no_start.size(), r_no_end.data(), r_no_end.size());
   range[1].set_range_var(
       r_no_start.data(), r_no_start.size(), r_no_end.data(), r_no_end.size());
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -1022,7 +1066,7 @@ TEST_CASE(
       r_full_start_2.size(),
       r_full_end_2.data(),
       r_full_end_2.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -1043,7 +1087,7 @@ TEST_CASE(
       r_partial_start_2.size(),
       r_partial_end_2.data(),
       r_partial_end_2.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 1);
   CHECK(overlap.tiles_[0].first == 1);
@@ -1064,7 +1108,7 @@ TEST_CASE(
       r_partial_start_2.size(),
       r_partial_end_2.data(),
       r_partial_end_2.size());
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 2);
   CHECK(overlap.tiles_[0].first == 0);
@@ -1077,6 +1121,7 @@ TEST_CASE(
     "RTree: Test 2D R-tree (string, int), height 2",
     "[rtree][2d][string-dims][heter][2h]") {
   // Build tree
+  std::vector<bool> is_default(2, false);
   int32_t dom_int32[] = {1, 20};
   int32_t tile_extent = 5;
   Domain dom = create_domain(
@@ -1087,7 +1132,8 @@ TEST_CASE(
   std::vector<NDRange> mbrs = create_str_int32_mbrs(
       {"aa", "b", "eee", "g", "gggg", "ii"}, {1, 5, 7, 8, 10, 14});
 
-  RTree rtree(&dom, 3);
+  const Domain d1{&dom};
+  RTree rtree(&d1, 3);
   rtree.set_leaves(mbrs);
   rtree.build_tree();
   CHECK(rtree.height() == 2);
@@ -1107,7 +1153,7 @@ TEST_CASE(
       r_no_start.data(), r_no_start.size(), r_no_end.data(), r_no_end.size());
   int32_t r_no[] = {1, 20};
   range[1].set_range(r_no, sizeof(r_no));
-  auto overlap = rtree.get_tile_overlap(range);
+  auto overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.empty());
 
@@ -1121,7 +1167,7 @@ TEST_CASE(
       r_full_end_1.size());
   int32_t r_full[] = {1, 20};
   range[1].set_range(r_full, sizeof(r_full));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tiles_.empty());
   CHECK(overlap.tile_ranges_.size() == 1);
   CHECK(overlap.tile_ranges_[0].first == 0);
@@ -1137,7 +1183,7 @@ TEST_CASE(
       r_partial_end_1.size());
   int32_t r_partial[] = {11, 12};
   range[1].set_range(r_partial, sizeof(r_partial));
-  overlap = rtree.get_tile_overlap(range);
+  overlap = rtree.get_tile_overlap(range, is_default);
   CHECK(overlap.tile_ranges_.empty());
   CHECK(overlap.tiles_.size() == 1);
   CHECK(overlap.tiles_[0].first == 2);
