@@ -31,7 +31,9 @@
  */
 
 #include "test/src/helpers.h"
+#include "tiledb/common/common.h"
 #include "tiledb/sm/c_api/tiledb.h"
+#include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/sparse_index_reader_base.h"
 #include "tiledb/sm/query/sparse_unordered_with_dups_reader.h"
@@ -57,7 +59,6 @@ struct CSparseUnorderedWithDupsFx {
   std::string temp_dir_;
   std::string array_name_;
   const char* ARRAY_NAME = "test_sparse_unordered_with_dups";
-  tiledb_array_t* array_ = nullptr;
   std::string total_budget_;
   std::string ratio_tile_ranges_;
   std::string ratio_array_data_;
@@ -65,8 +66,10 @@ struct CSparseUnorderedWithDupsFx {
   std::string ratio_query_condition_;
 
   void create_default_array_1d();
+  void create_default_array_1d_string();
   void write_1d_fragment(
       int* coords, uint64_t* coords_size, int* data, uint64_t* data_size);
+  void write_1d_fragment_empty_strings(int* coords, uint64_t* coords_size);
   int32_t read(
       bool set_subarray,
       bool set_qc,
@@ -76,6 +79,13 @@ struct CSparseUnorderedWithDupsFx {
       uint64_t* data_size,
       tiledb_query_t** query = nullptr,
       tiledb_array_t** array_ret = nullptr);
+  int32_t read_strings(
+      int* coords,
+      uint64_t* coords_size,
+      char* data,
+      uint64_t* data_size,
+      uint64_t* data_offsets,
+      uint64_t* data_offsets_size);
   void reset_config();
   void update_config();
 
@@ -97,7 +107,6 @@ CSparseUnorderedWithDupsFx::CSparseUnorderedWithDupsFx() {
 }
 
 CSparseUnorderedWithDupsFx::~CSparseUnorderedWithDupsFx() {
-  tiledb_array_free(&array_);
   remove_dir(temp_dir_, ctx_, vfs_);
   tiledb_ctx_free(&ctx_);
   tiledb_vfs_free(&vfs_);
@@ -197,6 +206,27 @@ void CSparseUnorderedWithDupsFx::create_default_array_1d() {
       true);  // allows dups.
 }
 
+void CSparseUnorderedWithDupsFx::create_default_array_1d_string() {
+  int domain[] = {1, 20};
+  int tile_extent = 2;
+  create_array(
+      ctx_,
+      array_name_,
+      TILEDB_SPARSE,
+      {"d"},
+      {TILEDB_INT32},
+      {domain},
+      {&tile_extent},
+      {"a"},
+      {TILEDB_STRING_ASCII},
+      {TILEDB_VAR_NUM},
+      {tiledb::test::Compressor(TILEDB_FILTER_NONE, -1)},
+      TILEDB_ROW_MAJOR,
+      TILEDB_ROW_MAJOR,
+      2,
+      true);  // allows dups.
+}
+
 void CSparseUnorderedWithDupsFx::write_1d_fragment(
     int* coords, uint64_t* coords_size, int* data, uint64_t* data_size) {
   // Open array for writing.
@@ -230,6 +260,49 @@ void CSparseUnorderedWithDupsFx::write_1d_fragment(
   tiledb_query_free(&query);
 }
 
+void CSparseUnorderedWithDupsFx::write_1d_fragment_empty_strings(
+    int* coords, uint64_t* coords_size) {
+  const uint64_t num_cells = *coords_size / sizeof(TILEDB_INT32);
+
+  // Open array for writing.
+  tiledb_array_t* array;
+  auto rc = tiledb_array_alloc(ctx_, array_name_.c_str(), &array);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
+  REQUIRE(rc == TILEDB_OK);
+
+  char data[1];
+  uint64_t data_size = 0;
+  std::vector<uint64_t> data_offsets(num_cells, 0);
+  uint64_t data_offsets_size = num_cells * sizeof(uint64_t);
+
+  // Create the query.
+  tiledb_query_t* query;
+  rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_layout(ctx_, query, TILEDB_UNORDERED);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(ctx_, query, "a", data, &data_size);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_offsets_buffer(
+      ctx_, query, "a", data_offsets.data(), &data_offsets_size);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(ctx_, query, "d", coords, coords_size);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Submit query.
+  rc = tiledb_query_submit(ctx_, query);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Close array.
+  rc = tiledb_array_close(ctx_, array);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Clean up.
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+}
+
 int32_t CSparseUnorderedWithDupsFx::read(
     bool set_subarray,
     bool set_qc,
@@ -297,18 +370,60 @@ int32_t CSparseUnorderedWithDupsFx::read(
   return ret;
 }
 
+int32_t CSparseUnorderedWithDupsFx::read_strings(
+    int* coords,
+    uint64_t* coords_size,
+    char* data,
+    uint64_t* data_size,
+    uint64_t* data_offsets,
+    uint64_t* data_offsets_size) {
+  // Open array for reading.
+  tiledb_array_t* array;
+  auto rc = tiledb_array_alloc(ctx_, array_name_.c_str(), &array);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, array, TILEDB_READ);
+  CHECK(rc == TILEDB_OK);
+
+  // Create query.
+  tiledb_query_t* query;
+  rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
+  CHECK(rc == TILEDB_OK);
+
+  rc = tiledb_query_set_layout(ctx_, query, TILEDB_UNORDERED);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(ctx_, query, "a", data, data_size);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_offsets_buffer(
+      ctx_, query, "a", data_offsets, data_offsets_size);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(ctx_, query, "d", coords, coords_size);
+  CHECK(rc == TILEDB_OK);
+
+  // Submit query.
+  auto ret = tiledb_query_submit(ctx_, query);
+
+  // Clean up.
+  rc = tiledb_array_close(ctx_, array);
+  CHECK(rc == TILEDB_OK);
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+  return ret;
+}
+
 struct CSparseUnorderedWithDupsVarDataFx {
   tiledb_ctx_t* ctx_ = nullptr;
   tiledb_vfs_t* vfs_ = nullptr;
   std::string temp_dir_;
   std::string array_name_;
   const char* ARRAY_NAME = "test_sparse_unordered_with_dups_var_data";
-  tiledb_array_t* array_ = nullptr;
 
   void create_default_array_2d();
   void write_2d_fragment();
   void read_and_check_data(bool set_subarray);
 
+  tuple<tiledb_array_t*, std::vector<shared_ptr<FragmentMetadata>>>
+  open_default_array_1d_with_fragments();
+
   CSparseUnorderedWithDupsVarDataFx();
   ~CSparseUnorderedWithDupsVarDataFx();
 };
@@ -334,7 +449,6 @@ CSparseUnorderedWithDupsVarDataFx::CSparseUnorderedWithDupsVarDataFx() {
 }
 
 CSparseUnorderedWithDupsVarDataFx::~CSparseUnorderedWithDupsVarDataFx() {
-  tiledb_array_free(&array_);
   remove_dir(temp_dir_, ctx_, vfs_);
   tiledb_ctx_free(&ctx_);
   tiledb_vfs_free(&vfs_);
@@ -485,6 +599,48 @@ void CSparseUnorderedWithDupsVarDataFx::read_and_check_data(bool set_subarray) {
   tiledb_query_free(&query);
 }
 
+tuple<tiledb_array_t*, std::vector<shared_ptr<FragmentMetadata>>>
+CSparseUnorderedWithDupsVarDataFx::open_default_array_1d_with_fragments() {
+  int64_t domain[] = {1, 10};
+  int64_t tile_extent = 5;
+  // Create array
+  create_array(
+      ctx_,
+      array_name_,
+      TILEDB_SPARSE,
+      {"d"},
+      {TILEDB_INT64},
+      {domain},
+      {&tile_extent},
+      {"a"},
+      {TILEDB_STRING_ASCII},
+      {1},
+      {tiledb::test::Compressor(TILEDB_FILTER_NONE, -1)},
+      TILEDB_ROW_MAJOR,
+      TILEDB_ROW_MAJOR,
+      5);
+
+  // Open array for writing.
+  tiledb_array_t* array;
+  auto rc = tiledb_array_alloc(ctx_, array_name_.c_str(), &array);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, array, TILEDB_READ);
+  REQUIRE(rc == TILEDB_OK);
+
+  std::vector<shared_ptr<FragmentMetadata>> fragments;
+  shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
+      HERE(),
+      nullptr,
+      nullptr,
+      array->array_->array_schema_latest_ptr(),
+      URI(),
+      std::make_pair<uint64_t, uint64_t>(0, 0),
+      true);
+  fragments.emplace_back(std::move(fragment));
+
+  return {array, std::move(fragments)};
+}
+
 /* ********************************* */
 /*                TESTS              */
 /* ********************************* */
@@ -622,80 +778,46 @@ TEST_CASE_METHOD(
   tiledb_array_t* array = nullptr;
   tiledb_query_t* query = nullptr;
 
-  uint32_t rc;
-  uint64_t coords_r_size;
-  uint64_t data_r_size;
-  for (int i = 0; i < num_frags; i++) {
-    // Try to read.
-    int coords_r[5];
-    int data_r[5];
-    coords_r_size = sizeof(coords_r);
-    data_r_size = sizeof(data_r);
+  // Try to read.
+  int coords_r[10];
+  int data_r[10];
+  uint64_t coords_r_size = sizeof(coords_r);
+  uint64_t data_r_size = sizeof(data_r);
 
-    if (i == 0) {
-      rc = read(
-          use_subarray,
-          false,
-          coords_r,
-          &coords_r_size,
-          data_r,
-          &data_r_size,
-          &query,
-          &array);
-    } else {
-      rc = tiledb_query_submit(ctx_, query);
-    }
-    CHECK(rc == TILEDB_OK);
+  auto rc = read(
+      use_subarray,
+      false,
+      coords_r,
+      &coords_r_size,
+      data_r,
+      &data_r_size,
+      &query,
+      &array);
+  CHECK(rc == TILEDB_OK);
 
-    // Check incomplete query status.
-    tiledb_query_status_t status;
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(status == TILEDB_INCOMPLETE);
+  // Check the internal loop count against expected value.
+  auto stats =
+      ((SparseUnorderedWithDupsReader<uint8_t>*)query->query_->strategy())
+          ->stats();
+  REQUIRE(stats != nullptr);
+  auto counters = stats->counters();
+  REQUIRE(counters != nullptr);
+  auto loop_num =
+      counters->find("Context.StorageManager.Query.Reader.loop_num");
+  CHECK(uint64_t(num_frags * 3) == loop_num->second);
 
-    // Should only read one tile (2 values).
-    CHECK(8 == data_r_size);
-    CHECK(8 == coords_r_size);
+  // Check query status.
+  tiledb_query_status_t status;
+  tiledb_query_get_status(ctx_, query, &status);
+  CHECK(status == TILEDB_COMPLETED);
 
-    int coords_c[] = {1 + i * 5, 2 + i * 5};
-    int data_c[] = {1 + i * 5, 2 + i * 5};
-    CHECK(!std::memcmp(coords_c, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c, data_r, data_r_size));
+  CHECK(uint64_t(num_frags * 20) == data_r_size);
+  CHECK(uint64_t(num_frags * 20) == coords_r_size);
 
-    // Read again.
-    rc = tiledb_query_submit(ctx_, query);
-    CHECK(rc == TILEDB_OK);
-
-    // Check incomplete query status.
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(status == TILEDB_INCOMPLETE);
-
-    // Should only read one more tile (2 values).
-    CHECK(8 == data_r_size);
-    CHECK(8 == coords_r_size);
-
-    int coords_c_2[] = {3 + i * 5, 4 + i * 5};
-    int data_c_2[] = {3 + i * 5, 4 + i * 5};
-    CHECK(!std::memcmp(coords_c_2, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c_2, data_r, data_r_size));
-
-    // Read again.
-    rc = tiledb_query_submit(ctx_, query);
-    CHECK(rc == TILEDB_OK);
-
-    // Check complete query status.
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(
-        status == (i == num_frags - 1 ? TILEDB_COMPLETED : TILEDB_INCOMPLETE));
-
-    // Should read last tile (1 value).
-    CHECK(4 == data_r_size);
-    CHECK(4 == coords_r_size);
-
-    int coords_c_3[] = {5 + i * 5};
-    int data_c_3[] = {5 + i * 5};
-    CHECK(!std::memcmp(coords_c_3, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c_3, data_r, data_r_size));
-  }
+  int coords_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  int data_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  CHECK(!std::memcmp(coords_c, coords_r, coords_r_size));
+  CHECK(!std::memcmp(data_c, data_r, data_r_size));
 
   // Clean up.
   rc = tiledb_array_close(ctx_, array);
@@ -850,80 +972,46 @@ TEST_CASE_METHOD(
   tiledb_array_t* array = nullptr;
   tiledb_query_t* query = nullptr;
 
-  uint32_t rc;
-  uint64_t coords_r_size;
-  uint64_t data_r_size;
-  for (int i = 0; i < num_frags; i++) {
-    // Try to read.
-    int coords_r[5];
-    int data_r[5];
-    coords_r_size = sizeof(coords_r);
-    data_r_size = sizeof(data_r);
+  // Try to read.
+  int coords_r[10];
+  int data_r[10];
+  uint64_t coords_r_size = sizeof(coords_r);
+  uint64_t data_r_size = sizeof(data_r);
 
-    if (i == 0) {
-      rc = read(
-          use_subarray,
-          true,
-          coords_r,
-          &coords_r_size,
-          data_r,
-          &data_r_size,
-          &query,
-          &array);
-    } else {
-      rc = tiledb_query_submit(ctx_, query);
-    }
-    CHECK(rc == TILEDB_OK);
+  auto rc = read(
+      use_subarray,
+      true,
+      coords_r,
+      &coords_r_size,
+      data_r,
+      &data_r_size,
+      &query,
+      &array);
+  CHECK(rc == TILEDB_OK);
 
-    // Check incomplete query status.
-    tiledb_query_status_t status;
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(status == TILEDB_INCOMPLETE);
+  // Check the internal loop count against expected value.
+  auto stats =
+      ((SparseUnorderedWithDupsReader<uint8_t>*)query->query_->strategy())
+          ->stats();
+  REQUIRE(stats != nullptr);
+  auto counters = stats->counters();
+  REQUIRE(counters != nullptr);
+  auto loop_num =
+      counters->find("Context.StorageManager.Query.Reader.loop_num");
+  CHECK(uint64_t(num_frags * 3) == loop_num->second);
 
-    // Should only read one tile (2 values).
-    CHECK(8 == data_r_size);
-    CHECK(8 == coords_r_size);
+  // Check query status.
+  tiledb_query_status_t status;
+  tiledb_query_get_status(ctx_, query, &status);
+  CHECK(status == TILEDB_COMPLETED);
 
-    int coords_c[] = {1 + i * 5, 2 + i * 5};
-    int data_c[] = {1 + i * 5, 2 + i * 5};
-    CHECK(!std::memcmp(coords_c, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c, data_r, data_r_size));
+  CHECK(uint64_t(num_frags * 20) == data_r_size);
+  CHECK(uint64_t(num_frags * 20) == coords_r_size);
 
-    // Read again.
-    rc = tiledb_query_submit(ctx_, query);
-    CHECK(rc == TILEDB_OK);
-
-    // Check incomplete query status.
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(status == TILEDB_INCOMPLETE);
-
-    // Should only read one more tile (2 values).
-    CHECK(8 == data_r_size);
-    CHECK(8 == coords_r_size);
-
-    int coords_c_2[] = {3 + i * 5, 4 + i * 5};
-    int data_c_2[] = {3 + i * 5, 4 + i * 5};
-    CHECK(!std::memcmp(coords_c_2, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c_2, data_r, data_r_size));
-
-    // Read again.
-    rc = tiledb_query_submit(ctx_, query);
-    CHECK(rc == TILEDB_OK);
-
-    // Check complete query status.
-    tiledb_query_get_status(ctx_, query, &status);
-    CHECK(
-        status == (i == num_frags - 1 ? TILEDB_COMPLETED : TILEDB_INCOMPLETE));
-
-    // Should read last tile (1 value).
-    CHECK(4 == data_r_size);
-    CHECK(4 == coords_r_size);
-
-    int coords_c_3[] = {5 + i * 5};
-    int data_c_3[] = {5 + i * 5};
-    CHECK(!std::memcmp(coords_c_3, coords_r, coords_r_size));
-    CHECK(!std::memcmp(data_c_3, data_r, data_r_size));
-  }
+  int coords_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  int data_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  CHECK(!std::memcmp(coords_c, coords_r, coords_r_size));
+  CHECK(!std::memcmp(data_c, data_r, data_r_size));
 
   // Clean up.
   rc = tiledb_array_close(ctx_, array);
@@ -1208,32 +1296,17 @@ TEST_CASE_METHOD(
   read_and_check_data(use_subarray);
 }
 
-/** Make a tile that will return num_cells when cell_num() is called. */
-ResultTileWithBitmap<uint64_t> make_tile(uint64_t num_cells) {
-  Domain domain;
-  Dimension dim("d", Datatype::UINT8);
-  uint8_t bounds[2] = {1, 10};
-  Range range(bounds, 2 * sizeof(uint8_t));
-  REQUIRE(dim.set_domain(range).ok());
-  REQUIRE(domain.add_dimension(&dim).ok());
-
-  ArraySchema schema;
-  REQUIRE(schema.set_domain(&domain).ok());
-  ResultTileWithBitmap<uint64_t> result_tile(0, 0, &schema);
-  result_tile.init_coord_tile("a", 0);
-  auto tuple = result_tile.tile_tuple("a");
-  REQUIRE(std::get<0>(*tuple)
-              .init_unfiltered(0, Datatype::UINT8, num_cells, 1, 1, true)
-              .ok());
-
-  return result_tile;
-}
-
-TEST_CASE(
+TEST_CASE_METHOD(
+    CSparseUnorderedWithDupsVarDataFx,
     "Sparse unordered with dups reader: test compute_var_size_offsets",
     "[sparse-unordered-with-dups][compute_var_size_offsets]") {
+  auto&& [array, fragments] = open_default_array_1d_with_fragments();
+
   // Make a vector of tiles.
-  std::vector<ResultTileWithBitmap<uint64_t>> rt = {make_tile(5)};
+  ResultTileWithBitmap<uint64_t> result_tile(
+      0, 0, array->array_->array_schema_latest());
+  std::vector<ResultTileWithBitmap<uint64_t>> rt;
+  rt.push_back(std::move(result_tile));
 
   SECTION("- No bitmap") {
   }
@@ -1253,7 +1326,7 @@ TEST_CASE(
   uint64_t offset = 0;
   for (uint64_t i = 0; i < rt.size(); i++) {
     cell_offsets[i] = offset;
-    offset += rt[i].cell_num();
+    offset += 5;
   }
   cell_offsets[rt.size()] = offset;
 
@@ -1263,35 +1336,45 @@ TEST_CASE(
   uint64_t offsets_size = sizeof(offsets);
   query_buffer.buffer_ = offsets;
   query_buffer.buffer_size_ = &offsets_size;
-  uint64_t var_size = 6;
-  query_buffer.buffer_var_size_ = &var_size;
+  query_buffer.original_buffer_size_ = offsets_size;
+  uint64_t buffer_var_size = 0;
+  query_buffer.buffer_var_size_ = &buffer_var_size;
+  query_buffer.original_buffer_var_size_ = 6;
 
   // Call the function.
-  uint64_t new_result_tiles_size = result_tiles.size();
-  uint64_t var_buffer_size = 0;
-  REQUIRE(SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
-              uint64_t>(
-              &tiledb::test::g_helper_stats,
-              &result_tiles,
-              0,
-              &cell_offsets,
-              &query_buffer,
-              &new_result_tiles_size,
-              &var_buffer_size)
-              .ok());
+  auto&& [buffers_full, var_buffer_size, result_tiles_size] =
+      SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
+          uint64_t>(
+          &tiledb::test::g_helper_stats,
+          fragments,
+          result_tiles,
+          0,
+          cell_offsets,
+          query_buffer);
 
   // Validate results.
+  CHECK(buffers_full == true);
   CHECK(cell_offsets[1] == 3);
-  CHECK(new_result_tiles_size == 1);
+  CHECK(result_tiles_size == 1);
   CHECK(var_buffer_size == 6);
+
+  // Clean up.
+  REQUIRE(tiledb_array_close(ctx_, array) == TILEDB_OK);
+  tiledb_array_free(&array);
 }
 
-TEST_CASE(
+TEST_CASE_METHOD(
+    CSparseUnorderedWithDupsVarDataFx,
     "Sparse unordered with dups reader: test compute_var_size_offsets count "
     "bitmap",
     "[sparse-unordered-with-dups][compute_var_size_offsets][count-bitmap]") {
+  auto&& [array, fragments] = open_default_array_1d_with_fragments();
+
   // Make a vector of tiles.
-  std::vector<ResultTileWithBitmap<uint64_t>> rt = {make_tile(5)};
+  ResultTileWithBitmap<uint64_t> result_tile(
+      0, 0, array->array_->array_schema_latest());
+  std::vector<ResultTileWithBitmap<uint64_t>> rt;
+  rt.push_back(std::move(result_tile));
   rt[0].bitmap_.resize(5);
   rt[0].bitmap_ = {0, 1, 2, 0, 2};
 
@@ -1306,7 +1389,7 @@ TEST_CASE(
   uint64_t offset = 0;
   for (uint64_t i = 0; i < rt.size(); i++) {
     cell_offsets[i] = offset;
-    offset += rt[i].cell_num();
+    offset += 5;
   }
   cell_offsets[rt.size()] = offset;
 
@@ -1316,35 +1399,44 @@ TEST_CASE(
   uint64_t offsets_size = sizeof(offsets);
   query_buffer.buffer_ = offsets;
   query_buffer.buffer_size_ = &offsets_size;
-  uint64_t var_size = 6;
-  query_buffer.buffer_var_size_ = &var_size;
+  uint64_t buffer_var_size = 0;
+  query_buffer.buffer_var_size_ = &buffer_var_size;
+  query_buffer.original_buffer_var_size_ = 6;
 
   // Call the function.
-  uint64_t new_result_tiles_size = result_tiles.size();
-  uint64_t var_buffer_size = 0;
-  REQUIRE(SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
-              uint64_t>(
-              &tiledb::test::g_helper_stats,
-              &result_tiles,
-              0,
-              &cell_offsets,
-              &query_buffer,
-              &new_result_tiles_size,
-              &var_buffer_size)
-              .ok());
+  auto&& [buffers_full, var_buffer_size, result_tiles_size] =
+      SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
+          uint64_t>(
+          &tiledb::test::g_helper_stats,
+          fragments,
+          result_tiles,
+          0,
+          cell_offsets,
+          query_buffer);
 
   // Validate results.
+  CHECK(buffers_full == true);
   CHECK(cell_offsets[1] == 3);
-  CHECK(new_result_tiles_size == 1);
+  CHECK(result_tiles_size == 1);
   CHECK(var_buffer_size == 6);
+
+  // Clean up.
+  REQUIRE(tiledb_array_close(ctx_, array) == TILEDB_OK);
+  tiledb_array_free(&array);
 }
 
-TEST_CASE(
+TEST_CASE_METHOD(
+    CSparseUnorderedWithDupsVarDataFx,
     "Sparse unordered with dups reader: test compute_var_size_offsets "
     "continuation",
     "[sparse-unordered-with-dups][compute_var_size_offsets][continuation]") {
+  auto&& [array, fragments] = open_default_array_1d_with_fragments();
+
   // Make a vector of tiles.
-  std::vector<ResultTileWithBitmap<uint64_t>> rt = {make_tile(5)};
+  ResultTileWithBitmap<uint64_t> result_tile(
+      0, 0, array->array_->array_schema_latest());
+  std::vector<ResultTileWithBitmap<uint64_t>> rt;
+  rt.push_back(std::move(result_tile));
 
   SECTION("- No bitmap") {
   }
@@ -1364,7 +1456,7 @@ TEST_CASE(
   uint64_t offset = 0;
   for (uint64_t i = 0; i < rt.size(); i++) {
     cell_offsets[i] = offset;
-    offset += rt[i].cell_num() - 2;
+    offset += 3;
   }
   cell_offsets[rt.size()] = offset;
 
@@ -1374,25 +1466,58 @@ TEST_CASE(
   uint64_t offsets_size = sizeof(offsets);
   query_buffer.buffer_ = offsets;
   query_buffer.buffer_size_ = &offsets_size;
-  uint64_t var_size = 5;
-  query_buffer.buffer_var_size_ = &var_size;
+  uint64_t buffer_var_size = 0;
+  query_buffer.buffer_var_size_ = &buffer_var_size;
+  query_buffer.original_buffer_var_size_ = 5;
 
   // Call the function.
-  uint64_t new_result_tiles_size = result_tiles.size();
-  uint64_t var_buffer_size = 0;
-  REQUIRE(SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
-              uint64_t>(
-              &tiledb::test::g_helper_stats,
-              &result_tiles,
-              2,
-              &cell_offsets,
-              &query_buffer,
-              &new_result_tiles_size,
-              &var_buffer_size)
-              .ok());
+  auto&& [buffers_full, var_buffer_size, result_tiles_size] =
+      SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
+          uint64_t>(
+          &tiledb::test::g_helper_stats,
+          fragments,
+          result_tiles,
+          2,
+          cell_offsets,
+          query_buffer);
 
   // Validate results.
+  CHECK(buffers_full == true);
   CHECK(cell_offsets[1] == 2);
-  CHECK(new_result_tiles_size == 1);
+  CHECK(result_tiles_size == 1);
   CHECK(var_buffer_size == 4);
+
+  // Clean up.
+  REQUIRE(tiledb_array_close(ctx_, array) == TILEDB_OK);
+  tiledb_array_free(&array);
+}
+
+TEST_CASE_METHOD(
+    CSparseUnorderedWithDupsFx,
+    "Sparse unordered with dups reader: empty strings",
+    "[sparse-unordered-with-dups][empty-strings]") {
+  // Create default array.
+  reset_config();
+  create_default_array_1d_string();
+
+  // Write a fragment.
+  int coords[] = {1, 2};
+  uint64_t coords_size = sizeof(coords);
+  write_1d_fragment_empty_strings(coords, &coords_size);
+
+  // Try to read.
+  int coords_r[5];
+  char data_r[5];
+  uint64_t data_offsets_r[5];
+  uint64_t coords_r_size = sizeof(coords_r);
+  uint64_t data_r_size = sizeof(data_r);
+  uint64_t data_offsets_r_size = sizeof(data_offsets_r);
+  auto rc = read_strings(
+      coords_r,
+      &coords_r_size,
+      data_r,
+      &data_r_size,
+      data_offsets_r,
+      &data_offsets_r_size);
+  CHECK(rc == TILEDB_OK);
 }
\ No newline at end of file
