@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,23 +33,29 @@
 #ifndef TILEDB_DOMAIN_H
 #define TILEDB_DOMAIN_H
 
+#include "tiledb/common/common.h"
 #include "tiledb/common/macros.h"
 #include "tiledb/common/status.h"
+#include "tiledb/common/types/dynamic_typed_datum.h"
+#include "tiledb/common/types/untyped_datum.h"
 #include "tiledb/sm/misc/types.h"
-#include "tiledb/sm/query/query_buffer.h"
-#include "tiledb/sm/query/result_coords.h"
 
 #include <vector>
 
 using namespace tiledb::common;
+using namespace tiledb::type;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::type {
+class DomainDataRef;
+class Range;
+}  // namespace tiledb::type
+
+namespace tiledb::sm {
 
 class Buffer;
 class ConstBuffer;
 class Dimension;
-
+class DomainTypedDataView;
 enum class Datatype : uint8_t;
 enum class Layout : uint8_t;
 
@@ -63,6 +69,12 @@ class Domain {
   /** Empty constructor. */
   Domain();
 
+  /** Constructor.*/
+  Domain(
+      Layout cell_order,
+      const std::vector<shared_ptr<Dimension>> dimensions,
+      Layout tile_order);
+
   /**
    * Constructor that clones the input domain.
    *
@@ -99,7 +111,7 @@ class Domain {
    * @param dim The dimension to be added.
    * @return Status
    */
-  Status add_dimension(const Dimension* dim);
+  Status add_dimension(shared_ptr<Dimension> dim);
 
   /** Returns true if all dimensions have fixed-sized domain datatypes. */
   bool all_dims_fixed() const;
@@ -124,7 +136,6 @@ class Domain {
    * of a regular tile grid, this function assumes that the cells are in the
    * same regular tile.
    *
-   * @tparam T The coordinates type.
    * @param dim_idx The dimension to compare the coordinates on.
    * @param a The first input coordinates.
    * @param b The second input coordinates.
@@ -134,7 +145,9 @@ class Domain {
    *    - +1 if the first coordinate succeeds the second
    */
   int cell_order_cmp(
-      unsigned dim_idx, const ResultCoords& a, const ResultCoords& b) const;
+      unsigned dim_idx,
+      const UntypedDatumView a,
+      const UntypedDatumView b) const;
 
   /**
    * Checks the cell order of the input coordinates. Since the coordinates
@@ -151,50 +164,31 @@ class Domain {
   template <class T>
   static int cell_order_cmp_2(const void* coord_a, const void* coord_b);
 
-  /**
-   * Checks the cell order of the input coordinates. Since the coordinates
-   * are given for a single dimension, this function simply checks which
-   * coordinate is larger.
-   *
-   * @param dim The dimension to check the coordinates on.
-   * @param buff The buffer that stores all coordinates.
-   * @param a The position of the first coordinate in the buffer to check.
-   * @param b The position of the second coordinate in the buffer to check.
-   * @return One of the following:
-   *    - -1 if the first coordinate is smaller than the second
-   *    -  0 if the two coordinates have the same cell order
-   *    - +1 if the first coordinate is larger than the second
-   */
-  template <class T>
-  static int cell_order_cmp(
-      const Dimension* dim, const QueryBuffer* buff, uint64_t a, uint64_t b);
-
   /**
    * Checks the cell order of the input coordinates.
    *
-   * @param coord_buffs The input coordinates, given n separate buffers,
-   *     one per dimension. The buffers are sorted in the same order of the
-   *     dimensions as defined in the array schema.
-   * @param a The position of the first coordinate tuple across all buffers.
-   * @param b The position of the second coordinate tuple across all buffers.
+   * @param left Left operand
+   * @param right Right operand
    * @return One of the following:
-   *    - -1 if the first coordinates precede the second on the cell order
+   *    - -1 if the left coordinates precede the right on the cell order
    *    -  0 if the two coordinates have the same cell order
-   *    - +1 if the first coordinates succeed the second on the cell order
+   *    - +1 if the left coordinates succeed the right on the cell order
    */
   int cell_order_cmp(
-      const std::vector<const QueryBuffer*>& coord_buffs,
-      uint64_t a,
-      uint64_t b) const;
+      const type::DomainDataRef& left, const type::DomainDataRef& right) const;
 
   /**
    * Populates the object members from the data in the input binary buffer.
    *
    * @param buff The buffer to deserialize from.
    * @param version The array schema version.
-   * @return Status
+   * @return Status and Domain
    */
-  Status deserialize(ConstBuffer* buff, uint32_t version);
+  static tuple<Status, optional<shared_ptr<Domain>>> deserialize(
+      ConstBuffer* buff,
+      uint32_t version,
+      Layout cell_order,
+      Layout tile_order);
 
   /** Returns the cell order. */
   Layout cell_order() const;
@@ -208,7 +202,9 @@ class Domain {
   Layout tile_order() const;
 
   /** Returns the number of dimensions. */
-  unsigned int dim_num() const;
+  inline unsigned int dim_num() const {
+    return dim_num_;
+  }
 
   /** Returns the domain along the i-th dimension. */
   const Range& domain(unsigned i) const;
@@ -217,10 +213,17 @@ class Domain {
   NDRange domain() const;
 
   /** Returns the i-th dimensions (nullptr upon error). */
-  const Dimension* dimension(unsigned int i) const;
+  shared_ptr<const Dimension> dimension(unsigned int i) const;
+
+  inline const Dimension* dimension_ptr(unsigned int i) const {
+    if (i > dim_num_) {
+      throw std::invalid_argument("invalid dimension index");
+    }
+    return dimension_ptrs_[i];
+  }
 
   /** Returns the dimension given a name (nullptr upon error). */
-  const Dimension* dimension(const std::string& name) const;
+  shared_ptr<const Dimension> dimension(const std::string& name) const;
 
   /** Dumps the domain in ASCII format in the selected output. */
   void dump(FILE* out) const;
@@ -428,40 +431,24 @@ class Domain {
    * Return ratio of the overalp of the two input ND ranges over
    * the volume of `r2`.
    */
-  double overlap_ratio(const NDRange& r1, const NDRange& r2) const;
-
-  /**
-   * Checks the tile order of the input coordinates on the given dimension.
-   *
-   * @param The dimension to compare on.
-   * @param coord_a The first coordinate.
-   * @param coord_b The second coordinate.
-   * @return One of the following:
-   *    - -1 if the first coordinate precedes the second on the tile order
-   *    -  0 if the two coordinates have the same tile order
-   *    - +1 if the first coordinate succeeds the second on the tile order
-   */
-  template <class T>
-  static int tile_order_cmp(
-      const Dimension* dim, const void* coord_a, const void* coord_b);
+  double overlap_ratio(
+      const NDRange& r1,
+      const std::vector<bool>& r1_default,
+      const NDRange& r2) const;
 
   /**
    * Checks the tile order of the input coordinates.
    *
-   * @param coord_buffs The input coordinates, given n separate buffers,
-   *     one per dimension. The buffers are sorted in the same order of the
-   *     dimensions as defined in the array schema.
-   * @param a The position of the first coordinate tuple across all buffers.
-   * @param b The position of the second coordinate tuple across all buffers.
+   * @param left
+   * @param right
    * @return One of the following:
    *    - -1 if the first coordinates precede the second on the tile order
    *    -  0 if the two coordinates have the same tile order
    *    - +1 if the first coordinates succeed the second on the tile order
    */
   int tile_order_cmp(
-      const std::vector<const QueryBuffer*>& coord_buffs,
-      uint64_t a,
-      uint64_t b) const;
+      const tiledb::type::DomainDataRef& left,
+      const tiledb::type::DomainDataRef& right) const;
 
   /**
    * Checks the tile order of the input coordinates for a given dimension.
@@ -488,8 +475,17 @@ class Domain {
   /** The cell order of the array the domain belongs to. */
   Layout cell_order_;
 
-  /** The domain dimensions. */
-  std::vector<tdb_unique_ptr<Dimension>> dimensions_;
+  /**
+   * Allocation for the dimensions of this domain.
+   *
+   * This array holds the memory resource for the dimensions.
+   */
+  std::vector<shared_ptr<Dimension>> dimensions_;
+
+  /**
+   * Non-allocating mirror of the dimensions vector.
+   */
+  std::vector<const Dimension*> dimension_ptrs_;
 
   /** The number of dimensions. */
   unsigned dim_num_;
@@ -506,7 +502,7 @@ class Domain {
    * - a, b: The positions of the two coordinates in the buffer to compare.
    */
   std::vector<int (*)(
-      const Dimension* dim, const QueryBuffer* buff, uint64_t a, uint64_t b)>
+      const Dimension* dim, const UntypedDatumView a, const UntypedDatumView b)>
       cell_order_cmp_func_;
 
   /**
@@ -533,6 +529,38 @@ class Domain {
   /*           PRIVATE METHODS         */
   /* ********************************* */
 
+  /**
+   * Checks the cell order of the input coordinates. Since the coordinates
+   * are given for a single dimension, this function simply checks which
+   * coordinate is larger.
+   *
+   * @param dim The dimension to check the coordinates on.
+   * @param a The position of the first coordinate in the buffer to check.
+   * @param b The position of the second coordinate in the buffer to check.
+   * @return One of the following:
+   *    - -1 if the first coordinate is smaller than the second
+   *    -  0 if the two coordinates have the same cell order
+   *    - +1 if the first coordinate is larger than the second
+   */
+  template <class T>
+  static int cell_order_cmp_impl(
+      const Dimension* dim, UntypedDatumView a, UntypedDatumView b);
+
+  /**
+   * Checks the tile order of the input coordinates on the given dimension.
+   *
+   * @param dim The dimension to compare on.
+   * @param coord_a The first coordinate.
+   * @param coord_b The second coordinate.
+   * @return One of the following:
+   *    - -1 if the first coordinate precedes the second on the tile order
+   *    -  0 if the two coordinates have the same tile order
+   *    - +1 if the first coordinate succeeds the second on the tile order
+   */
+  template <class T>
+  static int tile_order_cmp_impl(
+      const Dimension* dim, const void* coord_a, const void* coord_b);
+
   /** Compute the number of cells per tile. */
   void compute_cell_num_per_tile();
 
@@ -641,7 +669,6 @@ class Domain {
   uint64_t get_tile_pos_row(const T* domain, const T* tile_coords) const;
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_DOMAIN_H
