@@ -34,7 +34,6 @@
 #define TILEDB_CONFIG_H
 
 #include "tiledb/common/status.h"
-#include "tiledb/sm/misc/utils.h"
 
 #include <map>
 #include <set>
@@ -78,6 +77,9 @@ class Config {
   /** The default exponential delay factor for retrying a http request. */
   static const std::string REST_RETRY_DELAY_FACTOR;
 
+  /** The default for Curl's verbose mode used by REST. */
+  static const std::string REST_CURL_VERBOSE;
+
   /** The prefix to use for checking for parameter environmental variables. */
   static const std::string CONFIG_ENVIRONMENT_VARIABLE_PREFIX;
 
@@ -176,10 +178,6 @@ class Config {
   /** Ratio of the sparse global order reader budget used for array data. */
   static const std::string SM_MEM_SPARSE_GLOBAL_ORDER_RATIO_ARRAY_DATA;
 
-  /** Ratio of the sparse global order reader budget used for result cell slabs.
-   */
-  static const std::string SM_MEM_SPARSE_GLOBAL_ORDER_RATIO_RCS;
-
   /** Ratio of the sparse unordered with dups reader budget used for coords. */
   static const std::string SM_MEM_SPARSE_UNORDERED_WITH_DUPS_RATIO_COORDS;
 
@@ -249,6 +247,7 @@ class Config {
    *     - "fragments": only the fragments will be consolidated
    *     - "fragment_meta": only the fragment metadata will be consolidated
    *     - "array_meta": only the array metadata will be consolidated
+   *     - "commits": only the commit files will be consolidated
 
    */
   static const std::string SM_CONSOLIDATION_MODE;
@@ -268,6 +267,7 @@ class Config {
    *     - "fragments": only the fragments will be vacuumed
    *     - "fragment_meta": only the fragment metadata will be vacuumed
    *     - "array_meta": only the array metadata will be vacuumed
+   *     - "commits": only the commit files will be vacuumed
    */
   static const std::string SM_VACUUM_MODE;
 
@@ -306,9 +306,22 @@ class Config {
    */
   static const std::string SM_MAX_TILE_OVERLAP_SIZE;
 
+  /**
+   * A group will open between this value and timestamp_end.
+   * */
+  static const std::string SM_GROUP_TIMESTAMP_START;
+
+  /**
+   * An group will open between timestamp_start and this value.
+   *  */
+  static const std::string SM_GROUP_TIMESTAMP_END;
+
   /** The default minimum number of bytes in a parallel VFS operation. */
   static const std::string VFS_MIN_PARALLEL_SIZE;
 
+  /** The default maximum number of bytes in a batched VFS read operation. */
+  static const std::string VFS_MAX_BATCH_SIZE;
+
   /**
    * The default minimum number of bytes between two VFS read batches.
    */
@@ -326,9 +339,6 @@ class Config {
   /** The default maximum number of parallel file:/// operations. */
   static const std::string VFS_FILE_MAX_PARALLEL_OPS;
 
-  /** Whether or not filelocks are enabled for VFS. */
-  static const std::string VFS_FILE_ENABLE_FILELOCKS;
-
   /** The maximum size (in bytes) to read-ahead in the VFS. */
   static const std::string VFS_READ_AHEAD_SIZE;
 
@@ -531,15 +541,7 @@ class Config {
    */
   template <class T>
   Status get_vector(
-      const std::string& param, std::vector<T>* value, bool* found) const {
-    // Check if parameter exists
-    const char* val = get_from_config_or_env(param, found);
-    if (!*found)
-      return Status::Ok();
-
-    // Parameter found, retrieve value
-    return utils::parse::convert<T>(val, value);
-  }
+      const std::string& param, std::vector<T>* value, bool* found) const;
 
   /** Returns the param -> value map. */
   const std::map<std::string, std::string>& param_values() const;
