@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -86,6 +86,11 @@ BitWidthReductionFilter::BitWidthReductionFilter()
   max_window_size_ = 256;
 }
 
+BitWidthReductionFilter::BitWidthReductionFilter(uint32_t max_window_size)
+    : Filter(FilterType::FILTER_BIT_WIDTH_REDUCTION)
+    , max_window_size_(max_window_size) {
+}
+
 void BitWidthReductionFilter::dump(FILE* out) const {
   if (out == nullptr)
     out = stdout;
@@ -93,11 +98,13 @@ void BitWidthReductionFilter::dump(FILE* out) const {
 }
 
 Status BitWidthReductionFilter::run_forward(
+    const Tile& tile,
+    Tile* const offsets_tile,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
     FilterBuffer* output) const {
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
   auto tile_type_size = static_cast<uint8_t>(datatype_size(tile_type));
 
   // If bit width compression can't work, just return the input unmodified.
@@ -107,30 +114,35 @@ Status BitWidthReductionFilter::run_forward(
     return Status::Ok();
   }
 
+  /* Note: Arithmetic operations cannot be performed on std::byte.
+    We will use uint8_t for the Datatype::BLOB case as it is the same size as
+    std::byte and can have arithmetic perfomed on it. */
   switch (tile_type) {
     case Datatype::INT8:
       return run_forward<int8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
+    case Datatype::BLOB:
     case Datatype::UINT8:
       return run_forward<uint8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT16:
       return run_forward<int16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT16:
       return run_forward<uint16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT32:
-      return run_forward<int>(input_metadata, input, output_metadata, output);
+      return run_forward<int>(
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT32:
       return run_forward<unsigned>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT64:
       return run_forward<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT64:
       return run_forward<uint64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -154,15 +166,17 @@ Status BitWidthReductionFilter::run_forward(
     case Datatype::TIME_FS:
     case Datatype::TIME_AS:
       return run_forward<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     default:
       return LOG_STATUS(
-          Status::FilterError("Cannot filter; Unsupported input type"));
+          Status_FilterError("Cannot filter; Unsupported input type"));
   }
 }
 
 template <typename T>
 Status BitWidthReductionFilter::run_forward(
+    const Tile&,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -264,6 +278,8 @@ Status BitWidthReductionFilter::compress_part(
 }
 
 Status BitWidthReductionFilter::run_reverse(
+    const Tile& tile,
+    Tile* const offsets_tile,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -271,7 +287,7 @@ Status BitWidthReductionFilter::run_reverse(
     const Config& config) const {
   (void)config;
 
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
   auto tile_type_size = static_cast<uint8_t>(datatype_size(tile_type));
 
   // If bit width compression wasn't applied, just return the input unmodified.
@@ -281,30 +297,35 @@ Status BitWidthReductionFilter::run_reverse(
     return Status::Ok();
   }
 
+  /* Note: Arithmetic operations cannot be performed on std::byte.
+    We will use uint8_t for the Datatype::BLOB case as it is the same size as
+    std::byte and can have arithmetic perfomed on it. */
   switch (tile_type) {
     case Datatype::INT8:
       return run_reverse<int8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
+    case Datatype::BLOB:
     case Datatype::UINT8:
       return run_reverse<uint8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT16:
       return run_reverse<int16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT16:
       return run_reverse<uint16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT32:
-      return run_reverse<int>(input_metadata, input, output_metadata, output);
+      return run_reverse<int>(
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT32:
       return run_reverse<unsigned>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT64:
       return run_reverse<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT64:
       return run_reverse<uint64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -328,20 +349,22 @@ Status BitWidthReductionFilter::run_reverse(
     case Datatype::TIME_FS:
     case Datatype::TIME_AS:
       return run_reverse<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     default:
       return LOG_STATUS(
-          Status::FilterError("Cannot filter; Unsupported input type"));
+          Status_FilterError("Cannot filter; Unsupported input type"));
   }
 }
 
 template <typename T>
 Status BitWidthReductionFilter::run_reverse(
+    const Tile& tile,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
     FilterBuffer* output) const {
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
   auto tile_type_size = datatype_size(tile_type);
 
   uint32_t num_windows, orig_length;
@@ -508,7 +531,7 @@ Status BitWidthReductionFilter::read_compressed_value(
 Status BitWidthReductionFilter::set_option_impl(
     FilterOption option, const void* value) {
   if (value == nullptr)
-    return LOG_STATUS(Status::FilterError(
+    return LOG_STATUS(Status_FilterError(
         "Bit width reduction filter error; invalid option value"));
 
   switch (option) {
@@ -516,7 +539,7 @@ Status BitWidthReductionFilter::set_option_impl(
       max_window_size_ = *(uint32_t*)value;
       return Status::Ok();
     default:
-      return LOG_STATUS(Status::FilterError(
+      return LOG_STATUS(Status_FilterError(
           "Bit width reduction filter error; unknown option"));
   }
 }
@@ -528,7 +551,7 @@ Status BitWidthReductionFilter::get_option_impl(
       *(uint32_t*)value = max_window_size_;
       return Status::Ok();
     default:
-      return LOG_STATUS(Status::FilterError(
+      return LOG_STATUS(Status_FilterError(
           "Bit width reduction filter error; unknown option"));
   }
 }
@@ -547,11 +570,6 @@ BitWidthReductionFilter* BitWidthReductionFilter::clone_impl() const {
   return clone;
 }
 
-Status BitWidthReductionFilter::deserialize_impl(ConstBuffer* buff) {
-  RETURN_NOT_OK(buff->read(&max_window_size_, sizeof(uint32_t)));
-  return Status::Ok();
-}
-
 Status BitWidthReductionFilter::serialize_impl(Buffer* buff) const {
   RETURN_NOT_OK(buff->write(&max_window_size_, sizeof(uint32_t)));
   return Status::Ok();
