@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,8 +46,8 @@
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/enums/query_status_details.h"
 #include "tiledb/sm/fragment/written_fragment_info.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/iquery_strategy.h"
+#include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/validity_vector.h"
 #include "tiledb/sm/subarray/subarray.h"
@@ -160,16 +160,6 @@ class Query {
   Status add_range(
       unsigned dim_idx, const void* start, const void* end, const void* stride);
 
-  /**
-   * @brief Set point ranges from an array
-   *
-   * @param dim_idx Dimension index
-   * @param start Pointer to start of the array
-   * @param count Number of elements to add
-   * @return Status
-   */
-  Status add_point_ranges(unsigned dim_idx, const void* start, uint64_t count);
-
   /**
    * Adds a variable-sized range to the (read/write) query on the input
    * dimension by index, in the form of (start, end).
@@ -357,7 +347,7 @@ class Query {
   Array* array();
 
   /** Returns the array schema. */
-  const ArraySchema* array_schema() const;
+  const ArraySchema& array_schema() const;
 
   /** Returns the names of the buffers set by the user for the query. */
   std::vector<std::string> buffer_names() const;
@@ -839,6 +829,17 @@ class Query {
   /** Returns the query subarray. */
   const Subarray* subarray() const;
 
+  /**
+   * Sets the query subarray.
+   *
+   * @param subarray The subarray to be set.
+   * @return Status
+   *
+   * @note Calling set_subarray for sparse arrays, or for dense arrays
+   *     when performing unordered (sparse) writes, has no effect.
+   */
+  Status set_subarray(const tiledb::sm::Subarray& subarray);
+
   /** Sets the query subarray, without performing any checks. */
   Status set_subarray_unsafe(const Subarray& subarray);
 
@@ -869,7 +870,7 @@ class Query {
   stats::Stats* stats() const;
 
   /** Returns the scratch space used for REST requests. */
-  tdb_shared_ptr<Buffer> rest_scratch() const;
+  shared_ptr<Buffer> rest_scratch() const;
 
   /** Use the refactored dense reader or not. */
   bool use_refactored_dense_reader();
@@ -881,7 +882,7 @@ class Query {
   bool use_refactored_sparse_unordered_with_dups_reader();
 
   /** Returns if all ranges for this query are non overlapping. */
-  std::tuple<Status, bool> non_overlapping_ranges();
+  tuple<Status, optional<bool>> non_overlapping_ranges();
 
  private:
   /* ********************************* */
@@ -892,7 +893,7 @@ class Query {
   Array* array_;
 
   /** The array schema. */
-  ArraySchema* array_schema_;
+  shared_ptr<const ArraySchema> array_schema_;
 
   /** The config for query-level parameters only. */
   Config config_;
@@ -925,7 +926,7 @@ class Query {
   stats::Stats* stats_;
 
   /** The class logger. */
-  tdb_shared_ptr<Logger> logger_;
+  shared_ptr<Logger> logger_;
 
   /** UID of the logger instance */
   inline static std::atomic<uint64_t> logger_id_ = 0;
@@ -947,7 +948,7 @@ class Query {
   QueryCondition condition_;
 
   /** The fragment metadata that this query will focus on. */
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata_;
+  std::vector<shared_ptr<FragmentMetadata>> fragment_metadata_;
 
   /** The current serialization state. */
   SerializationState serialization_state_;
@@ -983,7 +984,7 @@ class Query {
   URI fragment_uri_;
 
   /* Scratch space used for REST requests. */
-  tdb_shared_ptr<Buffer> rest_scratch_;
+  shared_ptr<Buffer> rest_scratch_;
 
   /* ********************************* */
   /*           PRIVATE METHODS         */
