@@ -35,10 +35,10 @@
 
 #include <atomic>
 
+#include "tiledb/common/common.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
-#include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/query/iquery_strategy.h"
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_condition.h"
@@ -66,7 +66,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
   /** Constructor. */
   Reader(
       stats::Stats* stats,
-      tdb_shared_ptr<Logger> logger,
+      shared_ptr<Logger> logger,
       StorageManager* storage_manager,
       Array* array,
       Config& config,
@@ -138,10 +138,10 @@ class Reader : public ReaderBase, public IQueryStrategy {
   template <class T>
   Status compute_result_cell_slabs(
       const Subarray& subarray,
-      std::map<const T*, ResultSpaceTile<T>>* result_space_tiles,
-      std::vector<ResultCoords>* result_coords,
-      std::vector<ResultTile*>* result_tiles,
-      std::vector<ResultCellSlab>* result_cell_slabs) const;
+      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
+      std::vector<ResultCoords>& result_coords,
+      std::vector<ResultTile*>& result_tiles,
+      std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Computes the result cell slabs for the input subarray, given the
@@ -174,12 +174,12 @@ class Reader : public ReaderBase, public IQueryStrategy {
   template <class T>
   Status compute_result_cell_slabs_row_col(
       const Subarray& subarray,
-      std::map<const T*, ResultSpaceTile<T>>* result_space_tiles,
-      std::vector<ResultCoords>* result_coords,
+      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
+      std::vector<ResultCoords>& result_coords,
       uint64_t* result_coords_pos,
-      std::vector<ResultTile*>* result_tiles,
-      std::set<std::pair<unsigned, uint64_t>>* frag_tile_set,
-      std::vector<ResultCellSlab>* result_cell_slabs) const;
+      std::vector<ResultTile*>& result_tiles,
+      std::set<std::pair<unsigned, uint64_t>>& frag_tile_set,
+      std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Computes the result cell slabs for the input subarray, given the
@@ -204,10 +204,10 @@ class Reader : public ReaderBase, public IQueryStrategy {
   template <class T>
   Status compute_result_cell_slabs_global(
       const Subarray& subarray,
-      std::map<const T*, ResultSpaceTile<T>>* result_space_tiles,
-      std::vector<ResultCoords>* result_coords,
-      std::vector<ResultTile*>* result_tiles,
-      std::vector<ResultCellSlab>* result_cell_slabs) const;
+      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
+      std::vector<ResultCoords>& result_coords,
+      std::vector<ResultTile*>& result_tiles,
+      std::vector<ResultCellSlab>& result_cell_slabs) const;
 
  private:
   /* ********************************* */
@@ -220,6 +220,20 @@ class Reader : public ReaderBase, public IQueryStrategy {
   /** Read state. */
   ReadState read_state_;
 
+  /* ********************************* */
+  /*         PRIVATE DATATYPES         */
+  /* ********************************* */
+
+  /** Bitflags for individual dimension/attributes in `process_tiles()`. */
+  typedef uint8_t ProcessTileFlags;
+
+  /** Bitflag values applicable to `ProcessTileFlags`. */
+  enum ProcessTileFlag { READ = 1, COPY = 2 };
+
+  typedef std::
+      unordered_map<ResultTile*, std::vector<std::pair<uint64_t, uint64_t>>>
+          ResultCellSlabsIndex;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
@@ -237,9 +251,9 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status apply_query_condition(
-      std::vector<ResultCellSlab>* result_cell_slabs,
-      std::vector<ResultTile*>* result_tiles,
-      Subarray* subarray,
+      std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<ResultTile*>& result_tiles,
+      Subarray& subarray,
       uint64_t stride = UINT64_MAX);
 
   /**
@@ -251,7 +265,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
    */
   Status compute_result_cell_slabs(
       const std::vector<ResultCoords>& result_coords,
-      std::vector<ResultCellSlab>* result_cell_slabs) const;
+      std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Retrieves the coordinates that overlap the input N-dimensional range
@@ -265,11 +279,11 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status compute_range_result_coords(
-      Subarray* subarray,
+      Subarray& subarray,
       unsigned frag_idx,
       ResultTile* tile,
       uint64_t range_idx,
-      std::vector<ResultCoords>* result_coords);
+      std::vector<ResultCoords>& result_coords);
 
   /**
    * Computes the result coordinates for each range of the query
@@ -286,11 +300,11 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status compute_range_result_coords(
-      Subarray* subarray,
+      Subarray& subarray,
       const std::vector<bool>& single_fragment,
       const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-      std::vector<ResultTile>* result_tiles,
-      std::vector<std::vector<ResultCoords>>* range_result_coords);
+      std::vector<ResultTile>& result_tiles,
+      std::vector<std::vector<ResultCoords>>& range_result_coords);
 
   /**
    * Computes the result coordinates of a given range of the query
@@ -306,11 +320,11 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status compute_range_result_coords(
-      Subarray* subarray,
+      Subarray& subarray,
       uint64_t range_idx,
       const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-      std::vector<ResultTile>* result_tiles,
-      std::vector<ResultCoords>* range_result_coords);
+      std::vector<ResultTile>& result_tiles,
+      std::vector<ResultCoords>& range_result_coords);
 
   /**
    * Computes the result coordinates of a given range of the query
@@ -327,12 +341,12 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status compute_range_result_coords(
-      Subarray* subarray,
+      Subarray& subarray,
       uint64_t range_idx,
       uint32_t fragment_idx,
       const std::map<std::pair<unsigned, uint64_t>, size_t>& result_tile_map,
-      std::vector<ResultTile>* result_tiles,
-      std::vector<ResultCoords>* range_result_coords);
+      std::vector<ResultTile>& result_tiles,
+      std::vector<ResultCoords>& range_result_coords);
 
   /**
    * Computes the final subarray result coordinates, which will be
@@ -347,8 +361,8 @@ class Reader : public ReaderBase, public IQueryStrategy {
    *     as it is done processing its elements to quickly reclaim memory.
    */
   Status compute_subarray_coords(
-      std::vector<std::vector<ResultCoords>>* range_result_coords,
-      std::vector<ResultCoords>* result_coords);
+      std::vector<std::vector<ResultCoords>>& range_result_coords,
+      std::vector<ResultCoords>& result_coords);
 
   /**
    * Computes info about the sparse result tiles, such as which fragment they
@@ -366,10 +380,191 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status compute_sparse_result_tiles(
-      std::vector<ResultTile>* result_tiles,
+      std::vector<ResultTile>& result_tiles,
       std::map<std::pair<unsigned, uint64_t>, size_t>* result_tile_map,
       std::vector<bool>* single_fragment);
 
+  /**
+   * Copies the result coordinates to the user buffers.
+   * It also appropriately cleans up the used result tiles.
+   */
+  Status copy_coordinates(
+      const std::vector<ResultTile*>& result_tiles,
+      std::vector<ResultCellSlab>& result_cell_slabs);
+
+  /**
+   * Copies the result attribute values to the user buffers.
+   * It also appropriately cleans up the used result tiles.
+   */
+  Status copy_attribute_values(
+      uint64_t stride,
+      std::vector<ResultTile*>& result_tiles,
+      std::vector<ResultCellSlab>& result_cell_slabs,
+      Subarray& subarray);
+
+  /**
+   * Copies the cells for the input **fixed-sized** attribute/dimension and
+   * result cell slabs into the corresponding result buffers.
+   *
+   * @param name The targeted attribute/dimension.
+   * @param stride If it is `UINT64_MAX`, then the cells in the result
+   *     cell slabs are all contiguous. Otherwise, each cell in the
+   *     result cell slabs are `stride` cells apart from each other.
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param fixed_cs_partitions The cell slab partitions.
+   * @return Status
+   */
+  Status copy_fixed_cells(
+      const std::string& name,
+      uint64_t stride,
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<size_t>* fixed_cs_partitions);
+
+  /**
+   * Compute cs partitions for fixed-sized cell copying.
+   *
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param fixed_cs_partitions The output partitions.
+   */
+  void compute_fixed_cs_partitions(
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<size_t>* fixed_cs_partitions);
+
+  /**
+   * Copies the cells for the input **fixed-sized** attribute/dimension and
+   * result cell slabs into the corresponding result buffers for the
+   * partition in `ctx_cache` at index `partition_idx`.
+   *
+   * @param name The partition index.
+   * @param name The targeted attribute/dimension.
+   * @param stride If it is `UINT64_MAX`, then the cells in the result
+   *     cell slabs are all contiguous. Otherwise, each cell in the
+   *     result cell slabs are `stride` cells apart from each other.
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param cs_offsets The cell slab offsets.
+   * @param cs_partitions The cell slab partitions to operate on.
+   * @return Status
+   */
+  Status copy_partitioned_fixed_cells(
+      size_t partition_idx,
+      const std::string* name,
+      uint64_t stride,
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      const std::vector<uint64_t>* cs_offsets,
+      const std::vector<size_t>* cs_partitions);
+
+  /**
+   * Copies the cells for the input **var-sized** attribute/dimension and result
+   * cell slabs into the corresponding result buffers.
+   *
+   * @param name The targeted attribute/dimension.
+   * @param stride If it is `UINT64_MAX`, then the cells in the result
+   *     cell slabs are all contiguous. Otherwise, each cell in the
+   *     result cell slabs are `stride` cells apart from each other.
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param var_cs_partitions The cell slab partitions.
+   * @param total_cs_length The total cell slab length.
+   * @return Status
+   */
+  Status copy_var_cells(
+      const std::string& name,
+      uint64_t stride,
+      std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<std::pair<size_t, size_t>>* var_cs_partitions,
+      size_t total_var_cs_length);
+
+  /**
+   * Compute cs partitions for var-sized cell copying.
+   *
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param var_cs_partitions The output partitions.
+   * @param total_var_cs_length The total cell slab length.
+   */
+  void compute_var_cs_partitions(
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<std::pair<size_t, size_t>>* var_cs_partitions,
+      size_t* total_var_cs_length);
+
+  /**
+   * Computes offsets into destination buffers for the given
+   * attribute/dimensions's offset and variable-length data, for the given list
+   * of result cell slabs.
+   *
+   * @param name The variable-length attribute/dimension.
+   * @param stride If it is `UINT64_MAX`, then the cells in the result
+   *     cell slabs are all contiguous. Otherwise, each cell in the
+   *     result cell slabs are `stride` cells apart from each other.
+   * @param result_cell_slabs The result cell slabs to compute destinations for.
+   * @param offset_offsets_per_cs Output to hold one vector per result cell
+   *    slab, and one element per cell in the slab. The elements are the
+   *    destination offsets for the attribute's offsets.
+   * @param var_offsets_per_cs Output to hold one vector per result cell slab,
+   *    and one element per cell in the slab. The elements are the destination
+   *    offsets for the attribute's variable-length data.
+   * @param total_offset_size Output set to the total size in bytes of the
+   *    offsets in the given list of result cell slabs.
+   * @param total_var_size Output set to the total size in bytes of the
+   *    attribute's variable-length in the given list of result cell slabs.
+   * @param total_validity_size Output set to the total size in bytes of the
+   *    attribute's validity vector in the given list of result cell slabs.
+   * @return Status
+   */
+  Status compute_var_cell_destinations(
+      const std::string& name,
+      uint64_t stride,
+      std::vector<ResultCellSlab>& result_cell_slabs,
+      std::vector<uint64_t>* offset_offsets_per_cs,
+      std::vector<uint64_t>* var_offsets_per_cs,
+      uint64_t* total_offset_size,
+      uint64_t* total_var_size,
+      uint64_t* total_validity_size);
+
+  /**
+   * Copies the cells for the input **var-sized** attribute/dimension and result
+   * cell slabs into the corresponding result buffers for the
+   * partition in `cs_partitions` at index `partition_idx`.
+   *
+   * @param name The partition index.
+   * @param name The targeted attribute/dimension.
+   * @param stride If it is `UINT64_MAX`, then the cells in the result
+   *     cell slabs are all contiguous. Otherwise, each cell in the
+   *     result cell slabs are `stride` cells apart from each other.
+   * @param result_cell_slabs The result cell slabs to copy cells for.
+   * @param offset_offsets_per_cs Maps each cell slab to its offset
+   *     for its attribute offsets.
+   * @param var_offsets_per_cs Maps each cell slab to its offset
+   *     for its variable-length data.
+   * @param cs_partitions The cell slab partitions to operate on.
+   * @return Status
+   */
+  Status copy_partitioned_var_cells(
+      size_t partition_idx,
+      const std::string* name,
+      uint64_t stride,
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      const std::vector<uint64_t>* offset_offsets_per_cs,
+      const std::vector<uint64_t>* var_offsets_per_cs,
+      const std::vector<std::pair<size_t, size_t>>* cs_partitions);
+
+  /**
+   * For each dimension/attribute in `names`, performs the actions
+   * defined in the `ProcessTileFlags`.
+   *
+   * @param names The dimension/attribute names to process.
+   * @param result_tiles The retrieved tiles will be stored inside the
+   *   `ResultTile` instances in this vector.
+   * @param result_cell_slabs The cell slabs to process.
+   * @param subarray Specifies the current subarray.
+   * @param stride The stride between cells, UINT64_MAX for contiguous.
+   * @return Status
+   */
+  Status process_tiles(
+      const std::unordered_map<std::string, ProcessTileFlags>& names,
+      std::vector<ResultTile*>& result_tiles,
+      std::vector<ResultCellSlab>& result_cell_slabs,
+      Subarray& subarray,
+      uint64_t stride);
+
   /**
    * Computes the result coordinates from the sparse fragments.
    *
@@ -377,8 +572,8 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @param result_coords This will store the result coordinates.
    */
   Status compute_result_coords(
-      std::vector<ResultTile>* result_tiles,
-      std::vector<ResultCoords>* result_coords);
+      std::vector<ResultTile>& result_tiles,
+      std::vector<ResultCoords>& result_coords);
 
   /**
    * Deduplicates the input result coordinates, breaking ties giving preference
@@ -387,7 +582,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @param result_coords The result coordinates to dedup.
    * @return Status
    */
-  Status dedup_result_coords(std::vector<ResultCoords>* result_coords) const;
+  Status dedup_result_coords(std::vector<ResultCoords>& result_coords) const;
 
   /** Performs a read on a dense array. */
   Status dense_read();
@@ -409,7 +604,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * @return Status
    */
   Status get_all_result_coords(
-      ResultTile* tile, std::vector<ResultCoords>* result_coords) const;
+      ResultTile* tile, std::vector<ResultCoords>& result_coords) const;
 
   /**
    * Returns `true` if a coordinate buffer for a separate dimension
@@ -454,7 +649,7 @@ class Reader : public ReaderBase, public IQueryStrategy {
    * Erases the coordinate tiles (zipped or separate) from the input result
    * tiles.
    */
-  void erase_coord_tiles(std::vector<ResultTile>* result_tiles) const;
+  void erase_coord_tiles(std::vector<ResultTile>& result_tiles) const;
 
   /** Gets statistics about the result cells. */
   void get_result_cell_stats(
