@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,6 +36,8 @@
 #include <iostream>
 #include <vector>
 
+#include "tiledb/common/types/dynamic_typed_datum.h"
+#include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/query/result_tile.h"
 
 using namespace tiledb::common;
@@ -76,7 +78,7 @@ struct ResultCoords {
   }
 
   /** Moves to the next cell */
-  bool next() {
+  inline bool next() {
     if (pos_ == tile_->cell_num() - 1)
       return false;
 
@@ -85,12 +87,12 @@ struct ResultCoords {
   }
 
   /** Invalidate this instance. */
-  void invalidate() {
+  inline void invalidate() {
     valid_ = false;
   }
 
   /** Return true if this instance is valid. */
-  bool valid() const {
+  inline bool valid() const {
     return valid_;
   }
 
@@ -98,7 +100,7 @@ struct ResultCoords {
    * Returns a string coordinate. Applicable only to string
    * dimensions.
    */
-  std::string_view coord_string(unsigned dim_idx) const {
+  inline std::string_view coord_string(unsigned dim_idx) const {
     return tile_->coord_string(pos_, dim_idx);
   }
 
@@ -109,10 +111,20 @@ struct ResultCoords {
    * @param dim_idx The index of the dimension to retrieve the coordinate for.
    * @return A constant pointer to the requested coordinate.
    */
-  const void* coord(unsigned dim_idx) const {
+  inline const void* coord(unsigned dim_idx) const {
     return tile_->coord(pos_, dim_idx);
   }
 
+  inline UntypedDatumView dimension_datum(
+      const Dimension& dim, unsigned dim_idx) const {
+    if (dim.var_size()) {
+      auto x{tile_->coord_string(pos_, dim_idx)};
+      return tdb::UntypedDatumView{x.data(), x.size()};
+    } else {
+      return tdb::UntypedDatumView{coord(dim_idx), dim.coord_size()};
+    }
+  }
+
   /**
    * Returns true if the coordinates (at the current position) of the
    * calling ResultCoords object and the input are the same across all
