@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,6 +35,7 @@
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
+#include "tiledb/type/range/range.h"
 
 #include <cassert>
 #include <iostream>
@@ -42,6 +43,7 @@
 #include <numeric>
 
 using namespace tiledb::common;
+using namespace tiledb::type;
 
 namespace tiledb {
 namespace sm {
@@ -51,16 +53,15 @@ namespace sm {
 /* ****************************** */
 
 ResultTile::ResultTile(
-    unsigned frag_idx, uint64_t tile_idx, const ArraySchema* array_schema)
-    : frag_idx_(frag_idx)
+    unsigned frag_idx, uint64_t tile_idx, const ArraySchema& array_schema)
+    : domain_(&array_schema.domain())
+    , frag_idx_(frag_idx)
     , tile_idx_(tile_idx) {
-  assert(array_schema != nullptr);
-  domain_ = array_schema->domain();
   coord_tiles_.resize(domain_->dim_num());
-  attr_tiles_.resize(array_schema->attribute_num());
-  for (uint64_t i = 0; i < array_schema->attribute_num(); i++) {
-    const Attribute* attribute = array_schema->attribute(i);
-    attr_tiles_[i] = std::make_pair(attribute->name(), std::nullopt);
+  attr_tiles_.resize(array_schema.attribute_num());
+  for (uint64_t i = 0; i < array_schema.attribute_num(); i++) {
+    auto attribute = array_schema.attribute(i);
+    attr_tiles_[i] = std::make_pair(attribute->name(), nullopt);
   }
   set_compute_results_func();
 
@@ -69,6 +70,38 @@ ResultTile::ResultTile(
   coord_func_ = &ResultTile::zipped_coord;
 }
 
+/** Move constructor. */
+ResultTile::ResultTile(ResultTile&& other) {
+  // Swap with the argument
+  swap(other);
+}
+
+/** Move-assign operator. */
+ResultTile& ResultTile::operator=(ResultTile&& other) {
+  // Swap with the argument
+  swap(other);
+
+  return *this;
+}
+
+void ResultTile::swap(ResultTile& tile) {
+  std::swap(domain_, tile.domain_);
+  std::swap(frag_idx_, tile.frag_idx_);
+  std::swap(tile_idx_, tile.tile_idx_);
+  std::swap(attr_tiles_, tile.attr_tiles_);
+  std::swap(coords_tile_, tile.coords_tile_);
+  std::swap(coord_tiles_, tile.coord_tiles_);
+  std::swap(compute_results_dense_func_, tile.compute_results_dense_func_);
+  std::swap(coord_func_, tile.coord_func_);
+  std::swap(compute_results_sparse_func_, tile.compute_results_sparse_func_);
+  std::swap(
+      compute_results_count_sparse_uint64_t_func_,
+      tile.compute_results_count_sparse_uint64_t_func_);
+  std::swap(
+      compute_results_count_sparse_uint8_t_func_,
+      tile.compute_results_count_sparse_uint8_t_func_);
+}
+
 /* ****************************** */
 /*               API              */
 /* ****************************** */
@@ -95,10 +128,6 @@ uint64_t ResultTile::cell_num() const {
   return 0;
 }
 
-const Domain* ResultTile::domain() const {
-  return domain_;
-}
-
 void ResultTile::erase_tile(const std::string& name) {
   // Handle zipped coordinates tiles
   if (name == constants::coords) {
@@ -130,7 +159,7 @@ void ResultTile::init_attr_tile(const std::string& name) {
 
   // Handle attributes
   for (auto& at : attr_tiles_) {
-    if (at.first == name && at.second == std::nullopt) {
+    if (at.first == name && at.second == nullopt) {
       at.second = TileTuple(Tile(), Tile(), Tile());
       return;
     }
@@ -169,17 +198,17 @@ ResultTile::TileTuple* ResultTile::tile_tuple(const std::string& name) {
 const void* ResultTile::unzipped_coord(uint64_t pos, unsigned dim_idx) const {
   const auto& coord_tile = std::get<0>(coord_tiles_[dim_idx].second);
   const uint64_t offset = pos * coord_tile.cell_size();
-  Buffer* const buffer = coord_tile.buffer();
-  void* const ret = static_cast<char*>(buffer->data()) + offset;
+  void* const ret = static_cast<char*>(coord_tile.data()) + offset;
   return ret;
 }
 
 const void* ResultTile::zipped_coord(uint64_t pos, unsigned dim_idx) const {
   auto coords_size = std::get<0>(coords_tile_).cell_size();
-  auto coord_size = coords_size / std::get<0>(coords_tile_).dim_num();
+  auto coord_size =
+      coords_size / std::get<0>(coords_tile_).zipped_coords_dim_num();
   const uint64_t offset = pos * coords_size + dim_idx * coord_size;
-  Buffer* const buffer = std::get<0>(coords_tile_).buffer();
-  void* const ret = static_cast<char*>(buffer->data()) + offset;
+  void* const ret =
+      static_cast<char*>(std::get<0>(coords_tile_).data()) + offset;
   return ret;
 }
 
@@ -191,22 +220,22 @@ std::string_view ResultTile::coord_string(
   auto val_size = coord_tile_val.size();
 
   uint64_t offset = 0;
-  Status st = coord_tile_off.buffer()->read(
-      &offset, pos * sizeof(uint64_t), sizeof(uint64_t));
+  Status st =
+      coord_tile_off.read(&offset, pos * sizeof(uint64_t), sizeof(uint64_t));
   assert(st.ok());
 
   uint64_t next_offset = 0;
   if (pos == cell_num - 1) {
     next_offset = val_size;
   } else {
-    st = coord_tile_off.buffer()->read(
+    st = coord_tile_off.read(
         &next_offset, (pos + 1) * sizeof(uint64_t), sizeof(uint64_t));
     assert(st.ok());
   }
 
   auto size = next_offset - offset;
 
-  auto* buffer = static_cast<char*>(coord_tile_val.buffer()->data()) + offset;
+  auto* buffer = static_cast<char*>(coord_tile_val.data()) + offset;
   return std::string_view(buffer, size);
 }
 
@@ -214,7 +243,7 @@ uint64_t ResultTile::coord_size(unsigned dim_idx) const {
   // Handle zipped coordinate tiles
   if (!std::get<0>(coords_tile_).empty())
     return std::get<0>(coords_tile_).cell_size() /
-           std::get<0>(coords_tile_).dim_num();
+           std::get<0>(coords_tile_).zipped_coords_dim_num();
 
   // Handle separate coordinate tiles
   assert(dim_idx < coord_tiles_.size());
@@ -266,7 +295,7 @@ Status ResultTile::read(
     auto cell_size = tile.cell_size();
     auto nbytes = len * cell_size;
     auto offset = pos * cell_size;
-    return tile.read(buffer, nbytes, offset);
+    return tile.read(buffer, offset, nbytes);
   } else if (
       name == constants::coords && !coord_tiles_[0].first.empty() &&
       std::get<0>(coords_tile_).empty()) {
@@ -278,11 +307,11 @@ Status ResultTile::read(
     auto buff_offset = 0;
     for (uint64_t c = 0; c < len; ++c) {
       for (unsigned d = 0; d < dim_num; ++d) {
-        auto coord_tile = std::get<0>(coord_tiles_[d].second);
+        auto& coord_tile = std::get<0>(coord_tiles_[d].second);
         auto cell_size = coord_tile.cell_size();
         auto tile_offset = (pos + c) * cell_size;
         RETURN_NOT_OK(
-            coord_tile.read(buff + buff_offset, cell_size, tile_offset));
+            coord_tile.read(buff + buff_offset, tile_offset, cell_size));
         buff_offset += cell_size;
       }
     }
@@ -304,7 +333,7 @@ Status ResultTile::read(
     uint64_t offset = pos * cell_size + dim_size * dim_offset;
     for (uint64_t c = 0; c < len; ++c) {
       RETURN_NOT_OK(std::get<0>(coords_tile_)
-                        .read(buff + (c * dim_size), dim_size, offset));
+                        .read(buff + (c * dim_size), offset, dim_size));
       offset += cell_size;
     }
   };
@@ -334,9 +363,9 @@ Status ResultTile::read_nullable(
   auto validity_nbytes = len * validity_cell_size;
   auto validity_offset = pos * validity_cell_size;
 
-  RETURN_NOT_OK(tile.read(buffer, nbytes, offset));
+  RETURN_NOT_OK(tile.read(buffer, offset, nbytes));
   RETURN_NOT_OK(
-      tile_validity.read(buffer_validity, validity_nbytes, validity_offset));
+      tile_validity.read(buffer_validity, validity_offset, validity_nbytes));
 
   return Status::Ok();
 }
@@ -361,7 +390,7 @@ void ResultTile::compute_results_dense(
     const ResultTile* result_tile,
     unsigned dim_idx,
     const Range& range,
-    const std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata,
+    const std::vector<shared_ptr<FragmentMetadata>> fragment_metadata,
     unsigned frag_idx,
     std::vector<uint8_t>* result_bitmap,
     std::vector<uint8_t>* overwritten_bitmap) {
@@ -377,8 +406,7 @@ void ResultTile::compute_results_dense(
   if (!stores_zipped_coords) {
     const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
 
-    Buffer* const buffer = coord_tile.buffer();
-    auto coords = (const T*)buffer->data();
+    auto coords = (const T*)coord_tile.data();
     T c;
 
     if (dim_idx == dim_num - 1) {
@@ -397,8 +425,8 @@ void ResultTile::compute_results_dense(
               for (unsigned d = 0; d < dim_num; ++d) {
                 const auto& coord_tile =
                     std::get<0>(result_tile->coord_tile(dim_idx));
-                Buffer* const buffer = coord_tile.buffer();
-                const T* const coords = static_cast<const T*>(buffer->data());
+                const T* const coords =
+                    static_cast<const T*>(coord_tile.data());
                 auto c_d = coords[pos];
                 auto dom = (const T*)meta->non_empty_domain()[d].data();
                 if (c_d < dom[0] || c_d > dom[1]) {
@@ -424,8 +452,7 @@ void ResultTile::compute_results_dense(
   // Handle zipped coordinates tile
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  Buffer* const buffer = coords_tile.buffer();
-  auto coords = (const T*)buffer->data();
+  auto coords = (const T*)coords_tile.data();
   T c;
 
   if (dim_idx == dim_num - 1) {
@@ -495,11 +522,11 @@ void ResultTile::compute_results_sparse<char>(
 
   // Get offset buffer
   const auto& coord_tile_off = std::get<0>(coord_tile);
-  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.buffer()->data());
+  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
 
   // Get string buffer
   const auto& coord_tile_str = std::get<1>(coord_tile);
-  auto buff_str = static_cast<const char*>(coord_tile_str.buffer()->data());
+  auto buff_str = static_cast<const char*>(coord_tile_str.data());
   auto buff_str_size = coord_tile_str.size();
 
   // For row-major cell orders, the first dimension is sorted.
@@ -603,7 +630,15 @@ void ResultTile::compute_results_sparse<char>(
         (i < coords_num - zeroed_size) ? zeroed_size : coords_num - i;
 
     // Check if all `r_bitmap` values are zero between `i` and
-    // `partition_size`.
+    // `partition_size`. To do so, first make sure the first byte is 0, then
+    // using memcmp(addr, addr + 1, size - 1), to validate the rest. This works
+    // as memcmp will validate that the current byte is equal to the next for
+    // all values of the memory buffer in order, and we start by knowing that
+    // the first byte is equal to 0. Through profiling, this has proven to be
+    // much faster than a solution using std::all_of for example, so even
+    // though this solution is less readable, it is still the best. Note that
+    // for this to work, we must make sure that the second address is exactly
+    // one byte after the first one.
     if (r_bitmap[i] == 0 &&
         memcmp(&r_bitmap[i], &r_bitmap[i + 1], partition_size - 1) == 0)
       continue;
@@ -650,8 +685,7 @@ void ResultTile::compute_results_sparse(
   // Handle separate coordinate tiles
   if (!stores_zipped_coords) {
     const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
-    Buffer* const buffer = coord_tile.buffer();
-    const T* const coords = static_cast<const T*>(buffer->data());
+    const T* const coords = static_cast<const T*>(coord_tile.data());
     for (uint64_t pos = 0; pos < coords_num; ++pos) {
       c = coords[pos];
       r_bitmap[pos] &= (uint8_t)(c >= r0 && c <= r1);
@@ -663,8 +697,7 @@ void ResultTile::compute_results_sparse(
   // Handle zipped coordinates tile
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  Buffer* const buffer = coords_tile.buffer();
-  const T* const coords = static_cast<const T*>(buffer->data());
+  const T* const coords = static_cast<const T*>(coords_tile.data());
   for (uint64_t pos = 0; pos < coords_num; ++pos) {
     c = coords[pos * dim_num + dim_idx];
     r_bitmap[pos] &= (uint8_t)(c >= r0 && c <= r1);
@@ -695,11 +728,11 @@ void ResultTile::compute_results_count_sparse_string(
 
   // Get offset buffer
   const auto& coord_tile_off = std::get<0>(coord_tile);
-  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.buffer()->data());
+  auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
 
   // Get string buffer
   const auto& coord_tile_str = std::get<1>(coord_tile);
-  auto buff_str = static_cast<const char*>(coord_tile_str.buffer()->data());
+  auto buff_str = static_cast<const char*>(coord_tile_str.data());
   auto buff_str_size = coord_tile_str.size();
 
   // For row-major cell orders, the first dimension is sorted.
@@ -817,15 +850,23 @@ void ResultTile::compute_results_count_sparse_string(
   const uint64_t zeroed_size = coords_num < 256 ? coords_num : 256;
   for (uint64_t i = min_cell; i < min_cell + coords_num; i += zeroed_size) {
     const uint64_t partition_size =
-        (i < cell_num - zeroed_size) ? zeroed_size : cell_num - i;
+        (i < max_cell - zeroed_size) ? zeroed_size : max_cell - i;
 
-    // Check if all `r_count` values are zero between `i` and
-    // `partition_size`.
+    // Check if all `r_bitmap` values are zero between `i` and
+    // `partition_size`. To do so, first make sure the first byte is 0, then
+    // using memcmp(addr, addr + 1, size - 1), to validate the rest. This works
+    // as memcmp will validate that the current byte is equal to the next for
+    // all values of the memory buffer in order, and we start by knowing that
+    // the first byte is equal to 0. Through profiling, this has proven to be
+    // much faster than a solution using std::all_of for example, so even
+    // though this solution is less readable, it is still the best. Note that
+    // for this to work, we must make sure that the second address is exactly
+    // one byte after the first one.
     if (result_count[i] == 0 &&
         memcmp(
             &result_count[i],
-            &result_count[i + 1],
-            (partition_size - 1) * sizeof(uint64_t)) == 0)
+            reinterpret_cast<uint8_t*>(&result_count[i]) + 1,
+            partition_size * sizeof(BitmapType) - 1) == 0)
       continue;
 
     {
@@ -873,8 +914,7 @@ void ResultTile::compute_results_count_sparse(
   // Handle separate coordinate tiles.
   if (!stores_zipped_coords) {
     const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
-    Buffer* const buffer = coord_tile.buffer();
-    const T* const coords = static_cast<const T*>(buffer->data());
+    const T* const coords = static_cast<const T*>(coord_tile.data());
     {
       // Iterate over all cells.
       for (uint64_t pos = min_cell; pos < max_cell; ++pos) {
@@ -934,8 +974,7 @@ void ResultTile::compute_results_count_sparse(
   // Handle zipped coordinates tile.
   assert(stores_zipped_coords);
   const auto& coords_tile = result_tile->zipped_coords_tile();
-  Buffer* const buffer = coords_tile.buffer();
-  const T* const coords = static_cast<const T*>(buffer->data());
+  const T* const coords = static_cast<const T*>(coords_tile.data());
   {
     for (uint64_t pos = min_cell; pos < max_cell; ++pos) {
       if (result_count[pos]) {
@@ -956,7 +995,7 @@ void ResultTile::compute_results_count_sparse(
 Status ResultTile::compute_results_dense(
     unsigned dim_idx,
     const Range& range,
-    const std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata,
+    const std::vector<shared_ptr<FragmentMetadata>> fragment_metadata,
     unsigned frag_idx,
     std::vector<uint8_t>* result_bitmap,
     std::vector<uint8_t>* overwritten_bitmap) const {
