@@ -38,6 +38,7 @@
 #include <unordered_map>
 #include <vector>
 
+#include "tiledb/common/common.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/misc/constants.h"
@@ -46,6 +47,11 @@
 #include "tiledb/sm/tile/tile.h"
 
 using namespace tiledb::common;
+using namespace tiledb::type;
+
+namespace tiledb::type {
+class Range;
+}
 
 namespace tiledb {
 namespace sm {
@@ -68,7 +74,7 @@ class ResultTile {
    * tile is the var-sized values tile. If the attribute is nullable, the third
    * tile contains the validity vector.
    */
-  typedef std::tuple<Tile, Tile, Tile> TileTuple;
+  typedef tuple<Tile, Tile, Tile> TileTuple;
 
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
@@ -81,27 +87,26 @@ class ResultTile {
    * Constructor. The number of dimensions `dim_num` is used to allocate
    * the separate coordinate tiles.
    */
-  ResultTile(unsigned frag_idx, uint64_t tile_idx, const ArraySchema* schema);
+  ResultTile(unsigned frag_idx, uint64_t tile_idx, const ArraySchema& schema);
+
+  DISABLE_COPY_AND_COPY_ASSIGN(ResultTile);
 
   /** Default destructor. */
   ~ResultTile() = default;
 
-  /** Default copy constructor. */
-  ResultTile(const ResultTile&) = default;
+  /** Move constructor. */
+  ResultTile(ResultTile&& tile);
 
-  /** Default move constructor. */
-  ResultTile(ResultTile&&) = default;
+  /** Move-assign operator. */
+  ResultTile& operator=(ResultTile&& tile);
+
+  /** Swaps the contents (all field values) of this tile with the given tile. */
+  void swap(ResultTile& tile);
 
   /* ********************************* */
   /*                API                */
   /* ********************************* */
 
-  /** Default copy-assign operator. */
-  ResultTile& operator=(const ResultTile&) = default;
-
-  /** Default move-assign operator. */
-  ResultTile& operator=(ResultTile&&) = default;
-
   /** Equality operator (mainly for debugging purposes). */
   bool operator==(const ResultTile& rt) const;
 
@@ -121,7 +126,9 @@ class ResultTile {
   const TileTuple& coord_tile(unsigned dim_idx) const;
 
   /** Returns the stored domain. */
-  const Domain* domain() const;
+  const Domain* domain() const {
+    return domain_;
+  }
 
   /** Erases the tile for the input attribute/dimension. */
   void erase_tile(const std::string& name);
@@ -207,7 +214,7 @@ class ResultTile {
       const ResultTile* result_tile,
       unsigned dim_idx,
       const Range& range,
-      const std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata,
+      const std::vector<shared_ptr<FragmentMetadata>> fragment_metadata,
       unsigned frag_idx,
       std::vector<uint8_t>* result_bitmap,
       std::vector<uint8_t>* overwritten_bitmap);
@@ -286,7 +293,7 @@ class ResultTile {
   Status compute_results_dense(
       unsigned dim_idx,
       const Range& range,
-      const std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata,
+      const std::vector<shared_ptr<FragmentMetadata>> fragment_metadata,
       unsigned frag_idx,
       std::vector<uint8_t>* result_bitmap,
       std::vector<uint8_t>* overwritten_bitmap) const;
@@ -340,7 +347,7 @@ class ResultTile {
   uint64_t tile_idx_ = UINT64_MAX;
 
   /** Attribute names to tiles based on attribute ordering from array schema. */
-  std::vector<std::pair<std::string, std::optional<TileTuple>>> attr_tiles_;
+  std::vector<std::pair<std::string, optional<TileTuple>>> attr_tiles_;
 
   /** The zipped coordinates tile. */
   TileTuple coords_tile_;
@@ -359,7 +366,7 @@ class ResultTile {
       const ResultTile*,
       unsigned,
       const Range&,
-      const std::vector<tdb_shared_ptr<FragmentMetadata>>,
+      const std::vector<shared_ptr<FragmentMetadata>>,
       unsigned,
       std::vector<uint8_t>*,
       std::vector<uint8_t>*)>>
