@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 
 #include "tiledb/sm/query/sparse_index_reader_base.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/filesystem/vfs.h"
@@ -39,9 +40,9 @@
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/resource_pool.h"
 #include "tiledb/sm/query/iquery_strategy.h"
+#include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/query/strategy_base.h"
-#include "tiledb/sm/storage_manager/open_array_memory_tracker.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 #include <numeric>
@@ -55,7 +56,7 @@ namespace sm {
 
 SparseIndexReaderBase::SparseIndexReaderBase(
     stats::Stats* stats,
-    tdb_shared_ptr<Logger> logger,
+    shared_ptr<Logger> logger,
     StorageManager* storage_manager,
     Array* array,
     Config& config,
@@ -75,7 +76,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
           condition)
     , initial_data_loaded_(false)
     , memory_budget_(0)
-    , array_memory_tracker_(nullptr)
+    , array_memory_tracker_(array->memory_tracker())
     , memory_used_for_coords_total_(0)
     , memory_used_qc_tiles_total_(0)
     , memory_used_result_tile_ranges_(0)
@@ -83,7 +84,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
     , memory_budget_ratio_query_condition_(0.25)
     , memory_budget_ratio_tile_ranges_(0.1)
     , memory_budget_ratio_array_data_(0.1)
-    , coords_loaded_(true) {
+    , buffers_full_(false) {
   read_state_.done_adding_result_tiles_ = false;
 }
 
@@ -103,14 +104,11 @@ typename SparseIndexReaderBase::ReadState* SparseIndexReaderBase::read_state() {
 Status SparseIndexReaderBase::init() {
   // Sanity checks
   if (storage_manager_ == nullptr)
-    return logger_->status(Status::ReaderError(
+    return logger_->status(Status_ReaderError(
         "Cannot initialize sparse global order reader; Storage manager not "
         "set"));
-  if (array_schema_ == nullptr)
-    return logger_->status(Status::ReaderError(
-        "Cannot initialize sparse global order reader; Array schema not set"));
   if (buffers_.empty())
-    return logger_->status(Status::ReaderError(
+    return logger_->status(Status_ReaderError(
         "Cannot initialize sparse global order reader; Buffers not set"));
 
   // Check subarray
@@ -122,8 +120,8 @@ Status SparseIndexReaderBase::init() {
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
     return logger_->status(
-        Status::ReaderError("Cannot initialize reader; Unsupported offsets "
-                            "format in configuration"));
+        Status_ReaderError("Cannot initialize reader; Unsupported offsets "
+                           "format in configuration"));
   }
   elements_mode_ = offsets_format_mode_ == "elements";
 
@@ -134,8 +132,8 @@ Status SparseIndexReaderBase::init() {
       "sm.var_offsets.bitsize", &offsets_bitsize_, &found));
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
     return logger_->status(
-        Status::ReaderError("Cannot initialize reader; "
-                            "Unsupported offsets bitsize in configuration"));
+        Status_ReaderError("Cannot initialize reader; "
+                           "Unsupported offsets bitsize in configuration"));
   }
 
   // Check the validity buffer sizes.
@@ -144,49 +142,59 @@ Status SparseIndexReaderBase::init() {
   return Status::Ok();
 }
 
+uint64_t SparseIndexReaderBase::cells_copied(
+    const std::vector<std::string>& names) {
+  auto& last_name = names.back();
+  auto buffer_size = *buffers_[last_name].buffer_size_;
+  if (array_schema_.var_size(last_name)) {
+    if (buffer_size == 0)
+      return 0;
+    else
+      return buffer_size / (offsets_bitsize_ / 8) - offsets_extra_element_;
+  } else {
+    return buffer_size / array_schema_.cell_size(last_name);
+  }
+}
+
 template <class BitmapType>
-Status SparseIndexReaderBase::get_coord_tiles_size(
-    bool include_coords,
-    unsigned dim_num,
-    unsigned f,
-    uint64_t t,
-    uint64_t* tiles_size,
-    uint64_t* tiles_size_qc) {
-  *tiles_size = 0;
+tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+SparseIndexReaderBase::get_coord_tiles_size(
+    bool include_coords, unsigned dim_num, unsigned f, uint64_t t) {
+  uint64_t tiles_size = 0;
 
   // Add the coordinate tiles size.
   if (include_coords) {
     for (unsigned d = 0; d < dim_num; d++) {
-      *tiles_size += fragment_metadata_[f]->tile_size(dim_names_[d], t);
+      tiles_size += fragment_metadata_[f]->tile_size(dim_names_[d], t);
 
       if (is_dim_var_size_[d]) {
-        uint64_t temp = 0;
-        RETURN_NOT_OK(
-            fragment_metadata_[f]->tile_var_size(dim_names_[d], t, &temp));
-        *tiles_size += temp;
+        auto&& [st, temp] =
+            fragment_metadata_[f]->tile_var_size(dim_names_[d], t);
+        RETURN_NOT_OK_TUPLE(st, nullopt);
+        tiles_size += *temp;
       }
     }
   }
 
   // Add the result tile structure size.
-  *tiles_size += sizeof(ResultTileWithBitmap<BitmapType>);
+  tiles_size += sizeof(ResultTileWithBitmap<BitmapType>);
 
   // Add the tile bitmap size if there is a subarray.
   if (subarray_.is_set())
-    *tiles_size += fragment_metadata_[f]->cell_num(t) * sizeof(BitmapType);
+    tiles_size += fragment_metadata_[f]->cell_num(t) * sizeof(BitmapType);
 
   // Compute query condition tile sizes.
-  *tiles_size_qc = 0;
+  uint64_t tiles_size_qc = 0;
   if (!qc_loaded_names_.empty()) {
     for (auto& name : qc_loaded_names_) {
       // Calculate memory consumption for this tile.
-      uint64_t tile_size = 0;
-      RETURN_NOT_OK(get_attribute_tile_size(name, f, t, &tile_size));
-      *tiles_size_qc += tile_size;
+      auto&& [st, tile_size] = get_attribute_tile_size(name, f, t);
+      RETURN_NOT_OK_TUPLE(st, nullopt);
+      tiles_size_qc += *tile_size;
     }
   }
 
-  return Status::Ok();
+  return {Status::Ok(), std::make_pair(tiles_size, tiles_size_qc)};
 }
 
 Status SparseIndexReaderBase::load_initial_data() {
@@ -209,8 +217,6 @@ Status SparseIndexReaderBase::load_initial_data() {
   auto fragment_num = fragment_metadata_.size();
 
   // Make sure there is enough space for tiles data.
-  read_state_.frag_tile_idx_.clear();
-  all_tiles_loaded_.clear();
   read_state_.frag_tile_idx_.resize(fragment_num);
   all_tiles_loaded_.resize(fragment_num);
 
@@ -226,7 +232,9 @@ Status SparseIndexReaderBase::load_initial_data() {
     // This is ok as it is a soft limit and will be taken into consideration
     // later.
     RETURN_NOT_OK(subarray_.precompute_all_ranges_tile_overlap(
-        storage_manager_->compute_tp(), &result_tile_ranges_));
+        storage_manager_->compute_tp(),
+        read_state_.frag_tile_idx_,
+        &result_tile_ranges_));
 
     for (auto frag_result_tile_ranges : result_tile_ranges_) {
       memory_used_result_tile_ranges_ += frag_result_tile_ranges.size() *
@@ -236,7 +244,7 @@ Status SparseIndexReaderBase::load_initial_data() {
     if (memory_used_result_tile_ranges_ >
         memory_budget_ratio_tile_ranges_ * memory_budget_)
       return logger_->status(
-          Status::ReaderError("Exceeded memory budget for result tile ranges"));
+          Status_ReaderError("Exceeded memory budget for result tile ranges"));
   }
 
   // Set a limit to the array memory.
@@ -246,39 +254,39 @@ Status SparseIndexReaderBase::load_initial_data() {
   // Preload zipped coordinate tile offsets. Note that this will
   // ignore fragments with a version >= 5.
   std::vector<std::string> zipped_coords_names = {constants::coords};
-  RETURN_CANCEL_OR_ERROR(load_tile_offsets(&subarray_, &zipped_coords_names));
+  RETURN_CANCEL_OR_ERROR(load_tile_offsets(subarray_, zipped_coords_names));
 
   // Preload unzipped coordinate tile offsets. Note that this will
   // ignore fragments with a version < 5.
-  const auto dim_num = array_schema_->dim_num();
+  const auto dim_num = array_schema_.dim_num();
   dim_names_.reserve(dim_num);
   is_dim_var_size_.reserve(dim_num);
   std::vector<std::string> var_size_to_load;
   for (unsigned d = 0; d < dim_num; ++d) {
-    dim_names_.emplace_back(array_schema_->dimension(d)->name());
-    is_dim_var_size_[d] = array_schema_->var_size(dim_names_[d]);
+    dim_names_.emplace_back(array_schema_.dimension(d)->name());
+    is_dim_var_size_.emplace_back(array_schema_.var_size(dim_names_[d]));
     if (is_dim_var_size_[d])
       var_size_to_load.emplace_back(dim_names_[d]);
   }
-  RETURN_CANCEL_OR_ERROR(load_tile_offsets(&subarray_, &dim_names_));
+  RETURN_CANCEL_OR_ERROR(load_tile_offsets(subarray_, dim_names_));
 
   // Compute tile offsets to load and var size to load for attributes.
   std::vector<std::string> attr_tile_offsets_to_load;
   for (auto& it : buffers_) {
     const auto& name = it.first;
-    if (array_schema_->is_dim(name))
+    if (array_schema_.is_dim(name))
       continue;
 
     attr_tile_offsets_to_load.emplace_back(name);
 
-    if (array_schema_->var_size(name))
+    if (array_schema_.var_size(name))
       var_size_to_load.emplace_back(name);
   }
 
   // Load tile offsets and var sizes for attributes.
-  RETURN_CANCEL_OR_ERROR(load_tile_var_sizes(&subarray_, &var_size_to_load));
+  RETURN_CANCEL_OR_ERROR(load_tile_var_sizes(subarray_, var_size_to_load));
   RETURN_CANCEL_OR_ERROR(
-      load_tile_offsets(&subarray_, &attr_tile_offsets_to_load));
+      load_tile_offsets(subarray_, attr_tile_offsets_to_load));
 
   logger_->debug("Initial data loaded");
   initial_data_loaded_ = true;
@@ -286,7 +294,7 @@ Status SparseIndexReaderBase::load_initial_data() {
 }
 
 Status SparseIndexReaderBase::read_and_unfilter_coords(
-    bool include_coords, const std::vector<ResultTile*>* result_tiles) {
+    bool include_coords, const std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("read_and_unfilter_coords");
 
   // Not including coords or no query condition, exit.
@@ -298,14 +306,14 @@ Status SparseIndexReaderBase::read_and_unfilter_coords(
     // this will ignore fragments with a version >= 5.
     std::vector<std::string> zipped_coords_names = {constants::coords};
     RETURN_CANCEL_OR_ERROR(
-        read_coordinate_tiles(&zipped_coords_names, result_tiles, true));
+        read_coordinate_tiles(zipped_coords_names, result_tiles, true));
     RETURN_CANCEL_OR_ERROR(
         unfilter_tiles(constants::coords, result_tiles, true));
 
     // Read and unfilter unzipped coordinate tiles. Note that
     // this will ignore fragments with a version < 5.
     RETURN_CANCEL_OR_ERROR(
-        read_coordinate_tiles(&dim_names_, result_tiles, true));
+        read_coordinate_tiles(dim_names_, result_tiles, true));
     for (const auto& dim_name : dim_names_) {
       RETURN_CANCEL_OR_ERROR(unfilter_tiles(dim_name, result_tiles, true));
     }
@@ -314,7 +322,7 @@ Status SparseIndexReaderBase::read_and_unfilter_coords(
   if (!condition_.empty()) {
     // Read and unfilter tiles for querty condition.
     RETURN_CANCEL_OR_ERROR(
-        read_attribute_tiles(&qc_loaded_names_, result_tiles, true));
+        read_attribute_tiles(qc_loaded_names_, result_tiles, true));
 
     for (const auto& name : qc_loaded_names_) {
       RETURN_CANCEL_OR_ERROR(unfilter_tiles(name, result_tiles, true));
@@ -341,13 +349,13 @@ Status SparseIndexReaderBase::allocate_tile_bitmap(
 
 template <class BitmapType>
 Status SparseIndexReaderBase::compute_tile_bitmaps(
-    std::vector<ResultTile*>* result_tiles) {
+    std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("compute_tile_bitmaps");
 
   // For easy reference.
-  const auto domain = array_schema_->domain();
-  const auto dim_num = array_schema_->dim_num();
-  const auto cell_order = array_schema_->cell_order();
+  const auto& domain{array_schema_.domain()};
+  const auto dim_num = array_schema_.dim_num();
+  const auto cell_order = array_schema_.cell_order();
 
   // No subarray set, return.
   if (!subarray_.is_set()) {
@@ -357,9 +365,9 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
   const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
-  if (result_tiles->size() < num_threads) {
+  if (result_tiles.size() < num_threads) {
     // Ceil the division between thread_num and tile_num.
-    num_range_threads = 1 + ((num_threads - 1) / result_tiles->size());
+    num_range_threads = 1 + ((num_threads - 1) / result_tiles.size());
   }
 
   // Perforance runs have shown that running multiple parallel_for's has a
@@ -370,10 +378,10 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
     auto status = parallel_for(
         storage_manager_->compute_tp(),
         0,
-        result_tiles->size(),
+        result_tiles.size(),
         [&](uint64_t t) {
           return allocate_tile_bitmap(
-              (ResultTileWithBitmap<BitmapType>*)result_tiles->at(t));
+              (ResultTileWithBitmap<BitmapType>*)result_tiles[t]);
         });
     RETURN_NOT_OK_ELSE(status, logger_->status(status));
   }
@@ -382,12 +390,12 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
   auto status = parallel_for_2d(
       storage_manager_->compute_tp(),
       0,
-      result_tiles->size(),
+      result_tiles.size(),
       0,
       num_range_threads,
       [&](uint64_t t, uint64_t range_thread_idx) {
         // For easy reference.
-        auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles->at(t);
+        auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles[t];
         auto cell_num =
             fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
 
@@ -425,17 +433,16 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
           // Compute the list of range index to process.
           std::vector<uint64_t> relevant_ranges;
           relevant_ranges.reserve(ranges_for_dim.size());
-          domain->dimension(dim_idx)->relevant_ranges(
+          domain.dimension(dim_idx)->relevant_ranges(
               ranges_for_dim, mbr[dim_idx], relevant_ranges);
 
           // For non overlapping ranges, if we have full overlap on any range
           // there is no need to compute bitmaps.
           const bool is_uint8_t = std::is_same<BitmapType, uint8_t>::value;
           if (is_uint8_t) {
-            std::vector<bool> covered_bitmap;
-            covered_bitmap.reserve(relevant_ranges.size());
-            domain->dimension(dim_idx)->covered_vec(
-                ranges_for_dim, mbr[dim_idx], relevant_ranges, covered_bitmap);
+            std::vector<bool> covered_bitmap =
+                domain.dimension(dim_idx)->covered_vec(
+                    ranges_for_dim, mbr[dim_idx], relevant_ranges);
 
             // See if any range is covered.
             uint64_t count = std::accumulate(
@@ -483,10 +490,10 @@ Status SparseIndexReaderBase::compute_tile_bitmaps(
     status = parallel_for(
         storage_manager_->compute_tp(),
         0,
-        result_tiles->size(),
+        result_tiles.size(),
         [&](uint64_t t) {
           return count_tile_bitmap_cells(
-              (ResultTileWithBitmap<BitmapType>*)result_tiles->at(t));
+              (ResultTileWithBitmap<BitmapType>*)result_tiles[t]);
         });
     RETURN_NOT_OK_ELSE(status, logger_->status(status));
   }
@@ -522,7 +529,7 @@ Status SparseIndexReaderBase::count_tile_bitmap_cells(
 
 template <class BitmapType>
 Status SparseIndexReaderBase::apply_query_condition(
-    std::vector<ResultTile*>* result_tiles) {
+    std::vector<ResultTile*>& result_tiles) {
   auto timer_se = stats_->start_timer("apply_query_condition");
 
   if (!condition_.empty()) {
@@ -530,10 +537,10 @@ Status SparseIndexReaderBase::apply_query_condition(
     auto status = parallel_for(
         storage_manager_->compute_tp(),
         0,
-        result_tiles->size(),
+        result_tiles.size(),
         [&](uint64_t t) {
           // For easy reference.
-          auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles->at(t);
+          auto rt = (ResultTileWithBitmap<BitmapType>*)result_tiles[t];
 
           auto cell_num =
               fragment_metadata_[rt->frag_idx()]->cell_num(rt->tile_idx());
@@ -543,23 +550,19 @@ Status SparseIndexReaderBase::apply_query_condition(
             rt->bitmap_result_num_ = cell_num;
           }
 
-          // Don't process already processed tiles.
-          if (!rt->qc_processed_) {
-            // Full overlap in bitmap calculation, make a bitmap.
-            if (rt->bitmap_.size() == 0) {
-              rt->bitmap_.resize(cell_num, 1);
-              rt->bitmap_result_num_ = cell_num;
-            }
-
-            // Compute the result of the query condition for this tile.
-            RETURN_NOT_OK(condition_.apply_sparse<BitmapType>(
-                fragment_metadata_[rt->frag_idx()]->array_schema(),
-                *rt,
-                rt->bitmap_,
-                &rt->bitmap_result_num_));
-            rt->qc_processed_ = true;
+          // Full overlap in bitmap calculation, make a bitmap.
+          if (rt->bitmap_.size() == 0) {
+            rt->bitmap_.resize(cell_num, 1);
+            rt->bitmap_result_num_ = cell_num;
           }
 
+          // Compute the result of the query condition for this tile.
+          RETURN_NOT_OK(condition_.apply_sparse<BitmapType>(
+              *(fragment_metadata_[rt->frag_idx()]->array_schema().get()),
+              *rt,
+              rt->bitmap_,
+              &rt->bitmap_result_num_));
+
           return Status::Ok();
         });
     RETURN_NOT_OK_ELSE(status, logger_->status(status));
@@ -569,6 +572,45 @@ Status SparseIndexReaderBase::apply_query_condition(
   return Status::Ok();
 }
 
+tuple<Status, optional<std::vector<uint64_t>>>
+SparseIndexReaderBase::read_and_unfilter_attributes(
+    const uint64_t memory_budget,
+    const std::vector<std::string>& names,
+    const std::vector<uint64_t>& mem_usage_per_attr,
+    uint64_t* buffer_idx,
+    std::vector<ResultTile*>& result_tiles) {
+  auto timer_se = stats_->start_timer("read_and_unfilter_attributes");
+
+  std::vector<std::string> names_to_read;
+  std::vector<uint64_t> index_to_copy;
+  uint64_t memory_used = 0;
+  while (*buffer_idx < names.size()) {
+    auto& name = names[*buffer_idx];
+    auto attr_mem_usage = mem_usage_per_attr[*buffer_idx];
+    if (memory_used + attr_mem_usage < memory_budget) {
+      memory_used += attr_mem_usage;
+
+      // We only read attributes, so dimensions have 0 cost.
+      if (attr_mem_usage != 0)
+        names_to_read.emplace_back(name);
+
+      index_to_copy.emplace_back(*buffer_idx);
+      (*buffer_idx)++;
+    } else {
+      break;
+    }
+  }
+
+  // Read and unfilter tiles.
+  RETURN_NOT_OK_TUPLE(
+      read_attribute_tiles(names_to_read, result_tiles, true), nullopt);
+
+  for (auto& name : names_to_read)
+    RETURN_NOT_OK_TUPLE(unfilter_tiles(name, result_tiles, true), nullopt);
+
+  return {Status::Ok(), std::move(index_to_copy)};
+}
+
 Status SparseIndexReaderBase::resize_output_buffers(uint64_t cells_copied) {
   // Resize buffers if the result cell slabs was truncated.
   for (auto& it : buffers_) {
@@ -576,7 +618,7 @@ Status SparseIndexReaderBase::resize_output_buffers(uint64_t cells_copied) {
     const auto size = *it.second.buffer_size_;
     uint64_t num_cells = 0;
 
-    if (array_schema_->var_size(name)) {
+    if (array_schema_.var_size(name)) {
       // Get the current number of cells from the offsets buffer.
       num_cells = size / constants::cell_var_offset_size;
 
@@ -595,19 +637,19 @@ Status SparseIndexReaderBase::resize_output_buffers(uint64_t cells_copied) {
         // loaded, use it.
         if (offsets_bitsize_ == 64) {
           uint64_t offset_div =
-              elements_mode_ ? datatype_size(array_schema_->type(name)) : 1;
+              elements_mode_ ? datatype_size(array_schema_.type(name)) : 1;
           *it.second.buffer_var_size_ =
               ((uint64_t*)it.second.buffer_)[cells_copied] * offset_div;
         } else {
           uint32_t offset_div =
-              elements_mode_ ? datatype_size(array_schema_->type(name)) : 1;
+              elements_mode_ ? datatype_size(array_schema_.type(name)) : 1;
           *it.second.buffer_var_size_ =
               ((uint32_t*)it.second.buffer_)[cells_copied] * offset_div;
         }
       }
     } else {
       // Always adjust the size for fixed size attributes.
-      auto cell_size = array_schema_->cell_size(name);
+      auto cell_size = array_schema_.cell_size(name);
       *(it.second.buffer_size_) = cells_copied * cell_size;
     }
 
@@ -625,7 +667,12 @@ Status SparseIndexReaderBase::resize_output_buffers(uint64_t cells_copied) {
 Status SparseIndexReaderBase::add_extra_offset() {
   for (const auto& it : buffers_) {
     const auto& name = it.first;
-    if (!array_schema_->var_size(name))
+    if (!array_schema_.var_size(name))
+      continue;
+
+    // Do not apply offset for empty results because we will
+    // write backwards and corrupt memory we don't own.
+    if (*it.second.buffer_size_ == 0)
       continue;
 
     auto buffer = static_cast<unsigned char*>(it.second.buffer_);
@@ -635,14 +682,14 @@ Status SparseIndexReaderBase::add_extra_offset() {
           it.second.buffer_var_size_,
           offsets_bytesize());
     } else if (offsets_format_mode_ == "elements") {
-      auto elements = *it.second.buffer_var_size_ /
-                      datatype_size(array_schema_->type(name));
+      auto elements =
+          *it.second.buffer_var_size_ / datatype_size(array_schema_.type(name));
       memcpy(
           buffer + *it.second.buffer_size_ - offsets_bytesize(),
           &elements,
           offsets_bytesize());
     } else {
-      return logger_->status(Status::ReaderError(
+      return logger_->status(Status_ReaderError(
           "Cannot add extra offset to buffer; Unsupported offsets format"));
     }
   }
@@ -659,18 +706,20 @@ void SparseIndexReaderBase::remove_result_tile_range(uint64_t f) {
 }
 
 // Explicit template instantiations
-template Status SparseIndexReaderBase::get_coord_tiles_size<uint64_t>(
-    bool, unsigned, unsigned, uint64_t, uint64_t*, uint64_t*);
-template Status SparseIndexReaderBase::get_coord_tiles_size<uint8_t>(
-    bool, unsigned, unsigned, uint64_t, uint64_t*, uint64_t*);
+template tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+SparseIndexReaderBase::get_coord_tiles_size<uint64_t>(
+    bool, unsigned, unsigned, uint64_t);
+template tuple<Status, optional<std::pair<uint64_t, uint64_t>>>
+SparseIndexReaderBase::get_coord_tiles_size<uint8_t>(
+    bool, unsigned, unsigned, uint64_t);
 template Status SparseIndexReaderBase::apply_query_condition<uint64_t>(
-    std::vector<ResultTile*>*);
+    std::vector<ResultTile*>&);
 template Status SparseIndexReaderBase::apply_query_condition<uint8_t>(
-    std::vector<ResultTile*>*);
+    std::vector<ResultTile*>&);
 template Status SparseIndexReaderBase::compute_tile_bitmaps<uint64_t>(
-    std::vector<ResultTile*>*);
+    std::vector<ResultTile*>&);
 template Status SparseIndexReaderBase::compute_tile_bitmaps<uint8_t>(
-    std::vector<ResultTile*>*);
+    std::vector<ResultTile*>&);
 
 }  // namespace sm
 }  // namespace tiledb
