@@ -35,10 +35,9 @@
 
 #include <queue>
 #include "reader_base.h"
+#include "tiledb/common/common.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
-#include "tiledb/sm/misc/types.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/result_cell_slab.h"
 
@@ -47,7 +46,7 @@ namespace sm {
 
 class Array;
 class ArraySchema;
-class OpenArrayMemoryTracker;
+class MemoryTracker;
 class StorageManager;
 class Subarray;
 
@@ -59,12 +58,29 @@ class ResultTileWithBitmap : public ResultTile {
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
   ResultTileWithBitmap(
-      unsigned frag_idx, uint64_t tile_idx, const ArraySchema* array_schema)
+      unsigned frag_idx, uint64_t tile_idx, const ArraySchema& array_schema)
       : ResultTile(frag_idx, tile_idx, array_schema)
       , bitmap_result_num_(std::numeric_limits<uint64_t>::max())
-      , qc_processed_(false) {
+      , coords_loaded_(false) {
   }
 
+  /** Move constructor. */
+  ResultTileWithBitmap(ResultTileWithBitmap<BitmapType>&& other) noexcept {
+    // Swap with the argument
+    swap(other);
+  }
+
+  /** Move-assign operator. */
+  ResultTileWithBitmap<BitmapType>& operator=(
+      ResultTileWithBitmap<BitmapType>&& other) {
+    // Swap with the argument
+    swap(other);
+
+    return *this;
+  }
+
+  DISABLE_COPY_AND_COPY_ASSIGN(ResultTileWithBitmap);
+
   /* ********************************* */
   /*          PUBLIC METHODS           */
   /* ********************************* */
@@ -72,6 +88,11 @@ class ResultTileWithBitmap : public ResultTile {
   /**
    * Returns the number of cells that are before a certain cell index in the
    * bitmap.
+   *
+   * @param start_pos Starting cell position in the bitmap.
+   * @param end_pos End position in the bitmap.
+   *
+   * @return Result number between the positions.
    */
   uint64_t result_num_between_pos(uint64_t start_pos, uint64_t end_pos) const {
     if (bitmap_.size() == 0)
@@ -86,6 +107,11 @@ class ResultTileWithBitmap : public ResultTile {
 
   /**
    * Returns cell index from a number of cells inside of the bitmap.
+   *
+   * @param start_pos Starting cell position in the bitmap.
+   * @param result_num Number of results to advance.
+   *
+   * @return Cell position found, or maximum position.
    */
   uint64_t pos_with_given_result_sum(
       uint64_t start_pos, uint64_t result_num) const {
@@ -103,8 +129,16 @@ class ResultTileWithBitmap : public ResultTile {
       }
     }
 
-    assert(false);
-    return std::numeric_limits<uint64_t>::max();
+    return bitmap_.size() - 1;
+  }
+
+  /** Swaps the contents (all field values) of this tile with the given tile. */
+  void swap(ResultTileWithBitmap<BitmapType>& tile) {
+    ResultTile::swap(tile);
+    std::swap(bitmap_, tile.bitmap_);
+    std::swap(bitmap_result_num_, tile.bitmap_result_num_);
+    std::swap(coords_loaded_, tile.coords_loaded_);
+    std::swap(hilbert_values_, tile.hilbert_values_);
   }
 
   /* ********************************* */
@@ -118,7 +152,10 @@ class ResultTileWithBitmap : public ResultTile {
   uint64_t bitmap_result_num_;
 
   /** Was the query condition processed for this tile. */
-  bool qc_processed_;
+  bool coords_loaded_;
+
+  /** Hilbert values for this tile. */
+  std::vector<uint64_t> hilbert_values_;
 };
 
 /**
@@ -139,9 +176,6 @@ class SparseIndexReaderBase : public ReaderBase {
 
   /** The state for a read sparse global order query. */
   struct ReadState {
-    /** The result cell slabs currently in process. */
-    std::vector<ResultCellSlab> result_cell_slabs_;
-
     /** The tile index inside of each fragments. */
     std::vector<std::pair<uint64_t, uint64_t>> frag_tile_idx_;
 
@@ -156,7 +190,7 @@ class SparseIndexReaderBase : public ReaderBase {
   /** Constructor. */
   SparseIndexReaderBase(
       stats::Stats* stats,
-      tdb_shared_ptr<Logger> logger,
+      shared_ptr<Logger> logger,
       StorageManager* storage_manager,
       Array* array,
       Config& config,
@@ -172,18 +206,36 @@ class SparseIndexReaderBase : public ReaderBase {
   /*          PUBLIC METHODS           */
   /* ********************************* */
 
-  /** Returns the current read state. */
+  /**
+   * Returns the current read state.
+   *
+   * @return pointer to the read state.
+   */
   const ReadState* read_state() const;
 
-  /** Initializes the reader. */
+  /**
+   * Returns the current read state.
+   *
+   * @return pointer to the read state.
+   */
+  ReadState* read_state();
+
+  /**
+   * Initializes the reader.
+   *
+   * @return Status.
+   */
   Status init();
 
-  /** Resize the output buffers to the correct size after copying. */
+  /**
+   * Resize the output buffers to the correct size after copying.
+   *
+   * @param cells_copied Number of cells copied.
+   *
+   * @return Status.
+   */
   Status resize_output_buffers(uint64_t cells_copied);
 
-  /** Returns the current read state. */
-  ReadState* read_state();
-
  protected:
   /* ********************************* */
   /*       PROTECTED ATTRIBUTES        */
@@ -215,7 +267,7 @@ class SparseIndexReaderBase : public ReaderBase {
   std::mutex mem_budget_mtx_;
 
   /** Memory tracker object for the array. */
-  OpenArrayMemoryTracker* array_memory_tracker_;
+  MemoryTracker* array_memory_tracker_;
 
   /** Memory used for coordinates tiles. */
   uint64_t memory_used_for_coords_total_;
@@ -238,66 +290,137 @@ class SparseIndexReaderBase : public ReaderBase {
   /** How much of the memory budget is reserved for array data. */
   double memory_budget_ratio_array_data_;
 
-  /** Indicate if the coordinates are loaded for the result tiles. */
-  bool coords_loaded_;
-
   /** Are we in elements mode. */
   bool elements_mode_;
 
   /** Names of dim/attr loaded for query condition. */
   std::vector<std::string> qc_loaded_names_;
 
+  /* Are the users buffers full. */
+  bool buffers_full_;
+
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
 
-  /** Get the coordinate tiles size for a dimension. */
-  template <class BitmapType>
-  Status get_coord_tiles_size(
-      bool include_coords,
-      unsigned dim_num,
-      unsigned f,
-      uint64_t t,
-      uint64_t* tiles_size,
-      uint64_t* tiles_size_qc);
+  /**
+   * Return how many cells were copied to the users buffers so far.
+   *
+   * @param names Attribute/dimensions to compute for.
+   *
+   * @return Number of cells copied.
+   */
+  uint64_t cells_copied(const std::vector<std::string>& names);
 
-  /** Load tile offsets and result tile ranges. */
+  /**
+   * Get the coordinate tiles size for a dimension.
+   *
+   * @param include_coords Include coordinates or not in the calculation.
+   * @param dim_num Number of dimensions.
+   * @param f Fragment index.
+   * @param t Tile index.
+   *
+   * @return Status, tiles_size, tiles_size_qc.
+   */
+  template <class BitmapType>
+  tuple<Status, optional<std::pair<uint64_t, uint64_t>>> get_coord_tiles_size(
+      bool include_coords, unsigned dim_num, unsigned f, uint64_t t);
+
+  /**
+   * Load tile offsets and result tile ranges.
+   *
+   * @return Status.
+   */
   Status load_initial_data();
 
-  /** Read and unfilter coord tiles. */
+  /**
+   * Read and unfilter coord tiles.
+   *
+   * @param include_coords Include coordinates or not.
+   * @param result_tiles The result tiles to process.
+   *
+   * @return Status.
+   */
   Status read_and_unfilter_coords(
-      bool include_coords, const std::vector<ResultTile*>* result_tiles);
+      bool include_coords, const std::vector<ResultTile*>& result_tiles);
 
-  /** Allocate a tile bitmap if required for this tile. */
+  /**
+   * Allocate a tile bitmap if required for this tile.
+   *
+   * @param rt Result tile currently in process.
+   *
+   * @return Status.
+   */
   template <class BitmapType>
   Status allocate_tile_bitmap(ResultTileWithBitmap<BitmapType>* rt);
 
-  /** Compute tile bitmaps. */
+  /**
+   * Compute tile bitmaps.
+   *
+   * @param result_tiles Result tiles to process.
+   *
+   * @return Status.
+   * */
   template <class BitmapType>
-  Status compute_tile_bitmaps(std::vector<ResultTile*>* result_tiles);
+  Status compute_tile_bitmaps(std::vector<ResultTile*>& result_tiles);
 
-  /** Count the number of cells in a bitmap. */
+  /**
+   * Count the number of cells in a bitmap.
+   *
+   * @param rt Result tile currently in process.
+   *
+   * @return Status.
+   */
   template <class BitmapType>
   Status count_tile_bitmap_cells(ResultTileWithBitmap<BitmapType>* rt);
 
-  /** Apply query condition. */
+  /**
+   * Apply query condition.
+   *
+   * @param result_tiles Result tiles to process.
+   *
+   * @return Status.
+   */
   template <class BitmapType>
-  Status apply_query_condition(std::vector<ResultTile*>* result_tiles);
+  Status apply_query_condition(std::vector<ResultTile*>& result_tiles);
+
+  /**
+   * Read and unfilter as many attributes as can fit in the memory budget and
+   * return the names loaded in 'names_to_copy'. Also keep the 'buffer_idx'
+   * updated to keep track of progress.
+   *
+   * @param memory_budget Memory budget allowed for this operation.
+   * @param names Attribute/dimensions to compute for.
+   * @param mem_usage_per_attr Computed per attribute memory usage.
+   * @param buffer_idx Stores/return the current buffer index in process.
+   * @param result_tiles Result tiles to process.
+   *
+   * @return Status, index_to_copy.
+   */
+  tuple<Status, optional<std::vector<uint64_t>>> read_and_unfilter_attributes(
+      const uint64_t memory_budget,
+      const std::vector<std::string>& names,
+      const std::vector<uint64_t>& mem_usage_per_attr,
+      uint64_t* buffer_idx,
+      std::vector<ResultTile*>& result_tiles);
 
   /**
    * Adds an extra offset in the end of the offsets buffer indicating the
    * returned data size if an attribute is var-sized.
+   *
+   * @return Status.
    */
   Status add_extra_offset();
 
-  /** Remove a result tile range for a specific fragment */
+  /**
+   * Remove a result tile range for a specific fragment.
+   *
+   * @param f Fragment index.
+   */
   void remove_result_tile_range(uint64_t f);
-
-  /** Remove a result tile range for a specific range */
-  void remove_range_result_tile_range(uint64_t r);
 };
 
 }  // namespace sm
 }  // namespace tiledb
 
-#endif  // TILEDB_SPARSE_INDEX_READER_BASE_H
\ No newline at end of file
+#endif  // TILEDB_SPARSE_INDEX_READER_BASE_H
