@@ -1,5 +1,5 @@
 /**
- * @file   query.cc
+ * @file query.cc
  *
  * @section LICENSE
  *
@@ -36,10 +36,14 @@
 #include <capnp/compat/json.h>
 #include <capnp/message.h>
 #include <capnp/serialize.h>
+#include "tiledb/sm/serialization/array.h"
+#include "tiledb/sm/serialization/array_schema.h"
 #include "tiledb/sm/serialization/capnp_utils.h"
 #endif
 // clang-format on
 
+#include "tiledb/sm/query/query.h"
+#include "tiledb/common/common.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/array/array.h"
@@ -51,13 +55,13 @@
 #include "tiledb/sm/enums/query_type.h"
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
-#include "tiledb/sm/misc/utils.h"
-#include "tiledb/sm/query/query.h"
-#include "tiledb/sm/query/reader.h"
+#include "tiledb/sm/misc/hash.h"
+#include "tiledb/sm/misc/parse_argument.h"
 #include "tiledb/sm/query/dense_reader.h"
+#include "tiledb/sm/query/reader.h"
 #include "tiledb/sm/query/sparse_global_order_reader.h"
 #include "tiledb/sm/query/sparse_unordered_with_dups_reader.h"
-#include "tiledb/sm/query/writer.h"
+#include "tiledb/sm/query/writer_base.h"
 #include "tiledb/sm/serialization/config.h"
 #include "tiledb/sm/serialization/query.h"
 #include "tiledb/sm/subarray/subarray.h"
@@ -74,7 +78,7 @@ namespace serialization {
 
 enum class SerializationContext { CLIENT, SERVER, BACKUP };
 
-tdb_shared_ptr<Logger> dummy_logger = tdb_make_shared(Logger, "");
+shared_ptr<Logger> dummy_logger = make_shared<Logger>(HERE(), "");
 
 Status stats_to_capnp(Stats& stats, capnp::Stats::Builder* stats_builder) {
   // Build counters
@@ -118,7 +122,7 @@ Status stats_from_capnp(
 
   if (stats_reader.hasTimers()) {
     auto timers = stats->timers();
-    auto timers_reader = stats_reader.getCounters();
+    auto timers_reader = stats_reader.getTimers();
     for (const auto entry : timers_reader.getEntries()) {
       (*timers)[std::string(entry.getKey().cStr())] = entry.getValue();
     }
@@ -127,38 +131,8 @@ Status stats_from_capnp(
   return Status::Ok();
 }
 
-Status array_to_capnp(
-    const Array& array, capnp::Array::Builder* array_builder) {
-  // The serialized URI is set if it exists
-  // this is used for backwards compatibility with pre TileDB 2.5 clients that
-  // want to serialized a query object TileDB >= 2.5 no longer needs to send the
-  // array URI
-  if (!array.array_uri_serialized().to_string().empty()) {
-    array_builder->setUri(array.array_uri_serialized());
-  }
-  array_builder->setStartTimestamp(array.timestamp_start());
-  array_builder->setEndTimestamp(array.timestamp_end());
-
-  return Status::Ok();
-}
-
-Status array_from_capnp(
-    const capnp::Array::Reader& array_reader, Array* array) {
-  // The serialized URI is set if it exists
-  // this is used for backwards compatibility with pre TileDB 2.5 clients that
-  // want to serialized a query object TileDB >= 2.5 no longer needs to receive
-  // the array URI
-  if (array_reader.hasUri()) {
-    RETURN_NOT_OK(array->set_uri_serialized(array_reader.getUri().cStr()));
-  }
-  RETURN_NOT_OK(array->set_timestamp_start(array_reader.getStartTimestamp()));
-  RETURN_NOT_OK(array->set_timestamp_end(array_reader.getEndTimestamp()));
-
-  return Status::Ok();
-}
-
 Status subarray_to_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const Subarray* subarray,
     capnp::Subarray::Builder* builder) {
   builder->setLayout(layout_str(subarray->layout()));
@@ -166,7 +140,7 @@ Status subarray_to_capnp(
   const uint32_t dim_num = subarray->dim_num();
   auto ranges_builder = builder->initRanges(dim_num);
   for (uint32_t i = 0; i < dim_num; i++) {
-    const auto datatype = schema->dimension(i)->type();
+    const auto datatype = schema.dimension(i)->type();
     auto range_builder = ranges_builder[i];
     const auto& ranges = subarray->ranges_for_dim(i);
     range_builder.setType(datatype_str(datatype));
@@ -271,13 +245,13 @@ Status subarray_from_capnp(
 }
 
 Status subarray_partitioner_to_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const SubarrayPartitioner& partitioner,
     capnp::SubarrayPartitioner::Builder* builder) {
   // Subarray
   auto subarray_builder = builder->initSubarray();
   RETURN_NOT_OK(
-      subarray_to_capnp(schema, partitioner.subarray(), &subarray_builder));
+      subarray_to_capnp(schema, &partitioner.subarray(), &subarray_builder));
 
   // Per-attr/dim mem budgets
   const auto* budgets = partitioner.get_result_budgets();
@@ -288,7 +262,7 @@ Status subarray_partitioner_to_capnp(
       const std::string& name = pair.first;
       auto budget_builder = mem_budgets_builder[idx];
       budget_builder.setAttribute(name);
-      auto var_size = schema->var_size(name);
+      auto var_size = schema.var_size(name);
 
       if (name == constants::coords || !var_size) {
         budget_builder.setOffsetBytes(0);
@@ -358,6 +332,7 @@ Status subarray_partitioner_to_capnp(
 }
 
 Status subarray_partitioner_from_capnp(
+    Stats* query_stats,
     Stats* reader_stats,
     const Config* config,
     const Array* array,
@@ -380,7 +355,7 @@ Status subarray_partitioner_from_capnp(
   RETURN_NOT_OK(layout_enum(subarray_reader.getLayout(), &layout));
 
   // Subarray, which is used to initialize the partitioner.
-  Subarray subarray(array, layout, reader_stats, dummy_logger, false);
+  Subarray subarray(array, layout, query_stats, dummy_logger, false);
   RETURN_NOT_OK(subarray_from_capnp(reader.getSubarray(), &subarray));
   *partitioner = SubarrayPartitioner(
       config,
@@ -394,14 +369,14 @@ Status subarray_partitioner_from_capnp(
 
   // Per-attr mem budgets
   if (reader.hasBudget()) {
-    const ArraySchema* schema = array->array_schema_latest();
+    const auto& schema = array->array_schema_latest();
     auto mem_budgets_reader = reader.getBudget();
     auto num_attrs = mem_budgets_reader.size();
     for (size_t i = 0; i < num_attrs; i++) {
       auto mem_budget_reader = mem_budgets_reader[i];
       std::string attr_name = mem_budget_reader.getAttribute();
-      auto var_size = schema->var_size(attr_name);
-      auto nullable = schema->is_nullable(attr_name);
+      auto var_size = schema.var_size(attr_name);
+      auto nullable = schema.is_nullable(attr_name);
 
       if (attr_name == constants::coords || !var_size) {
         if (nullable) {
@@ -439,7 +414,7 @@ Status subarray_partitioner_from_capnp(
     partition_info->split_multi_range_ =
         partition_info_reader.getSplitMultiRange();
     partition_info->partition_ =
-        Subarray(array, layout, reader_stats, dummy_logger, false);
+        Subarray(array, layout, query_stats, dummy_logger, false);
     RETURN_NOT_OK(subarray_from_capnp(
         partition_info_reader.getSubarray(), &partition_info->partition_));
 
@@ -463,7 +438,7 @@ Status subarray_partitioner_from_capnp(
   for (unsigned i = 0; i < num_sr; i++) {
     auto subarray_reader_ = sr_reader[i];
     state->single_range_.emplace_back(
-        array, layout, reader_stats, dummy_logger, false);
+        array, layout, query_stats, dummy_logger, false);
     Subarray& subarray_ = state->single_range_.back();
     RETURN_NOT_OK(subarray_from_capnp(subarray_reader_, &subarray_));
   }
@@ -472,7 +447,7 @@ Status subarray_partitioner_from_capnp(
   for (unsigned i = 0; i < num_m; i++) {
     auto subarray_reader_ = m_reader[i];
     state->multi_range_.emplace_back(
-        array, layout, reader_stats, dummy_logger, false);
+        array, layout, query_stats, dummy_logger, false);
     Subarray& subarray_ = state->multi_range_.back();
     RETURN_NOT_OK(subarray_from_capnp(subarray_reader_, &subarray_));
   }
@@ -496,7 +471,7 @@ Status subarray_partitioner_from_capnp(
 }
 
 Status read_state_to_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const Reader& reader,
     capnp::QueryReader::Builder* builder) {
   auto read_state = reader.read_state();
@@ -522,17 +497,6 @@ Status index_read_state_to_capnp(
   read_state_builder.setDoneAddingResultTiles(
       read_state->done_adding_result_tiles_);
 
-  auto rcs_builder = read_state_builder.initResultCellSlab(
-      read_state->result_cell_slabs_.size());
-  for (size_t i = 0; i < read_state->result_cell_slabs_.size(); ++i) {
-    rcs_builder[i].setFragIdx(
-        read_state->result_cell_slabs_[i].tile_->frag_idx());
-    rcs_builder[i].setTileIdx(
-        read_state->result_cell_slabs_[i].tile_->tile_idx());
-    rcs_builder[i].setStart(read_state->result_cell_slabs_[i].start_);
-    rcs_builder[i].setLength(read_state->result_cell_slabs_[i].length_);
-  }
-
   auto frag_tile_idx_builder =
       read_state_builder.initFragTileIdx(read_state->frag_tile_idx_.size());
   for (size_t i = 0; i < read_state->frag_tile_idx_.size(); ++i) {
@@ -544,7 +508,7 @@ Status index_read_state_to_capnp(
 }
 
 Status dense_read_state_to_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const DenseReader& reader,
     capnp::QueryReader::Builder* builder) {
   auto read_state = reader.read_state();
@@ -577,6 +541,7 @@ Status read_state_from_capnp(
   // Subarray partitioner
   if (read_state_reader.hasSubarrayPartitioner()) {
     RETURN_NOT_OK(subarray_partitioner_from_capnp(
+        query->stats(),
         reader->stats(),
         query->config(),
         array,
@@ -593,7 +558,7 @@ Status read_state_from_capnp(
 }
 
 Status index_read_state_from_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const capnp::ReadStateIndex::Reader& read_state_reader,
     SparseIndexReaderBase* reader) {
   auto read_state = reader->read_state();
@@ -601,40 +566,6 @@ Status index_read_state_from_capnp(
   read_state->done_adding_result_tiles_ =
       read_state_reader.getDoneAddingResultTiles();
 
-  // Only sparse global order reader has a result cell slab.
-  if (read_state_reader.hasResultCellSlab()) {
-    SparseGlobalOrderReader* sparse_global_order_reader =
-        (SparseGlobalOrderReader*)reader;
-    RETURN_NOT_OK(sparse_global_order_reader->clear_result_tiles());
-    read_state->result_cell_slabs_.clear();
-
-    std::unordered_map<
-        std::pair<unsigned, uint64_t>,
-        ResultTile*,
-        tiledb::sm::utils::hash::pair_hash>
-        result_tile_map;
-    for (const auto rcs : read_state_reader.getResultCellSlab()) {
-      auto start = rcs.getStart();
-      auto length = rcs.getLength();
-      auto frag_idx = rcs.getFragIdx();
-      auto tile_idx = rcs.getTileIdx();
-
-      ResultTile* rt = nullptr;
-      auto it = result_tile_map.find(
-          std::pair<unsigned, uint64_t>(frag_idx, tile_idx));
-      if (it != result_tile_map.end()) {
-        rt = it->second;
-      } else {
-        rt = sparse_global_order_reader->add_result_tile_unsafe(
-            frag_idx, tile_idx, schema);
-        result_tile_map.emplace(
-            std::pair<unsigned, uint64_t>(frag_idx, tile_idx), rt);
-      }
-
-      read_state->result_cell_slabs_.emplace_back(rt, start, length);
-    }
-  }
-
   assert(read_state_reader.hasFragTileIdx());
   read_state->frag_tile_idx_.clear();
   for (const auto rcs : read_state_reader.getFragTileIdx()) {
@@ -662,6 +593,7 @@ Status dense_read_state_from_capnp(
   // Subarray partitioner
   if (read_state_reader.hasSubarrayPartitioner()) {
     RETURN_NOT_OK(subarray_partitioner_from_capnp(
+        query->stats(),
         reader->stats(),
         query->config(),
         array,
@@ -721,7 +653,7 @@ Status reader_to_capnp(
     const Query& query,
     const Reader& reader,
     capnp::QueryReader::Builder* reader_builder) {
-  auto array_schema = query.array_schema();
+  const auto& array_schema = query.array_schema();
 
   // Subarray layout
   const auto& layout = layout_str(query.layout());
@@ -755,7 +687,7 @@ Status index_reader_to_capnp(
     const Query& query,
     const SparseIndexReaderBase& reader,
     capnp::ReaderIndex::Builder* reader_builder) {
-  auto array_schema = query.array_schema();
+  const auto& array_schema = query.array_schema();
 
   // Subarray layout
   const auto& layout = layout_str(query.layout());
@@ -789,7 +721,7 @@ Status dense_reader_to_capnp(
     const Query& query,
     const DenseReader& reader,
     capnp::QueryReader::Builder* reader_builder) {
-  auto array_schema = query.array_schema();
+  const auto& array_schema = query.array_schema();
 
   // Subarray layout
   const auto& layout = layout_str(query.layout());
@@ -872,7 +804,7 @@ Status reader_from_capnp(
   RETURN_NOT_OK(query->set_layout_unsafe(layout));
 
   // Subarray
-  Subarray subarray(array, layout, reader->stats(), dummy_logger, false);
+  Subarray subarray(array, layout, query->stats(), dummy_logger, false);
   auto subarray_reader = reader_reader.getSubarray();
   RETURN_NOT_OK(subarray_from_capnp(subarray_reader, &subarray));
   RETURN_NOT_OK(query->set_subarray_unsafe(subarray));
@@ -903,7 +835,7 @@ Status reader_from_capnp(
 }
 
 Status index_reader_from_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const capnp::ReaderIndex::Reader& reader_reader,
     Query* query,
     SparseIndexReaderBase* reader) {
@@ -914,7 +846,7 @@ Status index_reader_from_capnp(
   RETURN_NOT_OK(layout_enum(reader_reader.getLayout(), &layout));
 
   // Subarray
-  Subarray subarray(array, layout, reader->stats(), dummy_logger, false);
+  Subarray subarray(array, layout, query->stats(), dummy_logger, false);
   auto subarray_reader = reader_reader.getSubarray();
   RETURN_NOT_OK(subarray_from_capnp(subarray_reader, &subarray));
   RETURN_NOT_OK(query->set_subarray_unsafe(subarray));
@@ -945,7 +877,7 @@ Status index_reader_from_capnp(
 }
 
 Status dense_reader_from_capnp(
-    const ArraySchema* schema,
+    const ArraySchema& schema,
     const capnp::QueryReader::Reader& reader_reader,
     Query* query,
     DenseReader* reader,
@@ -957,7 +889,7 @@ Status dense_reader_from_capnp(
   RETURN_NOT_OK(layout_enum(reader_reader.getLayout(), &layout));
 
   // Subarray
-  Subarray subarray(array, layout, reader->stats(), dummy_logger, false);
+  Subarray subarray(array, layout, query->stats(), dummy_logger, false);
   auto subarray_reader = reader_reader.getSubarray();
   RETURN_NOT_OK(subarray_from_capnp(subarray_reader, &subarray));
   RETURN_NOT_OK(query->set_subarray_unsafe(subarray));
@@ -989,15 +921,15 @@ Status dense_reader_from_capnp(
 
 Status writer_to_capnp(
     const Query& query,
-    Writer& writer,
+    WriterBase& writer,
     capnp::Writer::Builder* writer_builder) {
   writer_builder->setCheckCoordDups(writer.get_check_coord_dups());
   writer_builder->setCheckCoordOOB(writer.get_check_coord_oob());
   writer_builder->setDedupCoords(writer.get_dedup_coords());
 
-  const auto* array_schema = query.array_schema();
+  const auto& array_schema = query.array_schema();
 
-  if (array_schema->dense()) {
+  if (array_schema.dense()) {
     std::vector<uint8_t> subarray_flat;
     RETURN_NOT_OK(query.subarray()->to_byte_vec(&subarray_flat));
     auto subarray_builder = writer_builder->initSubarray();
@@ -1024,7 +956,7 @@ Status writer_to_capnp(
 }
 
 Status writer_from_capnp(
-    const capnp::Writer::Reader& writer_reader, Writer* writer) {
+    const capnp::Writer::Reader& writer_reader, WriterBase* writer) {
   writer->set_check_coord_dups(writer_reader.getCheckCoordDups());
   writer->set_check_coord_oob(writer_reader.getCheckCoordOOB());
   writer->set_dedup_coords(writer_reader.getDedupCoords());
@@ -1041,7 +973,10 @@ Status writer_from_capnp(
   return Status::Ok();
 }
 
-Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
+Status query_to_capnp(
+    Query& query,
+    capnp::Query::Builder* query_builder,
+    const bool client_side) {
   // For easy reference
   auto layout = query.layout();
   auto type = query.type();
@@ -1049,22 +984,14 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
 
   if (layout == Layout::GLOBAL_ORDER && query.type() == QueryType::WRITE)
     return LOG_STATUS(
-        Status::SerializationError("Cannot serialize; global order "
-                                   "serialization not supported for writes."));
+        Status_SerializationError("Cannot serialize; global order "
+                                  "serialization not supported for writes."));
 
   if (array == nullptr)
     return LOG_STATUS(
-        Status::SerializationError("Cannot serialize; array is null."));
+        Status_SerializationError("Cannot serialize; array is null."));
 
-  const auto* schema = query.array_schema();
-  if (schema == nullptr)
-    return LOG_STATUS(
-        Status::SerializationError("Cannot serialize; array schema is null."));
-
-  const auto* domain = schema->domain();
-  if (domain == nullptr)
-    return LOG_STATUS(
-        Status::SerializationError("Cannot serialize; array domain is null."));
+  const auto& schema = query.array_schema();
 
   // Serialize basic fields
   query_builder->setType(query_type_str(type));
@@ -1074,7 +1001,7 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
   // Serialize array
   if (query.array() != nullptr) {
     auto builder = query_builder->initArray();
-    RETURN_NOT_OK(array_to_capnp(*array, &builder));
+    RETURN_NOT_OK(array_to_capnp(array, &builder, client_side));
   }
 
   // Serialize attribute buffer metadata
@@ -1135,14 +1062,14 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
     for (auto& frag_md : array->fragment_metadata())
       all_dense &= frag_md->dense();
     if (query.use_refactored_sparse_unordered_with_dups_reader() &&
-        !schema->dense() && layout == Layout::UNORDERED &&
-        schema->allows_dups()) {
+        !schema.dense() && layout == Layout::UNORDERED &&
+        schema.allows_dups()) {
       auto builder = query_builder->initReaderIndex();
 
       auto&& [st, non_overlapping_ranges]{query.non_overlapping_ranges()};
       RETURN_NOT_OK(st);
 
-      if (non_overlapping_ranges) {
+      if (*non_overlapping_ranges) {
         auto reader = (SparseUnorderedWithDupsReader<uint8_t>*)query.strategy();
 
         query_builder->setVarOffsetsMode(reader->offsets_mode());
@@ -1161,7 +1088,7 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
         RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
       }
     } else if (
-        query.use_refactored_sparse_global_order_reader() && !schema->dense() &&
+        query.use_refactored_sparse_global_order_reader() && !schema.dense() &&
         (layout == Layout::GLOBAL_ORDER ||
          (layout == Layout::UNORDERED && query.subarray()->range_num() <= 1))) {
       auto builder = query_builder->initReaderIndex();
@@ -1173,7 +1100,7 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
       query_builder->setVarOffsetsBitsize(reader->offsets_bitsize());
       RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
     } else if (
-        query.use_refactored_dense_reader() && all_dense && schema->dense()) {
+        query.use_refactored_dense_reader() && all_dense && schema.dense()) {
       auto builder = query_builder->initDenseReader();
       auto reader = (DenseReader*)query.strategy();
 
@@ -1194,7 +1121,7 @@ Status query_to_capnp(Query& query, capnp::Query::Builder* query_builder) {
     }
   } else {
     auto builder = query_builder->initWriter();
-    auto writer = (Writer*)query.strategy();
+    auto writer = (WriterBase*)query.strategy();
 
     query_builder->setVarOffsetsMode(writer->offsets_mode());
     query_builder->setVarOffsetsAddExtraElement(
@@ -1229,26 +1156,17 @@ Status query_from_capnp(
 
   auto type = query->type();
   auto array = query->array();
-
-  const auto* schema = query->array_schema();
-  if (schema == nullptr)
-    return LOG_STATUS(Status::SerializationError(
-        "Cannot deserialize; array schema is null."));
-
-  const auto* domain = schema->domain();
-  if (domain == nullptr)
-    return LOG_STATUS(Status::SerializationError(
-        "Cannot deserialize; array domain is null."));
+  const auto& schema = query->array_schema();
 
   if (array == nullptr)
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; array pointer is null."));
 
   // Deserialize query type (sanity check).
   QueryType query_type = QueryType::READ;
   RETURN_NOT_OK(query_type_enum(query_reader.getType().cStr(), &query_type));
   if (query_type != type)
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; Query opened for " + query_type_str(type) +
         " but got serialized type for " + query_reader.getType().cStr()));
 
@@ -1272,7 +1190,7 @@ Status query_from_capnp(
 
   // Deserialize and set attribute buffers.
   if (!query_reader.hasAttributeBufferHeaders())
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; no attribute buffer headers in message."));
 
   auto buffer_headers = query_reader.getAttributeBufferHeaders();
@@ -1305,8 +1223,8 @@ Status query_from_capnp(
     uint64_t* existing_offset_buffer_size_ptr = nullptr;
     uint64_t* existing_validity_buffer_size_ptr = nullptr;
 
-    auto var_size = schema->var_size(name);
-    auto nullable = schema->is_nullable(name);
+    auto var_size = schema.var_size(name);
+    auto nullable = schema.is_nullable(name);
     if (type == QueryType::READ && context == SerializationContext::SERVER) {
       const QueryBuffer& query_buffer = query->buffer(name);
       // We use the query_buffer directly in order to get the original buffer
@@ -1424,7 +1342,7 @@ Status query_from_capnp(
           (var_size && offset_size_left >= fixedlen_size) || !var_size;
       const bool has_mem_for_validity = validity_size_left >= validitylen_size;
       if (!has_mem_for_data || !has_mem_for_offset || !has_mem_for_validity) {
-        return LOG_STATUS(Status::SerializationError(
+        return LOG_STATUS(Status_SerializationError(
             "Error deserializing read query; buffer too small for buffer "
             "'" +
             name + "'."));
@@ -1544,7 +1462,7 @@ Status query_from_capnp(
       // Always expect null buffers when deserializing.
       if (existing_buffer != nullptr || existing_offset_buffer != nullptr ||
           existing_validity_buffer != nullptr)
-        return LOG_STATUS(Status::SerializationError(
+        return LOG_STATUS(Status_SerializationError(
             "Error deserializing read query; unexpected "
             "buffer set on server-side."));
 
@@ -1700,7 +1618,7 @@ Status query_from_capnp(
   // on the reader or writer directly Now we set it on the query class after the
   // heterogeneous coordinate changes
   if (type == QueryType::READ) {
-    if (query_reader.hasReaderIndex() && !schema->dense() &&
+    if (query_reader.hasReaderIndex() && !schema.dense() &&
         layout == Layout::GLOBAL_ORDER) {
       // Strategy needs to be cleared here to create the correct reader.
       query->clear_strategy();
@@ -1727,8 +1645,8 @@ Status query_from_capnp(
       RETURN_NOT_OK(
           index_reader_from_capnp(schema, reader_reader, query, reader));
     } else if (
-        query_reader.hasReaderIndex() && !schema->dense() &&
-        layout == Layout::UNORDERED && schema->allows_dups()) {
+        query_reader.hasReaderIndex() && !schema.dense() &&
+        layout == Layout::UNORDERED && schema.allows_dups()) {
       // Strategy needs to be cleared here to create the correct reader.
       query->clear_strategy();
       RETURN_NOT_OK(query->set_layout_unsafe(layout));
@@ -1738,7 +1656,7 @@ Status query_from_capnp(
 
       auto reader_reader = query_reader.getReaderIndex();
 
-      if (non_overlapping_ranges) {
+      if (*non_overlapping_ranges) {
         auto reader =
             (SparseUnorderedWithDupsReader<uint8_t>*)query->strategy();
 
@@ -1826,7 +1744,7 @@ Status query_from_capnp(
     }
   } else {
     auto writer_reader = query_reader.getWriter();
-    auto writer = (Writer*)query->strategy();
+    auto writer = (WriterBase*)query->strategy();
 
     if (query_reader.hasVarOffsetsMode()) {
       RETURN_NOT_OK(writer->set_offsets_mode(query_reader.getVarOffsetsMode()));
@@ -1844,7 +1762,7 @@ Status query_from_capnp(
 
     // For sparse writes we want to explicitly set subarray to nullptr.
     const bool sparse_write =
-        !schema->dense() || query->layout() == Layout::UNORDERED;
+        !schema.dense() || query->layout() == Layout::UNORDERED;
     if (!sparse_write) {
       if (writer_reader.hasSubarray()) {
         auto subarray_reader = writer_reader.getSubarray();
@@ -1857,7 +1775,7 @@ Status query_from_capnp(
 
       // Subarray
       if (writer_reader.hasSubarrayRanges()) {
-        Subarray subarray(array, layout, writer->stats(), dummy_logger, false);
+        Subarray subarray(array, layout, query->stats(), dummy_logger, false);
         auto subarray_reader = writer_reader.getSubarrayRanges();
         RETURN_NOT_OK(subarray_from_capnp(subarray_reader, &subarray));
         RETURN_NOT_OK(query->set_subarray_unsafe(subarray));
@@ -1890,18 +1808,13 @@ Status query_serialize(
     bool clientside,
     BufferList* serialized_buffer) {
   if (serialize_type == SerializationType::JSON)
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot serialize query; json format not supported."));
 
-  const auto* array_schema = query->array_schema();
-  if (array_schema == nullptr || query->array() == nullptr)
-    return LOG_STATUS(Status::SerializationError(
-        "Cannot serialize; array or array schema is null."));
-
   try {
     ::capnp::MallocMessageBuilder message;
     capnp::Query::Builder query_builder = message.initRoot<capnp::Query>();
-    RETURN_NOT_OK(query_to_capnp(*query, &query_builder));
+    RETURN_NOT_OK(query_to_capnp(*query, &query_builder, clientside));
 
     // Determine whether we should be serializing the buffer data.
     const bool serialize_buffers =
@@ -1970,15 +1883,15 @@ Status query_serialize(
         break;
       }
       default:
-        return LOG_STATUS(Status::SerializationError(
+        return LOG_STATUS(Status_SerializationError(
             "Cannot serialize; unknown serialization type"));
     }
   } catch (kj::Exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot serialize; kj::Exception: " +
         std::string(e.getDescription().cStr())));
   } catch (std::exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot serialize; exception: " + std::string(e.what())));
   }
 
@@ -1993,7 +1906,7 @@ Status do_query_deserialize(
     Query* query,
     ThreadPool* compute_tp) {
   if (serialize_type == SerializationType::JSON)
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize query; json format not supported."));
 
   try {
@@ -2014,7 +1927,7 @@ Status do_query_deserialize(
       case SerializationType::CAPNP: {
         // Capnp FlatArrayMessageReader requires 64-bit alignment.
         if (!utils::is_aligned<sizeof(uint64_t)>(serialized_buffer.cur_data()))
-          return LOG_STATUS(Status::SerializationError(
+          return LOG_STATUS(Status_SerializationError(
               "Could not deserialize query; buffer is not 8-byte aligned."));
 
         // Set traversal limit to 10GI (TODO: make this a config option)
@@ -2038,15 +1951,15 @@ Status do_query_deserialize(
             query_reader, context, buffer_start, copy_state, query, compute_tp);
       }
       default:
-        return LOG_STATUS(Status::SerializationError(
+        return LOG_STATUS(Status_SerializationError(
             "Cannot deserialize; unknown serialization type."));
     }
   } catch (kj::Exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; kj::Exception: " +
         std::string(e.getDescription().cStr())));
   } catch (std::exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; exception: " + std::string(e.what())));
   }
   return Status::Ok();
@@ -2104,7 +2017,8 @@ Status query_deserialize(
         query,
         compute_tp);
     if (!st2.ok()) {
-      LOG_FATAL(st2.message());
+      LOG_ERROR(st2.message());
+      return st2;
     }
   }
 
@@ -2222,15 +2136,15 @@ Status query_est_result_size_serialize(
         break;
       }
       default:
-        return LOG_STATUS(Status::SerializationError(
+        return LOG_STATUS(Status_SerializationError(
             "Cannot serialize; unknown serialization type"));
     }
   } catch (kj::Exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot serialize; kj::Exception: " +
         std::string(e.getDescription().cStr())));
   } catch (std::exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Cannot serialize; exception: " + std::string(e.what())));
   }
 
@@ -2273,17 +2187,17 @@ Status query_est_result_size_deserialize(
       }
       default: {
         return LOG_STATUS(
-            Status::SerializationError("Error deserializing query est result "
-                                       "size; Unknown serialization type "
-                                       "passed"));
+            Status_SerializationError("Error deserializing query est result "
+                                      "size; Unknown serialization type "
+                                      "passed"));
       }
     }
   } catch (kj::Exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Error deserializing query est result size; kj::Exception: " +
         std::string(e.getDescription().cStr())));
   } catch (std::exception& e) {
-    return LOG_STATUS(Status::SerializationError(
+    return LOG_STATUS(Status_SerializationError(
         "Error deserializing query est result size; exception " +
         std::string(e.what())));
   }
@@ -2294,25 +2208,25 @@ Status query_est_result_size_deserialize(
 #else
 
 Status query_serialize(Query*, SerializationType, bool, BufferList*) {
-  return LOG_STATUS(Status::SerializationError(
+  return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
 Status query_deserialize(
     const Buffer&, SerializationType, bool, CopyState*, Query*, ThreadPool*) {
-  return LOG_STATUS(Status::SerializationError(
+  return LOG_STATUS(Status_SerializationError(
       "Cannot deserialize; serialization not enabled."));
 }
 
 Status query_est_result_size_serialize(
     Query*, SerializationType, bool, Buffer*) {
-  return LOG_STATUS(Status::SerializationError(
+  return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
 Status query_est_result_size_deserialize(
     Query*, SerializationType, bool, const Buffer&) {
-  return LOG_STATUS(Status::SerializationError(
+  return LOG_STATUS(Status_SerializationError(
       "Cannot deserialize; serialization not enabled."));
 }
 
